@@ -40,8 +40,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/identify_client.hpp"
-
-#include <boost/lexical_cast.hpp>
+#include "libtorrent/stat.hpp"
 
 namespace libtorrent
 {
@@ -97,6 +96,58 @@ namespace libtorrent
 		std::string url;
 	};
 
+	struct TORRENT_EXPORT read_piece_alert: torrent_alert
+	{
+		read_piece_alert(torrent_handle const& h
+			, int p, boost::shared_array<char> d, int s)
+			: torrent_alert(h)
+			, buffer(d)
+			, piece(p)
+			, size(s)
+		{}
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new read_piece_alert(*this)); }
+		const static int static_category = alert::storage_notification;
+		virtual int category() const { return static_category; }
+		virtual char const* what() const { return "read piece"; }
+		virtual std::string message() const
+		{
+			char msg[200];
+			snprintf(msg, sizeof(msg), "%s: piece %s %u", torrent_alert::message().c_str()
+				, buffer ? "successful" : "failed", piece);
+			return msg;
+		}
+
+		boost::shared_array<char> buffer;
+		int piece;
+		int size;
+	};
+
+	struct TORRENT_EXPORT file_completed_alert: torrent_alert
+	{
+		file_completed_alert(torrent_handle const& h
+			, int index_)
+			: torrent_alert(h)
+			, index(index_)
+		{}
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new file_completed_alert(*this)); }
+		const static int static_category = alert::progress_notification;
+		virtual int category() const { return static_category; }
+		virtual char const* what() const { return "file completed"; }
+		virtual std::string message() const
+		{
+			char msg[200];
+			snprintf(msg, sizeof(msg), "%s: file %d finished downloading"
+				, torrent_alert::message().c_str(), index);
+			return msg;
+		}
+
+		int index;
+	};
+
 	struct TORRENT_EXPORT file_renamed_alert: torrent_alert
 	{
 		file_renamed_alert(torrent_handle const& h
@@ -114,10 +165,10 @@ namespace libtorrent
 		virtual char const* what() const { return "file renamed"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << torrent_alert::message() << ": file "
-				<< index << " renamed to " << name;
-			return ret.str();
+			char msg[200 + NAME_MAX];
+			snprintf(msg, sizeof(msg), "%s: file %d renamed to %s", torrent_alert::message().c_str()
+				, index, name.c_str());
+			return msg;
 		}
 
 		std::string name;
@@ -127,11 +178,11 @@ namespace libtorrent
 	struct TORRENT_EXPORT file_rename_failed_alert: torrent_alert
 	{
 		file_rename_failed_alert(torrent_handle const& h
-			, std::string const& msg_
-			, int index_)
+			, int index_
+			, error_code ec_)
 			: torrent_alert(h)
-			, msg(msg_)
 			, index(index_)
+			, error(ec_)
 		{}
 
 		virtual std::auto_ptr<alert> clone() const
@@ -140,17 +191,17 @@ namespace libtorrent
 		virtual char const* what() const { return "file rename failed"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << torrent_alert::message() << ": failed to rename file "
-				<< index << ": " << msg;
-			return ret.str();
+			char ret[200 + NAME_MAX];
+			snprintf(ret, sizeof(ret), "%s: failed to rename file %d: %s"
+				, torrent_alert::message().c_str(), index, error.message().c_str());
+			return ret;
 		}
 
 		const static int static_category = alert::storage_notification;
 		virtual int category() const { return static_category; }
 
-		std::string msg;
 		int index;
+		error_code error;
 	};
 
 	struct TORRENT_EXPORT performance_alert: torrent_alert
@@ -158,7 +209,12 @@ namespace libtorrent
 		enum performance_warning_t
 		{
 			outstanding_disk_buffer_limit_reached,
-			outstanding_request_limit_reached
+			outstanding_request_limit_reached,
+			upload_limit_too_low,
+			download_limit_too_low,
+			send_buffer_watermark_too_low,
+
+			num_warnings
 		};
 
 		performance_alert(torrent_handle const& h
@@ -177,6 +233,9 @@ namespace libtorrent
 			{
 				"max outstanding disk writes reached",
 				"max outstanding piece requests reached",
+				"upload limit too low (download rate will suffer)",
+				"download limit too low (upload rate will suffer)",
+				"send buffer watermark too low (upload rate will suffer)"
 			};
 
 			return torrent_alert::message() + ": performance warning: "
@@ -192,9 +251,11 @@ namespace libtorrent
 	struct TORRENT_EXPORT state_changed_alert: torrent_alert
 	{
 		state_changed_alert(torrent_handle const& h
-			, torrent_status::state_t const& state_)
+			, torrent_status::state_t state_
+			, torrent_status::state_t prev_state_)
 			: torrent_alert(h)
 			, state(state_)
+			, prev_state(prev_state_)
 		{}
 
 		virtual std::auto_ptr<alert> clone() const
@@ -217,10 +278,24 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 
 		torrent_status::state_t state;
+		torrent_status::state_t prev_state;
 	};
 
 	struct TORRENT_EXPORT tracker_error_alert: tracker_alert
 	{
+		tracker_error_alert(torrent_handle const& h
+			, int times
+			, int status
+			, std::string const& url_
+			, error_code const& e)
+			: tracker_alert(h, url_)
+			, times_in_row(times)
+			, status_code(status)
+			, msg(e.message())
+		{
+			TORRENT_ASSERT(!url.empty());
+		}
+
 		tracker_error_alert(torrent_handle const& h
 			, int times
 			, int status
@@ -230,7 +305,9 @@ namespace libtorrent
 			, times_in_row(times)
 			, status_code(status)
 			, msg(msg_)
-		{ TORRENT_ASSERT(!url.empty()); }
+		{
+			TORRENT_ASSERT(!url.empty());
+		}
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new tracker_error_alert(*this)); }
@@ -239,10 +316,11 @@ namespace libtorrent
 		virtual char const* what() const { return "tracker error"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << tracker_alert::message() << " (" << status_code << ") "
-				<< msg << " (" << times_in_row << ")";
-			return ret.str();
+			char ret[400];
+			snprintf(ret, sizeof(ret), "%s (%d) %s (%d)"
+				, tracker_alert::message().c_str(), status_code
+				, msg.c_str(), times_in_row);
+			return ret;
 		}
 
 		int times_in_row;
@@ -291,15 +369,22 @@ namespace libtorrent
 		virtual char const* what() const { return "tracker scrape reply"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << tracker_alert::message() << " scrape reply: " << incomplete
-				<< " " << complete;
-			return ret.str();
+			char ret[400];
+			snprintf(ret, sizeof(ret), "%s scrape reply: %u %u"
+				, tracker_alert::message().c_str(), incomplete, complete);
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT scrape_failed_alert: tracker_alert
 	{
+		scrape_failed_alert(torrent_handle const& h
+			, std::string const& url_
+			, error_code const& e)
+			: tracker_alert(h, url_)
+			, msg(e.message())
+		{ TORRENT_ASSERT(!url.empty()); }
+
 		scrape_failed_alert(torrent_handle const& h
 			, std::string const& url_
 			, std::string const& msg_)
@@ -336,10 +421,10 @@ namespace libtorrent
 		virtual char const* what() const { return "tracker reply"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << tracker_alert::message() << " received peers: "
-				<< num_peers;
-			return ret.str();
+			char ret[400];
+			snprintf(ret, sizeof(ret), "%s received peers: %u"
+				, tracker_alert::message().c_str(), num_peers);
+			return ret;
 		}
 	};
 
@@ -358,10 +443,10 @@ namespace libtorrent
 		virtual char const* what() const { return "DHT reply"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << tracker_alert::message() << " received DHT peers: "
-				<< num_peers;
-			return ret.str();
+			char ret[400];
+			snprintf(ret, sizeof(ret), "%s received DHT peers: %u"
+				, tracker_alert::message().c_str(), num_peers);
+			return ret;
 		}
 	};
 
@@ -401,10 +486,10 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << torrent_alert::message() << " hash for piece "
-				<< piece_index << " failed";
-			return ret.str();
+			char ret[400];
+			snprintf(ret, sizeof(ret), "%s hash for piece %u failed"
+				, torrent_alert::message().c_str(), piece_index);
+			return ret;
 		}
 
 		int piece_index;
@@ -412,9 +497,9 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT peer_ban_alert: peer_alert
 	{
-		peer_ban_alert(torrent_handle h, tcp::endpoint const& ip
-			, peer_id const& pid)
-			: peer_alert(h, ip, pid)
+		peer_ban_alert(torrent_handle h, tcp::endpoint const& ep
+			, peer_id const& peer_id)
+			: peer_alert(h, ep, peer_id)
 		{}
 
 		virtual std::auto_ptr<alert> clone() const
@@ -429,9 +514,9 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT peer_unsnubbed_alert: peer_alert
 	{
-		peer_unsnubbed_alert(torrent_handle h, tcp::endpoint const& ip
-			, peer_id const& pid)
-			: peer_alert(h, ip, pid)
+		peer_unsnubbed_alert(torrent_handle h, tcp::endpoint const& ep
+			, peer_id const& peer_id)
+			: peer_alert(h, ep, peer_id)
 		{}
 
 		virtual std::auto_ptr<alert> clone() const
@@ -445,9 +530,9 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT peer_snubbed_alert: peer_alert
 	{
-		peer_snubbed_alert(torrent_handle h, tcp::endpoint const& ip
-			, peer_id const& pid)
-			: peer_alert(h, ip, pid)
+		peer_snubbed_alert(torrent_handle h, tcp::endpoint const& ep
+			, peer_id const& peer_id)
+			: peer_alert(h, ep, peer_id)
 		{}
 
 		virtual std::auto_ptr<alert> clone() const
@@ -461,11 +546,15 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT peer_error_alert: peer_alert
 	{
-		peer_error_alert(torrent_handle const& h, tcp::endpoint const& ip
-			, peer_id const& pid, std::string const& msg_)
-			: peer_alert(h, ip, pid)
-			, msg(msg_)
-		{}
+		peer_error_alert(torrent_handle const& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, error_code const& e)
+			: peer_alert(h, ep, peer_id)
+			, error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new peer_error_alert(*this)); }
@@ -475,17 +564,21 @@ namespace libtorrent
 		virtual std::string message() const
 		{
 			error_code ec;
-			return peer_alert::message() + " peer error: " + msg;
+			return peer_alert::message() + " peer error: " + error.message();
 		}
 
+		error_code error;
+
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 	};
 
 	struct TORRENT_EXPORT peer_connect_alert: peer_alert
 	{
-		peer_connect_alert(torrent_handle h, tcp::endpoint const& ip
-			, peer_id const& pid)
-			: peer_alert(h, ip, pid)
+		peer_connect_alert(torrent_handle h, tcp::endpoint const& ep
+			, peer_id const& peer_id)
+			: peer_alert(h, ep, peer_id)
 		{}
 
 		virtual std::auto_ptr<alert> clone() const
@@ -501,11 +594,15 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT peer_disconnected_alert: peer_alert
 	{
-		peer_disconnected_alert(torrent_handle const& h, tcp::endpoint const& ip
-			, peer_id const& pid, std::string const& msg_)
-			: peer_alert(h, ip, pid)
-			, msg(msg_)
-		{}
+		peer_disconnected_alert(torrent_handle const& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, error_code const& e)
+			: peer_alert(h, ep, peer_id)
+			, error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new peer_disconnected_alert(*this)); }
@@ -514,17 +611,21 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			return peer_alert::message() + " disconnecting: " + msg;
+			return peer_alert::message() + " disconnecting: " + error.message();
 		}
 
+		error_code error;
+
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 	};
 
 	struct TORRENT_EXPORT invalid_request_alert: peer_alert
 	{
-		invalid_request_alert(torrent_handle const& h, tcp::endpoint const& ip
-			, peer_id const& pid, peer_request const& r)
-			: peer_alert(h, ip, pid)
+		invalid_request_alert(torrent_handle const& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, peer_request const& r)
+			: peer_alert(h, ep, peer_id)
 			, request(r)
 		{}
 
@@ -533,11 +634,10 @@ namespace libtorrent
 		virtual char const* what() const { return "invalid piece request"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << peer_alert::message() << " peer sent an invalid piece request "
-				"( piece: " << request.piece << " start: " << request.start
-				<< " len: " << request.length << ")";
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s peer sent an invalid piece request (piece: %u start: %u len: %u)"
+				, torrent_alert::message().c_str(), request.piece, request.start, request.length);
+			return ret;
 		}
 
 		peer_request request;
@@ -579,18 +679,18 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << torrent_alert::message() << " piece " << piece_index
-				<< " finished downloading";
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s piece: %u finished downloading"
+				, torrent_alert::message().c_str(), piece_index);
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT request_dropped_alert: peer_alert
 	{
-		request_dropped_alert(const torrent_handle& h, tcp::endpoint const& ip
-			, peer_id const& pid, int block_num, int piece_num)
-			: peer_alert(h, ip, pid)
+		request_dropped_alert(const torrent_handle& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, int block_num, int piece_num)
+			: peer_alert(h, ep, peer_id)
 			, block_index(block_num)
 			, piece_index(piece_num)
 		{ TORRENT_ASSERT(block_index >= 0 && piece_index >= 0);}
@@ -606,18 +706,18 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << peer_alert::message() << " peer dropped block ( piece: "
-				<< piece_index << " block: " << block_index << ")";
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s peer dropped block ( piece: %u block: %u)"
+				, torrent_alert::message().c_str(), piece_index, block_index);
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT block_timeout_alert: peer_alert
 	{
-		block_timeout_alert(const torrent_handle& h, tcp::endpoint const& ip
-			, peer_id const& pid, int block_num, int piece_num)
-			: peer_alert(h, ip, pid)
+		block_timeout_alert(const torrent_handle& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, int block_num, int piece_num)
+			: peer_alert(h, ep, peer_id)
 			, block_index(block_num)
 			, piece_index(piece_num)
 		{ TORRENT_ASSERT(block_index >= 0 && piece_index >= 0);}
@@ -633,18 +733,18 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << peer_alert::message() << " peer timed out request ( piece: "
-				<< piece_index << " block: " << block_index << ")";
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s peer timed out request ( piece: %u block: %u)"
+				, torrent_alert::message().c_str(), piece_index, block_index);
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT block_finished_alert: peer_alert
 	{
-		block_finished_alert(const torrent_handle& h, tcp::endpoint const& ip
-			, peer_id const& pid, int block_num, int piece_num)
-			: peer_alert(h, ip, pid)
+		block_finished_alert(const torrent_handle& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, int block_num, int piece_num)
+			: peer_alert(h, ep, peer_id)
 			, block_index(block_num)
 			, piece_index(piece_num)
 		{ TORRENT_ASSERT(block_index >= 0 && piece_index >= 0);}
@@ -659,18 +759,18 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << peer_alert::message() << " block finished downloading ( piece: "
-				<< piece_index << " block: " << block_index << ")";
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s block finished downloading (piece: %u block: %u)"
+				, torrent_alert::message().c_str(), piece_index, block_index);
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT block_downloading_alert: peer_alert
 	{
-		block_downloading_alert(const torrent_handle& h, tcp::endpoint const& ip
-			, peer_id const& pid, char const* speedmsg, int block_num, int piece_num)
-			: peer_alert(h, ip, pid)
+		block_downloading_alert(const torrent_handle& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, char const* speedmsg, int block_num, int piece_num)
+			: peer_alert(h, ep, peer_id)
 			, peer_speedmsg(speedmsg)
 			, block_index(block_num)
 			, piece_index(piece_num)
@@ -687,18 +787,18 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << peer_alert::message() << " requested block ( piece: "
-				<< piece_index << " block: " << block_index << ") " << peer_speedmsg;
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s requested block (piece: %u block: %u) %s"
+				, torrent_alert::message().c_str(), piece_index, block_index, peer_speedmsg);
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT unwanted_block_alert: peer_alert
 	{
-		unwanted_block_alert(const torrent_handle& h, tcp::endpoint const& ip
-			, peer_id const& pid, int block_num, int piece_num)
-			: peer_alert(h, ip, pid)
+		unwanted_block_alert(const torrent_handle& h, tcp::endpoint const& ep
+			, peer_id const& peer_id, int block_num, int piece_num)
+			: peer_alert(h, ep, peer_id)
 			, block_index(block_num)
 			, piece_index(piece_num)
 		{ TORRENT_ASSERT(block_index >= 0 && piece_index >= 0);}
@@ -711,10 +811,10 @@ namespace libtorrent
 		virtual char const* what() const { return "unwanted block received"; }
 		virtual std::string message() const
 		{
-			std::stringstream ret;
-			ret << peer_alert::message() << " received block not in download queue ( piece: "
-				<< piece_index << " block: " << block_index << ")";
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "%s received block not in download queue (piece: %u block: %u)"
+				, torrent_alert::message().c_str(), piece_index, block_index);
+			return ret;
 		}
 	};
 
@@ -762,9 +862,9 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT torrent_deleted_alert: torrent_alert
 	{
-		torrent_deleted_alert(torrent_handle const& h)
+		torrent_deleted_alert(torrent_handle const& h, sha1_hash const& ih)
 			: torrent_alert(h)
-		{}
+		{ info_hash = ih; }
 	
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new torrent_deleted_alert(*this)); }
@@ -775,16 +875,26 @@ namespace libtorrent
 		{
 			return torrent_alert::message() + " deleted";
 		}
+
+		sha1_hash info_hash;
 	};
 
 	struct TORRENT_EXPORT torrent_delete_failed_alert: torrent_alert
 	{
-		torrent_delete_failed_alert(torrent_handle const& h, std::string msg_)
+		torrent_delete_failed_alert(torrent_handle const& h, error_code const& e)
 			: torrent_alert(h)
-			, msg(msg_)
-		{}
+			, error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 	
+		error_code error;
+
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new torrent_delete_failed_alert(*this)); }
@@ -795,7 +905,7 @@ namespace libtorrent
 		virtual std::string message() const
 		{
 			return torrent_alert::message() + " torrent deletion failed: "
-				+ msg;
+				+ error.message();
 		}
 	};
 
@@ -823,12 +933,20 @@ namespace libtorrent
 	struct TORRENT_EXPORT save_resume_data_failed_alert: torrent_alert
 	{
 		save_resume_data_failed_alert(torrent_handle const& h
-			, std::string const& msg_)
+			, error_code const& e)
 			: torrent_alert(h)
-			, msg(msg_)
-		{}
+			, error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 	
+		error_code error;
+
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 		
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new save_resume_data_failed_alert(*this)); }
@@ -839,7 +957,7 @@ namespace libtorrent
 		virtual std::string message() const
 		{
 			return torrent_alert::message() + " resume data was not generated: "
-				+ msg;
+				+ error.message();
 		}
 	};
 
@@ -898,8 +1016,17 @@ namespace libtorrent
 	{
 		url_seed_alert(
 			torrent_handle const& h
-			, const std::string& url_
-			, const std::string& msg_)
+			, std::string const& url_
+			, error_code const& e)
+			: torrent_alert(h)
+			, url(url_)
+			, msg(e.message())
+		{}
+
+		url_seed_alert(
+			torrent_handle const& h
+			, std::string const& url_
+			, std::string const& msg_)
 			: torrent_alert(h)
 			, url(url_)
 			, msg(msg_)
@@ -924,15 +1051,23 @@ namespace libtorrent
 	{
 		file_error_alert(
 			std::string const& f
-			, const torrent_handle& h
-			, const std::string& msg_)
+			, torrent_handle const& h
+			, error_code const& e)
 			: torrent_alert(h)
 			, file(f)
-			, msg(msg_)
-		{}
+			, error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 
 		std::string file;
+		error_code error;
+
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new file_error_alert(*this)); }
@@ -944,7 +1079,7 @@ namespace libtorrent
 		virtual std::string message() const
 		{
 			return torrent_alert::message() + " file (" + file + ") error: "
-				+ msg;
+				+ error.message();
 		}
 	};
 
@@ -1046,11 +1181,10 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			error_code ec;
-			std::stringstream ret;
-			ret << "listening on " << endpoint
-				<< " failed: " << error.message();
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "listening on %s failed: %s"
+				, print_endpoint(endpoint).c_str(), error.message().c_str());
+			return ret;
 		}
 	};
 
@@ -1069,22 +1203,28 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			error_code ec;
-			std::stringstream ret;
-			ret << "successfully listening on " << endpoint;
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "successfully listening on %s", print_endpoint(endpoint).c_str());
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT portmap_error_alert: alert
 	{
-		portmap_error_alert(int i, int t, const std::string& msg_)
-			:  mapping(i), type(t), msg(msg_)
-		{}
+		portmap_error_alert(int i, int t, error_code const& e)
+			:  mapping(i), type(t), error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 
 		int mapping;
 		int type;
+		error_code error;
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new portmap_error_alert(*this)); }
@@ -1096,7 +1236,7 @@ namespace libtorrent
 		{
 			static char const* type_str[] = {"NAT-PMP", "UPnP"};
 			return std::string("could not map port using ") + type_str[type]
-				+ ": " + msg;
+				+ ": " + error.message();
 		}
 	};
 
@@ -1118,22 +1258,53 @@ namespace libtorrent
 		virtual std::string message() const
 		{
 			static char const* type_str[] = {"NAT-PMP", "UPnP"};
-			std::stringstream ret;
-			ret << "successfully mapped port using " << type_str[type]
-				<< ". external port: " << external_port;
-			return ret.str();
+			char ret[200];
+			snprintf(ret, sizeof(ret), "successfully mapped port using %s. external port: %u"
+				, type_str[type], external_port);
+			return ret;
+		}
+	};
+
+	struct TORRENT_EXPORT portmap_log_alert: alert
+	{
+		portmap_log_alert(int t, std::string const& m)
+			: type(t), msg(m)
+		{}
+
+		int type;
+		std::string msg;
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new portmap_log_alert(*this)); }
+		virtual char const* what() const { return "port map log"; }
+		const static int static_category = alert::port_mapping_notification;
+		virtual int category() const { return static_category; }
+		virtual std::string message() const
+		{
+			static char const* type_str[] = {"NAT-PMP", "UPnP"};
+			char ret[600];
+			snprintf(ret, sizeof(ret), "%s: %s", type_str[type], msg.c_str());
+			return ret;
 		}
 	};
 
 	struct TORRENT_EXPORT fastresume_rejected_alert: torrent_alert
 	{
 		fastresume_rejected_alert(torrent_handle const& h
-			, std::string const& msg_)
+			, error_code const& e)
 			: torrent_alert(h)
-			, msg(msg_)
-		{}
+			, error(e)
+		{
+#ifndef TORRENT_NO_DEPRECATE
+			msg = error.message();
+#endif
+		}
 
+		error_code error;
+
+#ifndef TORRENT_NO_DEPRECATE
 		std::string msg;
+#endif
 
 		virtual std::auto_ptr<alert> clone() const
 		{ return std::auto_ptr<alert>(new fastresume_rejected_alert(*this)); }
@@ -1143,7 +1314,7 @@ namespace libtorrent
 		virtual int category() const { return static_category; }
 		virtual std::string message() const
 		{
-			return torrent_alert::message() + " fast resume rejected: " + msg;
+			return torrent_alert::message() + " fast resume rejected: " + error.message();
 		}
 	};
 
@@ -1166,6 +1337,102 @@ namespace libtorrent
 			return "blocked peer: " + ip.to_string(ec);
 		}
 	};
+
+	struct TORRENT_EXPORT dht_announce_alert: alert
+	{
+		dht_announce_alert(address const& ip_, int port_
+			, sha1_hash const& info_hash_)
+			: ip(ip_)
+			, port(port_)
+			, info_hash(info_hash_)
+		{}
+		
+		address ip;
+		int port;
+		sha1_hash info_hash;
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new dht_announce_alert(*this)); }
+		virtual char const* what() const { return "incoming dht announce"; }
+		const static int static_category = alert::dht_notification;
+		virtual int category() const { return static_category; }
+		virtual std::string message() const
+		{
+			error_code ec;
+			char ih_hex[41];
+			to_hex((const char*)&info_hash[0], 20, ih_hex);
+			char msg[200];
+			snprintf(msg, sizeof(msg), "incoming dht announce: %s:%u (%s)"
+				, ip.to_string(ec).c_str(), port, ih_hex);
+			return msg;
+		}
+	};
+
+	struct TORRENT_EXPORT dht_get_peers_alert: alert
+	{
+		dht_get_peers_alert(sha1_hash const& info_hash_)
+			: info_hash(info_hash_)
+		{}
+		
+		sha1_hash info_hash;
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new dht_get_peers_alert(*this)); }
+		virtual char const* what() const { return "incoming dht get_peers request"; }
+		const static int static_category = alert::dht_notification;
+		virtual int category() const { return static_category; }
+		virtual std::string message() const
+		{
+			char ih_hex[41];
+			to_hex((const char*)&info_hash[0], 20, ih_hex);
+			char msg[200];
+			snprintf(msg, sizeof(msg), "incoming dht get_peers: %s", ih_hex);
+			return msg;
+		}
+	};
+
+	struct TORRENT_EXPORT stats_alert: torrent_alert
+	{
+		stats_alert(torrent_handle const& h, int interval
+			, stat const& s);
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new stats_alert(*this)); }
+		const static int static_category = alert::stats_notification;
+		virtual int category() const { return static_category; }
+		virtual char const* what() const { return "stats_alert"; }
+
+		virtual std::string message() const;
+
+		enum stats_channel
+		{
+			upload_payload,
+			upload_protocol,
+			upload_ip_protocol,
+			upload_dht_protocol,
+			upload_tracker_protocol,
+			download_payload,
+			download_protocol,
+			download_ip_protocol,
+			download_dht_protocol,
+			download_tracker_protocol,
+			num_channels
+		};
+
+		int transferred[num_channels];
+		int interval;
+	};
+
+	struct TORRENT_EXPORT cache_flushed_alert: torrent_alert
+	{
+		cache_flushed_alert(torrent_handle const& h);
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new cache_flushed_alert(*this)); }
+		const static int static_category = alert::storage_notification;
+		virtual int category() const { return static_category; }
+		virtual char const* what() const { return "cache_flushed_alert"; }
+	};
 }
 
 
