@@ -33,12 +33,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_SESSION_IMPL_HPP_INCLUDED
 #define TORRENT_SESSION_IMPL_HPP_INCLUDED
 
-#include <ctime>
 #include <algorithm>
 #include <vector>
 #include <set>
 #include <list>
-#include <deque>
 
 #ifndef TORRENT_DISABLE_GEO_IP
 #ifdef WITH_SHIPPED_GEOIP_H
@@ -52,12 +50,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #pragma warning(push, 1)
 #endif
 
-#include <boost/limits.hpp>
-#include <boost/tuple/tuple.hpp>
 #include <boost/filesystem/path.hpp>
 #include <boost/thread.hpp>
-#include <boost/thread/recursive_mutex.hpp>
 #include <boost/thread/condition.hpp>
+#include <boost/pool/object_pool.hpp>
 
 #ifdef _MSC_VER
 #pragma warning(pop)
@@ -68,7 +64,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket.hpp"
 #include "libtorrent/peer_id.hpp"
 #include "libtorrent/tracker_manager.hpp"
-#include "libtorrent/alert.hpp"
 #include "libtorrent/debug.hpp"
 #include "libtorrent/piece_block_progress.hpp"
 #include "libtorrent/ip_filter.hpp"
@@ -82,8 +77,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket_type.hpp"
 #include "libtorrent/connection_queue.hpp"
 #include "libtorrent/disk_io_thread.hpp"
-#include "libtorrent/assert.hpp"
 #include "libtorrent/udp_socket.hpp"
+#include "libtorrent/assert.hpp"
+#include "libtorrent/policy.hpp" // for policy::peer
+#include "libtorrent/alert.hpp" // for alert_manager
 
 #if TORRENT_COMPLETE_TYPES_REQUIRED
 #include "libtorrent/peer_connection.hpp"
@@ -97,11 +94,13 @@ namespace libtorrent
 	class upnp;
 	class natpmp;
 	class lsd;
-	class fingerprint;
+	struct fingerprint;
+	class torrent;
+	class alert;
 
 	namespace dht
 	{
-		class dht_tracker;
+		struct dht_tracker;
 	}
 
 	namespace aux
@@ -112,13 +111,20 @@ namespace libtorrent
 		struct tracker_logger;
 #endif
 
+		// used to initialize the g_current_time before
+		// anything else
+		struct initialize_timer
+		{
+			initialize_timer();
+		};
+
 		// this is the link between the main thread and the
 		// thread started to run the main downloader loop
-		struct session_impl: boost::noncopyable
+		struct session_impl: boost::noncopyable, initialize_timer
 		{
 
 			// the size of each allocation that is chained in the send buffer
-			enum { send_buffer_size = 200 };
+			enum { send_buffer_size = 128 };
 
 #ifdef TORRENT_DEBUG
 			friend class ::libtorrent::peer_connection;
@@ -161,19 +167,22 @@ namespace libtorrent
 			tcp::endpoint get_ipv4_interface() const;
 
 			void async_accept(boost::shared_ptr<socket_acceptor> const& listener);
-			void on_incoming_connection(boost::shared_ptr<socket_type> const& s
+			void on_accept_connection(boost::shared_ptr<socket_type> const& s
 				, boost::weak_ptr<socket_acceptor> listener, error_code const& e);
+			void on_socks_accept(boost::shared_ptr<socket_type> const& s
+				, error_code const& e);
+
+			void incoming_connection(boost::shared_ptr<socket_type> const& s);
 		
 			// must be locked to access the data
 			// in this struct
-			typedef boost::recursive_mutex mutex_t;
+			typedef boost::mutex mutex_t;
 			mutable mutex_t m_mutex;
 
 			boost::weak_ptr<torrent> find_torrent(const sha1_hash& info_hash);
 			peer_id const& get_peer_id() const { return m_peer_id; }
 
-			void close_connection(peer_connection const* p
-				, char const* message);
+			void close_connection(peer_connection const* p, error_code const& ec);
 
 			void set_settings(session_settings const& s);
 			session_settings const& settings() const { return m_settings; }
@@ -184,10 +193,14 @@ namespace libtorrent
 			void add_dht_router(std::pair<std::string, int> const& node);
 			void set_dht_settings(dht_settings const& s);
 			dht_settings const& get_dht_settings() const { return m_dht_settings; }
-			void start_dht(entry const& startup_state);
+			void start_dht();
 			void stop_dht();
+			void start_dht(entry const& startup_state);
 
-			entry dht_state() const;
+#ifndef TORRENT_NO_DEPRECATE
+			entry dht_state(session_impl::mutex_t::scoped_lock& l) const;
+#endif
+			void maybe_update_udp_mapping(int nat, int local_port, int external_port);
 			void on_dht_router_name_lookup(error_code const& e
 				, tcp::resolver::iterator host);
 #endif
@@ -197,9 +210,13 @@ namespace libtorrent
 			pe_settings const& get_pe_settings() const { return m_pe_settings; }
 #endif
 
+			void on_port_map_log(char const* msg, int map_transport);
+
+			void on_lsd_announce(error_code const& e);
+
 			// called when a port mapping is successful, or a router returns
 			// a failure to map a port
-			void on_port_mapping(int mapping, int port, std::string const& errmsg
+			void on_port_mapping(int mapping, int port, error_code const& ec
 				, int nat_transport);
 
 			bool is_aborted() const { return m_abort; }
@@ -218,23 +235,29 @@ namespace libtorrent
 				, const char* net_interface = 0);
 			bool is_listening() const;
 
-			torrent_handle add_torrent(add_torrent_params const&);
+			torrent_handle add_torrent(add_torrent_params const&, error_code& ec);
 
 			void remove_torrent(torrent_handle const& h, int options);
 
 			std::vector<torrent_handle> get_torrents();
 			
-			void check_torrent(boost::shared_ptr<torrent> const& t);
-			void done_checking(boost::shared_ptr<torrent> const& t);
+			void queue_check_torrent(boost::shared_ptr<torrent> const& t);
+			void dequeue_check_torrent(boost::shared_ptr<torrent> const& t);
 
 			void set_alert_mask(int m);
 			size_t set_alert_queue_size_limit(size_t queue_size_limit_);
 			std::auto_ptr<alert> pop_alert();
+			void set_alert_dispatch(boost::function<void(alert const&)> const&);
 
 			alert const* wait_for_alert(time_duration max_wait);
 
 			int upload_rate_limit() const;
 			int download_rate_limit() const;
+			int local_upload_rate_limit() const;
+			int local_download_rate_limit() const;
+
+			void set_local_download_rate_limit(int bytes_per_second);
+			void set_local_upload_rate_limit(int bytes_per_second);
 
 			void set_download_rate_limit(int bytes_per_second);
 			void set_upload_rate_limit(int bytes_per_second);
@@ -251,6 +274,7 @@ namespace libtorrent
 			{ return m_connections.size(); }
 
 			void unchoke_peer(peer_connection& c);
+			void choke_peer(peer_connection& c);
 
 			session_status status() const;
 			void set_peer_id(peer_id const& id);
@@ -263,8 +287,16 @@ namespace libtorrent
 
 			void announce_lsd(sha1_hash const& ih);
 
+			void save_state(entry& e, boost::uint32_t flags, session_impl::mutex_t::scoped_lock& l) const;
+			void load_state(lazy_entry const& e);
+
 			void set_peer_proxy(proxy_settings const& s)
-			{ m_peer_proxy = s; }
+			{
+				m_peer_proxy = s;
+				// in case we just set a socks proxy, we might have to
+				// open the socks incoming connection
+				if (!m_socks_listen_socket) open_new_incoming_socks_connection();
+			}
 			void set_web_seed_proxy(proxy_settings const& s)
 			{ m_web_seed_proxy = s; }
 			void set_tracker_proxy(proxy_settings const& s)
@@ -297,14 +329,19 @@ namespace libtorrent
 			bool load_country_db(char const* file);
 			bool has_country_db() const { return m_country_db; }
 			char const* country_for_ip(address const& a);
+
+#ifndef BOOST_FILESYSTEM_NARROW_ONLY
+			bool load_asnum_db(wchar_t const* file);
+			bool load_country_db(wchar_t const* file);
+#endif
 #endif
 
 			void load_state(entry const& ses_state);
 			entry state() const;
 
 			void start_lsd();
-			natpmp* start_natpmp();
-			upnp* start_upnp();
+			void start_natpmp(natpmp* n);
+			void start_upnp(upnp* u);
 
 			void stop_lsd();
 			void stop_natpmp();
@@ -324,13 +361,10 @@ namespace libtorrent
 				m_total_failed_bytes += b;
 			}
 
-			// handles delayed alerts
-			alert_manager m_alerts;
-
 			std::pair<char*, int> allocate_buffer(int size);
 			void free_buffer(char* buf, int size);
 
-			char* allocate_disk_buffer();
+			char* allocate_disk_buffer(char const* category);
 			void free_disk_buffer(char* buf);
 
 			void set_external_address(address const& ip);
@@ -338,9 +372,55 @@ namespace libtorrent
 
 //		private:
 
-			void dht_state_callback(boost::condition& c
+			void update_disk_thread_settings();
+			void on_dht_state_callback(boost::condition& c
 				, entry& e, bool& done) const;
 			void on_lsd_peer(tcp::endpoint peer, sha1_hash const& ih);
+			void setup_socket_buffers(socket_type& s);
+
+			// this is a shared pool where policy_peer objects
+			// are allocated. It's a pool since we're likely
+			// to have tens of thousands of peers, and a pool
+			// saves significant overhead
+#ifdef TORRENT_STATS
+			struct logging_allocator
+			{
+				typedef std::size_t size_type;
+				typedef std::ptrdiff_t difference_type;
+
+				static char* malloc(const size_type bytes)
+				{
+					allocated_bytes += bytes;
+					++allocations;
+					return (char*)::malloc(bytes);
+				}
+
+				static void free(char* const block)
+				{
+					--allocations;
+					return ::free(block);
+				}
+			
+				static int allocations;
+				static int allocated_bytes;
+			};
+			boost::object_pool<
+				policy::ipv4_peer, logging_allocator> m_ipv4_peer_pool;
+# if TORRENT_USE_IPV6
+			boost::object_pool<
+				policy::ipv6_peer, logging_allocator> m_ipv6_peer_pool;
+# endif
+#else
+			boost::object_pool<policy::ipv4_peer> m_ipv4_peer_pool;
+# if TORRENT_USE_IPV6
+			boost::object_pool<policy::ipv6_peer> m_ipv6_peer_pool;
+# endif
+#endif
+
+			// this vector is used to store the block_info
+			// objects pointed to by partial_piece_info returned
+			// by torrent::get_download_queue.
+			std::vector<block_info> m_block_info_storage;
 
 #ifndef TORRENT_DISABLE_POOL_ALLOCATOR
 			// this pool is used to allocate and recycle send
@@ -364,6 +444,9 @@ namespace libtorrent
 
 			tcp::resolver m_host_resolver;
 
+			// handles delayed alerts
+			alert_manager m_alerts;
+
 			// handles disk io requests asynchronously
 			// peers have pointers into the disk buffer
 			// pool, and must be destructed before this
@@ -384,14 +467,28 @@ namespace libtorrent
 			// handing out bandwidth to connections that
 			// asks for it, it can also throttle the
 			// rate.
-			bandwidth_manager<peer_connection, torrent> m_download_channel;
-			bandwidth_manager<peer_connection, torrent> m_upload_channel;
+			bandwidth_manager<peer_connection> m_download_rate;
+			bandwidth_manager<peer_connection> m_upload_rate;
 
-			bandwidth_manager<peer_connection, torrent>* m_bandwidth_manager[2];
+			// the global rate limiter bandwidth channels
+			bandwidth_channel m_download_channel;
+			bandwidth_channel m_upload_channel;
+
+			// bandwidth channels for local peers when
+			// rate limits are ignored. They are only
+			// throttled by these global rate limiters
+			// and they don't have a rate limit set by
+			// default
+			bandwidth_channel m_local_download_channel;
+			bandwidth_channel m_local_upload_channel;
+
+			bandwidth_channel* m_bandwidth_channel[2];
 
 			tracker_manager m_tracker_manager;
 			torrent_map m_torrents;
 			typedef std::list<boost::shared_ptr<torrent> > check_queue_t;
+
+			// this has all torrents that wants to be checked in it
 			check_queue_t m_queued_for_checking;
 
 			// this maps sockets to their peer_connection
@@ -450,6 +547,12 @@ namespace libtorrent
 			// we might need more than one listen socket
 			std::list<listen_socket_t> m_listen_sockets;
 
+			// when as a socks proxy is used for peers, also
+			// listen for incoming connections on a socks connection
+			boost::shared_ptr<socket_type> m_socks_listen_socket;
+
+			void open_new_incoming_socks_connection();
+
 			listen_socket_t setup_listener(tcp::endpoint ep, int retries, bool v6_only = false);
 
 			// the settings for the client
@@ -463,6 +566,9 @@ namespace libtorrent
 			proxy_settings m_dht_proxy;
 #endif
 
+#ifndef TORRENT_DISABLE_DHT	
+			entry m_dht_state;
+#endif
 			// set to true when the session object
 			// is being destructed and the thread
 			// should exit
@@ -521,12 +627,25 @@ namespace libtorrent
 			// NAT or not.
 			bool m_incoming_connection;
 			
-			void second_tick(error_code const& e);
+			void on_disk_queue();
+			void on_tick(error_code const& e);
+
+			int auto_manage_torrents(std::vector<torrent*>& list
+				, int hard_limit, int type_limit);
 			void recalculate_auto_managed_torrents();
 			void recalculate_unchoke_slots(int congested_torrents
 				, int uncongested_torrents);
+			void recalculate_optimistic_unchoke_slot();
+
+			ptime m_created;
+			int session_time() const { return total_seconds(time_now() - m_created); }
 
 			ptime m_last_tick;
+			ptime m_last_second_tick;
+
+			// the last time we went through the peers
+			// to decide which ones to choke/unchoke
+			ptime m_last_choke;
 
 			// when outgoing_ports is configured, this is the
 			// port we'll bind the next outgoing socket to
@@ -545,7 +664,7 @@ namespace libtorrent
 			// but for the udp port used by the DHT.
 			int m_external_udp_port;
 
-			udp_socket m_dht_socket;
+			rate_limited_udp_socket m_dht_socket;
 
 			// these are used when starting the DHT
 			// (and bootstrapping it), and then erased
@@ -567,20 +686,32 @@ namespace libtorrent
 			int m_tcp_mapping[2];
 			int m_udp_mapping[2];
 
-			// the timer used to fire the second_tick
+			// the timer used to fire the tick
 			deadline_timer m_timer;
 
+			// torrents are announced on the local network in a
+			// round-robin fashion. All torrents are cycled through
+			// within the LSD announce interval (which defaults to
+			// 5 minutes)
+			torrent_map::iterator m_next_lsd_torrent;
+
+			// this announce timer is used
+			// by Local service discovery
+			deadline_timer m_lsd_announce_timer;
+
 			// the index of the torrent that will be offered to
-			// connect to a peer next time second_tick is called.
+			// connect to a peer next time on_tick is called.
 			// This implements a round robin.
-			int m_next_connect_torrent;
+			torrent_map::iterator m_next_connect_torrent;
 #ifdef TORRENT_DEBUG
 			void check_invariant() const;
 #endif
 
-#ifdef TORRENT_STATS
+#if defined TORRENT_STATS && defined TORRENT_DISK_STATS
 			void log_buffer_usage();
+#endif
 
+#if defined TORRENT_STATS
 			// logger used to write bandwidth usage statistics
 			std::ofstream m_stats_logger;
 			int m_second_counter;
@@ -603,6 +734,9 @@ namespace libtorrent
 			boost::shared_ptr<logger> m_logger;
 		private:
 
+#endif
+#ifdef TORRENT_UPNP_LOGGING
+			std::ofstream m_upnp_log;
 #endif
 			address m_external_address;
 
@@ -643,26 +777,33 @@ namespace libtorrent
 			}
 
 			void tracker_response(tracker_request const&
+				, libtorrent::address const& tracker_ip
+				, std::list<address> const& ip_list
 				, std::vector<peer_entry>& peers
 				, int interval
+				, int min_interval
 				, int complete
 				, int incomplete
 				, address const& external_ip)
 			{
-				std::stringstream s;
-				s << "TRACKER RESPONSE:\n"
-					"interval: " << interval << "\n"
-					"peers:\n";
+				std::string s;
+				s = "TRACKER RESPONSE:\n";
+				char tmp[200];
+				snprintf(tmp, 200, "interval: %d\nmin_interval: %d\npeers:\n", interval, min_interval);
+				s += tmp;
 				for (std::vector<peer_entry>::const_iterator i = peers.begin();
 					i != peers.end(); ++i)
 				{
-					s << "  " << std::setfill(' ') << std::setw(16) << i->ip
-						<< " " << std::setw(5) << std::dec << i->port << "  ";
-					if (!i->pid.is_all_zeros()) s << " " << i->pid;
-					s << "\n";
+					char pid[41];
+					to_hex((const char*)&i->pid[0], 20, pid);
+					if (i->pid.is_all_zeros()) pid[0] = 0;
+
+					snprintf(tmp, 200, " %-16s %-5d %s\n", i->ip.c_str(), i->port, pid);
+					s += tmp;
 				}
-				s << "external ip: " << external_ip << "\n";
-				debug_log(s.str());
+				snprintf(tmp, 200, "external ip: %s\n", print_address(external_ip).c_str());
+				s += tmp;
+				debug_log(s);
 			}
 
 			void tracker_request_timed_out(
@@ -674,11 +815,12 @@ namespace libtorrent
 			void tracker_request_error(
 				tracker_request const&
 				, int response_code
-				, const std::string& str)
+				, const std::string& str
+				, int retry_interval)
 			{
-				debug_log(std::string("*** tracker error: ")
-					+ boost::lexical_cast<std::string>(response_code) + ": "
-					+ str);
+				char msg[256];
+				snprintf(msg, sizeof(msg), "*** tracker error: %d: %s", response_code, str.c_str());
+				debug_log(msg);
 			}
 			
 			void debug_log(const std::string& line)
