@@ -35,6 +35,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/assert.hpp"
 #include "libtorrent/config.hpp"
+#include <cstring> // for memset and memcpy
 
 namespace libtorrent
 {
@@ -45,22 +46,22 @@ namespace libtorrent
 		{ resize(bits); }
 		bitfield(int bits, bool val): m_bytes(0), m_size(0)
 		{ resize(bits, val); }
-		bitfield(char const* bytes, int bits): m_bytes(0), m_size(0)
-		{ assign(bytes, bits); }
+		bitfield(char const* b, int bits): m_bytes(0), m_size(0)
+		{ assign(b, bits); }
 		bitfield(bitfield const& rhs): m_bytes(0), m_size(0), m_own(false)
 		{ assign(rhs.bytes(), rhs.size()); }
 
-		void borrow_bytes(char* bytes, int bits)
+		void borrow_bytes(char* b, int bits)
 		{
 			dealloc();
-			m_bytes = (unsigned char*)bytes;
+			m_bytes = (unsigned char*)b;
 			m_size = bits;
 			m_own = false;
 		}
 		~bitfield() { dealloc(); }
 
-		void assign(char const* bytes, int bits)
-		{ resize(bits); std::memcpy(m_bytes, bytes, (bits + 7) / 8); clear_trailing_bits(); }
+		void assign(char const* b, int bits)
+		{ resize(bits); std::memcpy(m_bytes, b, (bits + 7) / 8); clear_trailing_bits(); }
 
 		bool operator[](int index) const
 		{ return get_bit(index); }
@@ -69,7 +70,7 @@ namespace libtorrent
 		{
 			TORRENT_ASSERT(index >= 0);
 			TORRENT_ASSERT(index < m_size);
-			return m_bytes[index / 8] & (0x80 >> (index & 7));
+			return (m_bytes[index / 8] & (0x80 >> (index & 7))) != 0;
 		}
 		
 		void clear_bit(int index)
@@ -134,7 +135,7 @@ namespace libtorrent
 			typedef bool& reference;
 			typedef std::forward_iterator_tag iterator_category;
 
-			bool operator*() { return *byte & bit; }
+			bool operator*() { return (*byte & bit) != 0; }
 			const_iterator& operator++() { inc(); return *this; }
 			const_iterator operator++(int)
 			{ const_iterator ret(*this); inc(); return ret; }
@@ -220,31 +221,33 @@ namespace libtorrent
 	
 		void resize(int bits)
 		{
-			const int bytes = (bits + 7) / 8;
+			const int b = (bits + 7) / 8;
 			if (m_bytes)
 			{
 				if (m_own)
 				{
-					m_bytes = (unsigned char*)std::realloc(m_bytes, bytes);
+					m_bytes = (unsigned char*)std::realloc(m_bytes, b);
 					m_own = true;
 				}
 				else if (bits > m_size)
 				{
-					unsigned char* tmp = (unsigned char*)std::malloc(bytes);
-					std::memcpy(tmp, m_bytes, (std::min)((m_size + 7)/ 8, bytes));
+					unsigned char* tmp = (unsigned char*)std::malloc(b);
+					std::memcpy(tmp, m_bytes, (std::min)(int(m_size + 7)/ 8, b));
 					m_bytes = tmp;
 					m_own = true;
 				}
 			}
 			else
 			{
-				m_bytes = (unsigned char*)std::malloc(bytes);
+				m_bytes = (unsigned char*)std::malloc(b);
 				m_own = true;
 			}
 			m_size = bits;
 			clear_trailing_bits();
 		}
 
+		void free() { dealloc(); m_size = 0; }
+
 	private:
 
 		void clear_trailing_bits()
@@ -255,8 +258,8 @@ namespace libtorrent
 
 		void dealloc() { if (m_own) std::free(m_bytes); m_bytes = 0; }
 		unsigned char* m_bytes;
-		int m_size; // in bits
-		bool m_own;
+		int m_size:31; // in bits
+		bool m_own:1;
 	};
 
 }
