@@ -40,6 +40,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 #include "libtorrent/storage.hpp"
 #include "libtorrent/hasher.hpp"
+#include "libtorrent/utf8.hpp"
+#include "libtorrent/allocator.hpp"
 
 #include <vector>
 #include <string>
@@ -52,8 +54,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/operations.hpp>
 #include <boost/optional.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
+#include <boost/date_time/posix_time/posix_time_types.hpp>
 #include <boost/scoped_ptr.hpp>
+#include <boost/config.hpp>
 
 #ifdef _MSC_VER
 #pragma warning(pop)
@@ -63,11 +66,20 @@ namespace libtorrent
 {
 	namespace fs = boost::filesystem;
 	namespace pt = boost::posix_time;
+	class torrent_info;
 
 	struct TORRENT_EXPORT create_torrent
 	{
-		create_torrent(file_storage& fs, int piece_size);
-		create_torrent(file_storage& fs);
+		enum {
+			optimize = 1
+			, merkle = 2
+			, modification_time = 4
+			, symlinks = 8
+		};
+
+		create_torrent(file_storage& fs, int piece_size = 0
+			, int pad_file_limit = -1, int flags = optimize);
+		create_torrent(torrent_info const& ti);
 		entry generate() const;
 
 		file_storage const& files() const { return m_files; }
@@ -88,6 +100,10 @@ namespace libtorrent
 	private:
 
 		file_storage& m_files;
+		// if m_info_dict is initialized, it is 
+		// used instead of m_files to generate
+		// the info dictionary
+		entry m_info_dict;
 
 		// the urls to the trackers
 		typedef std::pair<std::string, int> announce_entry;
@@ -124,72 +140,132 @@ namespace libtorrent
 		// to know if it should be written as a multifile torrent
 		// or not. e.g. test/test  there's one file and one directory
 		// and they have the same name.
-		bool m_multifile;
+		bool m_multifile:1;
 		
 		// this is true if the torrent is private. i.e., is should not
 		// be announced on the dht
-		bool m_private;
+		bool m_private:1;
+
+		// if set to one, a merkle torrent will be generated
+		bool m_merkle_torrent:1;
+
+		// if set, include the 'mtime' modification time in the
+		// torrent file
+		bool m_include_mtime:1;
+
+		// if set, symbolic links are declared as such in
+		// the torrent file. The full data of the pointed-to
+		// file is still included
+		bool m_include_symlinks:1;
 	};
 
 	namespace detail
 	{
 		inline bool default_pred(boost::filesystem::path const&) { return true; }
+#if TORRENT_USE_WPATH
+		inline bool wdefault_pred(boost::filesystem::wpath const&) { return true; }
+#endif
+
+		inline bool ignore_subdir(std::string const& leaf)
+		{ return leaf == ".." || leaf == "."; }
+
+		inline bool ignore_subdir(std::wstring const& leaf)
+		{ return leaf == L".." || leaf == L"."; }
 
 		inline void nop(int i) {}
 
-		template <class Pred>
-		void add_files_impl(file_storage& fs, boost::filesystem::path const& p
-			, boost::filesystem::path const& l, Pred pred)
+		int TORRENT_EXPORT get_file_attributes(boost::filesystem::path const& p);
+#if TORRENT_USE_WPATH
+		int TORRENT_EXPORT get_file_attributes(boost::filesystem::wpath const& p);
+#endif
+
+		std::time_t TORRENT_EXPORT get_file_mtime(boost::filesystem::path const& p);
+#if TORRENT_USE_WPATH
+		std::time_t TORRENT_EXPORT get_file_mtime(boost::filesystem::wpath const& p);
+#endif
+
+		fs::path TORRENT_EXPORT get_symlink_path(boost::filesystem::path const& p);
+#if TORRENT_USE_WPATH
+		fs::path TORRENT_EXPORT get_symlink_path(boost::filesystem::wpath const& p);
+#endif
+
+		template <class Pred, class Str, class PathTraits>
+		void add_files_impl(file_storage& fs, boost::filesystem::basic_path<Str, PathTraits> const& p
+			, boost::filesystem::basic_path<Str, PathTraits> const& l, Pred pred)
 		{
-			using boost::filesystem::path;
-			using boost::filesystem::directory_iterator;
-			if (!pred(l)) return;
-			path f(p / l);
+			using boost::filesystem::basic_path;
+			using boost::filesystem::basic_directory_iterator;
+			basic_path<Str, PathTraits> f(p / l);
+			if (!pred(f)) return;
 			if (is_directory(f))
 			{
-				for (directory_iterator i(f), end; i != end; ++i)
+				for (basic_directory_iterator<basic_path<Str, PathTraits> > i(f), end; i != end; ++i)
 				{
 #if BOOST_VERSION < 103600
-					std::string leaf = i->path().leaf();
+					Str const& leaf = i->path().leaf();
 #else
-					std::string leaf = i->path().filename();
+					Str const& leaf = i->path().filename();
 #endif
-					if (leaf == ".." || leaf == ".") continue;
+					if (ignore_subdir(leaf)) continue;
 					add_files_impl(fs, p, l / leaf, pred);
 				}
 			}
 			else
 			{
-				fs.add_file(l, file_size(f));
+				int file_flags = get_file_attributes(f);
+				std::time_t mtime = get_file_mtime(f);
+				//Masking all bits to check if the file is a symlink
+				if(file_flags & file_storage::attribute_symlink) 
+				{
+					fs::path sym_path = get_symlink_path(f);
+					fs.add_file(l, 0 ,file_flags, mtime, sym_path);
+				}
+				else
+				{
+					fs.add_file(l, file_size(f), file_flags, mtime);
+				}
 			}
 		}
 	}
 
+	// path versions
+
 	template <class Pred>
 	void add_files(file_storage& fs, boost::filesystem::path const& file, Pred p)
 	{
+		using boost::filesystem::path;
 		boost::filesystem::path f = file;
 #if BOOST_VERSION < 103600
 		if (f.leaf() == ".") f = f.branch_path();
-		detail::add_files_impl(fs, complete(f).branch_path(), f.leaf(), p);
+		detail::add_files_impl(fs, complete(f).branch_path(), path(f.leaf()), p);
 #else
 		if (f.filename() == ".") f = f.parent_path();
-		detail::add_files_impl(fs, complete(f).parent_path(), f.filename(), p);
+		detail::add_files_impl(fs, complete(f).parent_path(), path(f.filename()), p);
 #endif
 	}
 
 	inline void add_files(file_storage& fs, boost::filesystem::path const& file)
 	{
+		using boost::filesystem::path;
 		boost::filesystem::path f = file;
 #if BOOST_VERSION < 103600
 		if (f.leaf() == ".") f = f.branch_path();
-		detail::add_files_impl(fs, complete(f).branch_path(), f.leaf(), detail::default_pred);
+		detail::add_files_impl(fs, complete(f).branch_path(), path(f.leaf()), detail::default_pred);
 #else
 		if (f.filename() == ".") f = f.parent_path();
-		detail::add_files_impl(fs, complete(f).parent_path(), f.filename(), detail::default_pred);
+		detail::add_files_impl(fs, complete(f).parent_path(), path(f.filename()), detail::default_pred);
 #endif
 	}
 	
+	struct piece_holder
+	{
+		piece_holder(int bytes): m_piece(page_aligned_allocator::malloc(bytes)) {}
+		~piece_holder() { page_aligned_allocator::free(m_piece); }
+		char* bytes() { return m_piece; }
+	private:
+		char* m_piece;
+	};
+
 	template <class Fun>
 	void set_piece_hashes(create_torrent& t, boost::filesystem::path const& p, Fun f
 		, error_code& ec)
@@ -200,18 +276,18 @@ namespace libtorrent
 
 		// calculate the hash for all pieces
 		int num = t.num_pieces();
-		std::vector<char> buf(t.piece_length());
+		piece_holder buf(t.piece_length());
 		for (int i = 0; i < num; ++i)
 		{
 			// read hits the disk and will block. Progress should
 			// be updated in between reads
-			st->read(&buf[0], i, 0, t.piece_size(i));
+			st->read(buf.bytes(), i, 0, t.piece_size(i));
 			if (st->error())
 			{
 				ec = st->error();
 				return;
 			}
-			hasher h(&buf[0], t.piece_size(i));
+			hasher h(buf.bytes(), t.piece_size(i));
 			t.set_hash(i, h.final());
 			f(i);
 		}
@@ -238,7 +314,90 @@ namespace libtorrent
 	{
 		set_piece_hashes(t, p, detail::nop, ec);
 	}
+
+#if TORRENT_USE_WPATH
+	// wpath versions
+
+	template <class Pred>
+	void add_files(file_storage& fs, boost::filesystem::wpath const& file, Pred p)
+	{
+		using boost::filesystem::wpath;
+		wpath f = file;
+#if BOOST_VERSION < 103600
+		if (f.leaf() == L".") f = f.branch_path();
+		detail::add_files_impl(fs, complete(f).branch_path(), wpath(f.leaf()), p);
+#else
+		if (f.filename() == L".") f = f.parent_path();
+		detail::add_files_impl(fs, complete(f).parent_path(), wpath(f.filename()), p);
+#endif
+	}
+
+	inline void add_files(file_storage& fs, boost::filesystem::wpath const& file)
+	{
+		using boost::filesystem::wpath;
+		wpath f = file;
+#if BOOST_VERSION < 103600
+		if (f.leaf() == L".") f = f.branch_path();
+		detail::add_files_impl(fs, complete(f).branch_path(), wpath(f.leaf()), detail::wdefault_pred);
+#else
+		if (f.filename() == L".") f = f.parent_path();
+		detail::add_files_impl(fs, complete(f).parent_path(), wpath(f.filename()), detail::wdefault_pred);
+#endif
+	}
+	
+	template <class Fun>
+	void set_piece_hashes(create_torrent& t, boost::filesystem::wpath const& p, Fun f
+		, error_code& ec)
+	{
+		file_pool fp;
+		std::string utf8;
+		wchar_utf8(p.string(), utf8);
+		boost::scoped_ptr<storage_interface> st(
+			default_storage_constructor(const_cast<file_storage&>(t.files()), 0, utf8, fp));
+
+		// calculate the hash for all pieces
+		int num = t.num_pieces();
+		std::vector<char> buf(t.piece_length());
+		for (int i = 0; i < num; ++i)
+		{
+			// read hits the disk and will block. Progress should
+			// be updated in between reads
+			st->read(&buf[0], i, 0, t.piece_size(i));
+			if (st->error())
+			{
+				ec = st->error();
+				return;
+			}
+			hasher h(&buf[0], t.piece_size(i));
+			t.set_hash(i, h.final());
+			f(i);
+		}
+	}
+
+#ifndef BOOST_NO_EXCEPTIONS
+	template <class Fun>
+	void set_piece_hashes(create_torrent& t, boost::filesystem::wpath const& p, Fun f)
+	{
+		error_code ec;
+		set_piece_hashes(t, p, f, ec);
+		if (ec) throw libtorrent_exception(ec);
+	}
+
+	inline void set_piece_hashes(create_torrent& t, boost::filesystem::wpath const& p)
+	{
+		error_code ec;
+		set_piece_hashes(t, p, detail::nop, ec);
+		if (ec) throw libtorrent_exception(ec);
+	}
+#endif
+
+	inline void set_piece_hashes(create_torrent& t, boost::filesystem::wpath const& p, error_code& ec)
+	{
+		set_piece_hashes(t, p, detail::nop, ec);
+	}
+
+#endif // TORRENT_USE_WPATH
+
 }
 
 #endif
-
