@@ -34,14 +34,34 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_ESCAPE_STRING_HPP_INCLUDED
 
 #include <string>
+#include <limits>
 #include <boost/optional.hpp>
+#include <boost/array.hpp>
 #include "libtorrent/config.hpp"
+#include "libtorrent/size_type.hpp"
+#include "libtorrent/error_code.hpp"
 
 namespace libtorrent
 {
-	std::string TORRENT_EXPORT unescape_string(std::string const& s);
-	std::string TORRENT_EXPORT escape_string(const char* str, int len);
-	std::string TORRENT_EXPORT escape_path(const char* str, int len);
+	TORRENT_EXPORT boost::array<char, 3 + std::numeric_limits<size_type>::digits10> to_string(size_type n);
+	TORRENT_EXPORT bool is_digit(char c);
+	TORRENT_EXPORT bool is_print(char c);
+	TORRENT_EXPORT bool is_space(char c);
+	TORRENT_EXPORT char to_lower(char c);
+
+	TORRENT_EXPORT bool string_begins_no_case(char const* s1, char const* s2);
+	TORRENT_EXPORT bool string_equal_no_case(char const* s1, char const* s2);
+
+	TORRENT_EXPORT std::string unescape_string(std::string const& s, error_code& ec);
+	// replaces all disallowed URL characters by their %-encoding
+	TORRENT_EXPORT std::string escape_string(const char* str, int len);
+	// same as escape_string but does not encode '/'
+	TORRENT_EXPORT std::string escape_path(const char* str, int len);
+	// if the url does not appear to be encoded, and it contains illegal url characters
+	// it will be encoded
+	TORRENT_EXPORT std::string maybe_url_encode(std::string const& url);
+
+	TORRENT_EXPORT bool need_encoding(char const* str, int len);
 
 	// encodes a string using the base64 scheme
 	TORRENT_EXPORT std::string base64encode(std::string const& s);
@@ -50,9 +70,25 @@ namespace libtorrent
 	TORRENT_EXPORT std::string base32decode(std::string const& s);
 
 	TORRENT_EXPORT boost::optional<std::string> url_has_argument(
-		std::string const& url, std::string argument);
+		std::string const& url, std::string argument, int* out_pos = 0);
 
+	TORRENT_EXPORT std::string read_until(char const*& str, char delim, char const* end);
 	TORRENT_EXPORT std::string to_hex(std::string const& s);
+	TORRENT_EXPORT bool is_hex(char const *in, int len);
+	TORRENT_EXPORT void to_hex(char const *in, int len, char* out);
+	TORRENT_EXPORT bool from_hex(char const *in, int len, char* out);
+
+#if TORRENT_USE_WPATH
+	TORRENT_EXPORT std::wstring convert_to_wstring(std::string const& s);
+#endif
+	
+#if defined TORRENT_WINDOWS || TORRENT_USE_LOCALE_FILENAMES
+	TORRENT_EXPORT std::string convert_to_native(std::string const& s);
+#else
+	inline std::string const& convert_to_native(std::string const& s) { return s; }
+#endif		
+	
 }
 
 #endif // TORRENT_ESCAPE_STRING_HPP_INCLUDED
+
