@@ -63,6 +63,7 @@ namespace libtorrent
 
 	struct TORRENT_EXPORT piece_block
 	{
+		piece_block() {}
 		piece_block(int p_index, int b_index)
 			: piece_index(p_index)
 			, block_index(b_index)
@@ -130,7 +131,11 @@ namespace libtorrent
 			// always pick partial pieces before any other piece
 			prioritize_partials = 8,
 			// pick pieces in sequential order
-			sequential = 16
+			sequential = 16,
+			// have affinity to pieces with the same speed category
+			speed_affinity = 32,
+			// ignore the prefer_whole_pieces parameter
+			ignore_whole_pieces = 64
 		};
 
 		struct downloading_piece
@@ -184,6 +189,7 @@ namespace libtorrent
 
 		int cursor() const { return m_cursor; }
 		int reverse_cursor() const { return m_reverse_cursor; }
+		int sparse_regions() const { return m_sparse_regions; }
 
 		// sets all pieces to dont-have
 		void init(int blocks_per_piece, int blocks_in_last_piece, int total_num_pieces);
@@ -318,7 +324,7 @@ namespace libtorrent
 		void verify_pick(std::vector<piece_block> const& picked
 			, bitfield const& bits) const;
 #endif
-#if defined TORRENT_PICKER_LOG || defined TORRENT_DEBUG
+#if defined TORRENT_PICKER_LOG
 		void print_pieces() const;
 #endif
 
@@ -334,7 +340,7 @@ namespace libtorrent
 		int blocks_in_last_piece() const
 		{ return m_blocks_in_last_piece; }
 
-		float distributed_copies() const;
+		std::pair<int, int> distributed_copies() const;
 
 	private:
 
@@ -419,15 +425,15 @@ namespace libtorrent
 
 				// prio 4,5,6 halves the availability of a piece
 				int availability = peer_count;
-				int priority = piece_priority;
+				int p = piece_priority;
 				if (piece_priority >= priority_levels / 2)
 				{
 					availability /= 2;
-					priority -= (priority_levels - 2) / 2;
+					p -= (priority_levels - 2) / 2;
 				}
 
 				if (downloading) return availability * prio_factor;
-				return availability * prio_factor + (priority_levels / 2) - priority;
+				return availability * prio_factor + (priority_levels / 2) - p;
 			}
 
 			bool operator!=(piece_pos p) const
@@ -526,6 +532,9 @@ namespace libtorrent
 		// all the subsequent pieces
 		int m_reverse_cursor;
 
+		// the number of regions of pieces we don't have.
+		int m_sparse_regions;
+
 		// if this is set to true, it means update_pieces()
 		// has to be called before accessing m_pieces.
 		mutable bool m_dirty;
