@@ -33,19 +33,14 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_SESSION_HPP_INCLUDED
 #define TORRENT_SESSION_HPP_INCLUDED
 
-#include <ctime>
 #include <algorithm>
 #include <vector>
-#include <set>
-#include <list>
-#include <deque>
 
 #ifdef _MSC_VER
 #pragma warning(push, 1)
 #endif
 
 #include <boost/limits.hpp>
-#include <boost/tuple/tuple.hpp>
 #include <boost/filesystem/path.hpp>
 #include <boost/thread.hpp>
 
@@ -56,15 +51,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 #include "libtorrent/torrent_handle.hpp"
 #include "libtorrent/entry.hpp"
-#include "libtorrent/alert.hpp"
 #include "libtorrent/session_status.hpp"
 #include "libtorrent/version.hpp"
 #include "libtorrent/fingerprint.hpp"
-#include "libtorrent/time.hpp"
 #include "libtorrent/disk_io_thread.hpp"
 #include "libtorrent/peer_id.hpp"
+#include "libtorrent/alert.hpp"
 
 #include "libtorrent/storage.hpp"
+#include <boost/preprocessor/cat.hpp>
+
 
 #ifdef _MSC_VER
 #	include <eh.h>
@@ -79,9 +75,52 @@ namespace libtorrent
 	class connection_queue;
 	class natpmp;
 	class upnp;
+	class alert;
+
+	// this is used to create linker errors when trying to link to
+	// a library with a conflicting build configuration than the application
+#ifdef TORRENT_DEBUG
+#define G _release
+#else
+#define G _debug
+#endif
+
+#ifdef TORRENT_USE_OPENSSL
+#define S _ssl
+#else
+#define S _nossl
+#endif
+
+#ifdef TORRENT_DISABLE_DHT
+#define D _nodht
+#else
+#define D _dht
+#endif
+
+#ifdef TORRENT_DISABLE_POOL_ALLOCATOR
+#define P _nopoolalloc
+#else
+#define P _poolalloc
+#endif
+
+#define TORRENT_LINK_TEST_PREFIX libtorrent_build_config
+#define TORRENT_LINK_TEST_NAME BOOST_PP_CAT(TORRENT_LINK_TEST_PREFIX, BOOST_PP_CAT(P, BOOST_PP_CAT(D, BOOST_PP_CAT(S, G))))
+#undef P
+#undef D
+#undef S
+#undef G
+
+	inline void test_link()
+	{
+		extern void TORRENT_LINK_TEST_NAME();
+		TORRENT_LINK_TEST_NAME();
+	}
 
 	namespace fs = boost::filesystem;
 
+	session_settings min_memory_usage();
+	session_settings high_performance_seed();
+
 	namespace aux
 	{
 		// workaround for microsofts
@@ -133,6 +172,9 @@ namespace libtorrent
 			, duplicate_is_error(false)
 			, storage(sc)
 			, userdata(0)
+			, seed_mode(false)
+			, override_resume_data(false)
+			, upload_mode(false)
 		{}
 
 		boost::intrusive_ptr<torrent_info> ti;
@@ -147,6 +189,9 @@ namespace libtorrent
 		bool duplicate_is_error;
 		storage_constructor_type storage;
 		void* userdata;
+		bool seed_mode;
+		bool override_resume_data;
+		bool upload_mode;
 	};
 	
 	class TORRENT_EXPORT session: public boost::noncopyable, aux::eh_initializer
@@ -156,6 +201,7 @@ namespace libtorrent
 		session(fingerprint const& print = fingerprint("LT"
 			, LIBTORRENT_VERSION_MAJOR, LIBTORRENT_VERSION_MINOR, 0, 0)
 			, int flags = start_default_features | add_default_plugins
+			, int alert_mask = alert::error_notification
 #if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING || defined TORRENT_ERROR_LOGGING
 			, fs::path logpath = "."
 #endif
@@ -165,6 +211,7 @@ namespace libtorrent
 			, std::pair<int, int> listen_port_range
 			, char const* listen_interface = "0.0.0.0"
 			, int flags = start_default_features | add_default_plugins
+			, int alert_mask = alert::error_notification
 #if defined TORRENT_VERBOSE_LOGGING || defined TORRENT_LOGGING || defined TORRENT_ERROR_LOGGING
 			, fs::path logpath = "."
 #endif
@@ -172,17 +219,38 @@ namespace libtorrent
 			
 		~session();
 
+		enum save_state_flags_t
+		{
+			save_settings = 0x001,
+			save_dht_settings = 0x002,
+			save_dht_proxy = 0x004,
+			save_dht_state = 0x008,
+			save_i2p_proxy = 0x010,
+			save_encryption_settings = 0x020,
+			save_peer_proxy = 0x040,
+			save_web_proxy = 0x080,
+			save_tracker_proxy = 0x100,
+			save_as_map = 0x200,
+		};
+		void save_state(entry& e, boost::uint32_t flags = 0xffffffff) const;
+		void load_state(lazy_entry const& e);
+
 		// returns a list of all torrents in this session
 		std::vector<torrent_handle> get_torrents() const;
 		
+		io_service& get_io_service();
+
 		// returns an invalid handle in case the torrent doesn't exist
 		torrent_handle find_torrent(sha1_hash const& info_hash) const;
 
 		// all torrent_handles must be destructed before the session is destructed!
 		torrent_handle add_torrent(add_torrent_params const& params);
+		torrent_handle add_torrent(add_torrent_params const& params, error_code& ec);
 		
+#ifndef BOOST_NO_EXCEPTIONS
 #ifndef TORRENT_NO_DEPRECATE
 		// deprecated in 0.14
+		TORRENT_DEPRECATED_PREFIX
 		torrent_handle add_torrent(
 			torrent_info const& ti
 			, fs::path const& save_path
@@ -192,6 +260,7 @@ namespace libtorrent
 			, storage_constructor_type sc = default_storage_constructor) TORRENT_DEPRECATED;
 
 		// deprecated in 0.14
+		TORRENT_DEPRECATED_PREFIX
 		torrent_handle add_torrent(
 			boost::intrusive_ptr<torrent_info> ti
 			, fs::path const& save_path
@@ -202,6 +271,7 @@ namespace libtorrent
 			, void* userdata = 0) TORRENT_DEPRECATED;
 
 		// deprecated in 0.14
+		TORRENT_DEPRECATED_PREFIX
 		torrent_handle add_torrent(
 			char const* tracker_url
 			, sha1_hash const& info_hash
@@ -212,6 +282,7 @@ namespace libtorrent
 			, bool paused = false
 			, storage_constructor_type sc = default_storage_constructor
 			, void* userdata = 0) TORRENT_DEPRECATED;
+#endif
 #endif
 
 		session_proxy abort() { return session_proxy(m_impl); }
@@ -227,12 +298,20 @@ namespace libtorrent
 			, std::vector<cached_piece_info>& ret) const;
 
 #ifndef TORRENT_DISABLE_DHT
-		void start_dht(entry const& startup_state = entry());
+		void start_dht();
 		void stop_dht();
 		void set_dht_settings(dht_settings const& settings);
-		entry dht_state() const;
+#ifndef TORRENT_NO_DEPRECATE
+		// deprecated in 0.15
+		// use save_state and load_state instead
+		TORRENT_DEPRECATED_PREFIX
+		entry dht_state() const TORRENT_DEPRECATED;
+		TORRENT_DEPRECATED_PREFIX
+		void start_dht(entry const& startup_state) TORRENT_DEPRECATED;
+#endif
 		void add_dht_node(std::pair<std::string, int> const& node);
 		void add_dht_router(std::pair<std::string, int> const& node);
+		bool is_dht_running() const;
 #endif
 
 #ifndef TORRENT_DISABLE_ENCRYPTION
@@ -248,10 +327,20 @@ namespace libtorrent
 		int as_for_ip(address const& addr);
 		bool load_asnum_db(char const* file);
 		bool load_country_db(char const* file);
+#ifndef BOOST_FILESYSTEM_NARROW_ONLY
+		bool load_country_db(wchar_t const* file);
+		bool load_asnum_db(wchar_t const* file);
+#endif
 #endif
 
-		void load_state(entry const& ses_state);
-		entry state() const;
+#ifndef TORRENT_NO_DEPRECATE
+		// deprecated in 0.15
+		// use load_state and save_state instead
+		TORRENT_DEPRECATED_PREFIX
+		void load_state(entry const& ses_state) TORRENT_DEPRECATED;
+		TORRENT_DEPRECATED_PREFIX
+		entry state() const TORRENT_DEPRECATED;
+#endif
 
 		void set_ip_filter(ip_filter const& f);
 		ip_filter const& get_ip_filter() const;
@@ -320,22 +409,31 @@ namespace libtorrent
 
 		int upload_rate_limit() const;
 		int download_rate_limit() const;
+		int local_upload_rate_limit() const;
+		int local_download_rate_limit() const;
 		int max_half_open_connections() const;
 
+		void set_local_upload_rate_limit(int bytes_per_second);
+		void set_local_download_rate_limit(int bytes_per_second);
 		void set_upload_rate_limit(int bytes_per_second);
 		void set_download_rate_limit(int bytes_per_second);
 		void set_max_uploads(int limit);
 		void set_max_connections(int limit);
 		void set_max_half_open_connections(int limit);
 
+		int max_connections() const;
+		int max_uploads() const;
+
 		std::auto_ptr<alert> pop_alert();
 #ifndef TORRENT_NO_DEPRECATE
+		TORRENT_DEPRECATED_PREFIX
 		void set_severity_level(alert::severity_t s) TORRENT_DEPRECATED;
 #endif
 		void set_alert_mask(int m);
 		size_t set_alert_queue_size_limit(size_t queue_size_limit_);
 
 		alert const* wait_for_alert(time_duration max_wait);
+		void set_alert_dispatch(boost::function<void(alert const&)> const& fun);
 
 		connection_queue& get_connection_queue();
 
