@@ -35,7 +35,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <string>
 #include <vector>
-#include <iosfwd>
+//#include <iosfwd>
 
 #ifdef _MSC_VER
 #pragma warning(push, 1)
@@ -66,18 +66,116 @@ namespace libtorrent
 	namespace gr = boost::gregorian;
 	namespace fs = boost::filesystem;
 
+	enum
+	{
+		// wait 60 seconds before retrying a failed tracker
+		tracker_retry_delay_min = 10
+		// when tracker_failed_max trackers
+		// has failed, wait 60 minutes instead
+		, tracker_retry_delay_max = 60 * 60
+	};
+
 	struct TORRENT_EXPORT announce_entry
 	{
-		announce_entry(std::string const& u): url(u), tier(0) {}
+		announce_entry(std::string const& u)
+			: url(u)
+			, tier(0)
+			, fail_limit(3)
+			, fails(0)
+			, source(0)
+			, verified(false)
+			, updating(false)
+			, start_sent(false)
+			, complete_sent(false)
+			, send_stats(true)
+		{}
+
 		std::string url;
-		int tier;
+
+		int next_announce_in() const;
+		int min_announce_in() const;
+
+		// the time of next tracker announce
+		ptime next_announce;
+
+		// no announces before this time
+		ptime min_announce;
+
+		boost::uint8_t tier;
+		// the number of times this tracker can fail
+		// in a row before it's removed. 0 means unlimited
+		boost::uint8_t fail_limit;
+
+		// the number of times in a row this tracker has failed
+		boost::uint8_t fails;
+
+		enum tracker_source
+		{
+			source_torrent = 1,
+			source_client = 2,
+			source_magnet_link = 4,
+			source_tex = 8
+		};
+		// where did we get this tracker from
+		boost::uint8_t source;
+
+		// is set to true if we have ever received a response from
+		// this tracker
+		bool verified:1;
+
+		// true if we're currently trying to announce with 
+		// this tracker
+		bool updating:1;
+
+		// this is true if event start has been sent to the tracker
+		bool start_sent:1;
+
+		// this is true if event completed has been sent to the tracker
+		bool complete_sent:1;
+
+		// this is false the stats sent to this tracker will be 0
+		bool send_stats:1;
+
+		void reset()
+		{
+			start_sent = false;
+			next_announce = min_time();
+			min_announce = min_time();
+		}
+
+		void failed(int retry_interval = 0)
+		{
+			++fails;
+			int delay = (std::min)(tracker_retry_delay_min + int(fails) * int(fails)
+				* tracker_retry_delay_min, int(tracker_retry_delay_max));
+			delay = (std::max)(delay, retry_interval);
+			next_announce = time_now() + seconds(delay);
+			updating = false;
+		}
+
+		bool can_announce(ptime now) const
+		{
+			return now >= next_announce
+				&& now >= min_announce
+				&& (fails < fail_limit || fail_limit == 0)
+				&& !updating;
+		}
+
+		bool is_working() const
+		{
+			return fails == 0;
+		}
+
+		void trim()
+		{
+			while (!url.empty() && is_space(url[0]))
+				url.erase(url.begin());
+		}
 	};
 
 #ifndef BOOST_NO_EXCEPTIONS
-	struct TORRENT_EXPORT invalid_torrent_file: std::exception
-	{
-		virtual const char* what() const throw() { return "invalid torrent file"; }
-	};
+	// for backwards compatibility with 0.14
+	typedef libtorrent_exception invalid_torrent_file;
 #endif
 
 	int TORRENT_EXPORT load_file(fs::path const& filename, std::vector<char>& v);
@@ -85,15 +183,44 @@ namespace libtorrent
 	class TORRENT_EXPORT torrent_info : public intrusive_ptr_base<torrent_info>
 	{
 	public:
-
-		torrent_info(sha1_hash const& info_hash);
+		torrent_info(torrent_info const& t);
+#ifndef BOOST_NO_EXCEPTIONS
 		torrent_info(lazy_entry const& torrent_file);
 		torrent_info(char const* buffer, int size);
 		torrent_info(fs::path const& filename);
+#ifndef BOOST_FILESYSTEM_NARROW_ONLY
+		torrent_info(fs::wpath const& filename);
+#endif
+#endif
+
+		torrent_info(sha1_hash const& info_hash);
+		torrent_info(lazy_entry const& torrent_file, error_code& ec);
+		torrent_info(char const* buffer, int size, error_code& ec);
+		torrent_info(fs::path const& filename, error_code& ec);
+#ifndef BOOST_FILESYSTEM_NARROW_ONLY
+		torrent_info(fs::wpath const& filename, error_code& ec);
+#endif
+
 		~torrent_info();
 
 		file_storage const& files() const { return m_files; }
-		file_storage& files() { return m_files; }
+		file_storage const& orig_files() const { return m_orig_files ? *m_orig_files : m_files; }
+
+		void rename_file(int index, std::string const& new_filename)
+		{
+			copy_on_write();
+			m_files.rename_file(index, new_filename);
+		}
+
+#ifndef BOOST_FILESYSTEM_NARROW_ONLY
+		void rename_file(int index, std::wstring const& new_filename)
+		{
+			copy_on_write();
+			m_files.rename_file(index, new_filename);
+		}
+#endif
+
+		void remap_files(file_storage const& f);
 
 		void add_tracker(std::string const& url, int tier = 0);
 		std::vector<announce_entry> const& trackers() const { return m_urls; }
@@ -103,6 +230,11 @@ namespace libtorrent
 		void add_url_seed(std::string const& url)
 		{ m_url_seeds.push_back(url); }
 
+		std::vector<std::string> const& http_seeds() const
+		{ return m_http_seeds; }
+		void add_http_seed(std::string const& url)
+		{ m_http_seeds.push_back(url); }
+
 		size_type total_size() const { return m_files.total_size(); }
 		int piece_length() const { return m_files.piece_length(); }
 		int num_pieces() const { return m_files.num_pieces(); }
@@ -129,7 +261,9 @@ namespace libtorrent
 #ifndef TORRENT_NO_DEPRECATE
 // ------- start deprecation -------
 // these functions will be removed in a future version
+		TORRENT_DEPRECATED_PREFIX
 		torrent_info(entry const& torrent_file) TORRENT_DEPRECATED;
+		TORRENT_DEPRECATED_PREFIX
 		void print(std::ostream& os) const TORRENT_DEPRECATED;
 // ------- end deprecation -------
 #endif
@@ -143,14 +277,27 @@ namespace libtorrent
 		sha1_hash hash_for_piece(int index) const
 		{ return sha1_hash(hash_for_piece_ptr(index)); }
 
+		std::vector<sha1_hash> const& merkle_tree() const { return m_merkle_tree; }
+		void set_merkle_tree(std::vector<sha1_hash>& h)
+		{ TORRENT_ASSERT(h.size() == m_merkle_tree.size() ); m_merkle_tree.swap(h); }
+
 		char const* hash_for_piece_ptr(int index) const
 		{
 			TORRENT_ASSERT(index >= 0);
 			TORRENT_ASSERT(index < m_files.num_pieces());
-			TORRENT_ASSERT(m_piece_hashes);
-			TORRENT_ASSERT(m_piece_hashes >= m_info_section.get());
-			TORRENT_ASSERT(m_piece_hashes < m_info_section.get() + m_info_section_size);
-			return &m_piece_hashes[index*20];
+			if (is_merkle_torrent())
+			{
+				TORRENT_ASSERT(index < int(m_merkle_tree.size()) - m_merkle_first_leaf);
+				return (const char*)&m_merkle_tree[m_merkle_first_leaf + index][0];
+			}
+			else
+			{
+				TORRENT_ASSERT(m_piece_hashes);
+				TORRENT_ASSERT(m_piece_hashes >= m_info_section.get());
+				TORRENT_ASSERT(m_piece_hashes < m_info_section.get() + m_info_section_size);
+				TORRENT_ASSERT(index < m_info_section_size / 20);
+				return &m_piece_hashes[index*20];
+			}
 		}
 
 		boost::optional<pt::ptime> creation_date() const;
@@ -169,7 +316,7 @@ namespace libtorrent
 		void add_node(std::pair<std::string, int> const& node)
 		{ m_nodes.push_back(node); }
 		
-		bool parse_info_section(lazy_entry const& e, std::string& error);
+		bool parse_info_section(lazy_entry const& e, error_code& ex);
 
 		lazy_entry const* info(char const* key) const
 		{
@@ -186,15 +333,30 @@ namespace libtorrent
 
 		int metadata_size() const { return m_info_section_size; }
 
+		bool add_merkle_nodes(std::map<int, sha1_hash> const& subtree
+			, int piece);
+		std::map<int, sha1_hash> build_merkle_list(int piece) const;
+		bool is_merkle_torrent() const { return !m_merkle_tree.empty(); }
+
 	private:
 
-		bool parse_torrent_file(lazy_entry const& libtorrent, std::string& error);
+		// not assignable
+		torrent_info const& operator=(torrent_info const&);
+
+		void copy_on_write();
+		bool parse_torrent_file(lazy_entry const& libtorrent, error_code& ec);
 
 		file_storage m_files;
 
+		// if m_files is modified, it is first copied into
+		// m_orig_files so that the original name and
+		// filenames are preserved.
+		boost::shared_ptr<const file_storage> m_orig_files;
+
 		// the urls to the trackers
 		std::vector<announce_entry> m_urls;
 		std::vector<std::string> m_url_seeds;
+		std::vector<std::string> m_http_seeds;
 		nodes_t m_nodes;
 
 		// the hash that identifies this torrent
@@ -234,6 +396,14 @@ namespace libtorrent
 		// pointing to the first byte of the first sha-1 hash
 		char const* m_piece_hashes;
 
+		// if this is a merkle torrent, this is the merkle
+		// tree. It has space for merkle_num_nodes(merkle_num_leafs(num_pieces))
+		// hashes
+		std::vector<sha1_hash> m_merkle_tree;
+		// the index to the first leaf. This is where the hash for the
+		// first piece is stored
+		int m_merkle_first_leaf;
+
 		// the info section parsed. points into m_info_section
 		// parsed lazily
 		mutable lazy_entry m_info_dict;
