@@ -34,8 +34,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_UDP_SOCKET_HPP_INCLUDED
 
 #include "libtorrent/socket.hpp"
-#include "libtorrent/buffer.hpp"
 #include "libtorrent/session_settings.hpp"
+#include "libtorrent/buffer.hpp"
 
 #include <vector>
 #include <boost/function.hpp>
@@ -54,7 +54,14 @@ namespace libtorrent
 		udp_socket(io_service& ios, callback_t const& c, connection_queue& cc);
 		~udp_socket();
 
-		bool is_open() const { return m_ipv4_sock.is_open() || m_ipv6_sock.is_open(); }
+		bool is_open() const
+		{
+			return m_ipv4_sock.is_open()
+#if TORRENT_USE_IPV6
+				|| m_ipv6_sock.is_open()
+#endif
+				;
+		}
 		io_service& get_io_service() { return m_ipv4_sock.get_io_service(); }
 
 		void send(udp::endpoint const& ep, char const* p, int len, error_code& ec);
@@ -66,6 +73,8 @@ namespace libtorrent
 		void set_proxy_settings(proxy_settings const& ps);
 		proxy_settings const& get_proxy_settings() { return m_proxy_settings; }
 
+		bool is_closed() const { return m_abort; }
+
 	protected:
 
 		struct queued_packet
@@ -99,11 +108,15 @@ namespace libtorrent
 		mutable mutex_t m_mutex;
 
 		udp::socket m_ipv4_sock;
-		udp::socket m_ipv6_sock;
 		udp::endpoint m_v4_ep;
-		udp::endpoint m_v6_ep;
 		char m_v4_buf[1600];
+
+#if TORRENT_USE_IPV6
+		udp::socket m_ipv6_sock;
+		udp::endpoint m_v6_ep;
 		char m_v6_buf[1600];
+#endif
+
 		int m_bind_port;
 		char m_outstanding;
 
@@ -124,8 +137,28 @@ namespace libtorrent
 #ifdef TORRENT_DEBUG
 		bool m_started;
 		int m_magic;
+		int m_outstanding_when_aborted;
 #endif
 	};
+
+	struct rate_limited_udp_socket : public udp_socket
+	{
+		rate_limited_udp_socket(io_service& ios, callback_t const& c, connection_queue& cc);
+		void set_rate_limit(int limit) { m_rate_limit = limit; }
+		bool can_send() const { return int(m_queue.size()) >= m_queue_size_limit; }
+		bool send(udp::endpoint const& ep, char const* p, int len, error_code& ec, int flags = 0);
+		void close();
+
+	private:
+		void on_tick(error_code const& e);
+
+		deadline_timer m_timer;
+		int m_queue_size_limit;
+		int m_rate_limit;
+		int m_quota;
+		ptime m_last_tick;
+		std::list<queued_packet> m_queue;
+	};
 }
 
 #endif
