@@ -57,11 +57,11 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 	std::string auth;
 	std::string hostname;
 	std::string path;
-	char const* error;
+	error_code ec;
 	int port;
 
-	boost::tie(protocol, auth, hostname, port, path, error)
-		= parse_url_components(url);
+	boost::tie(protocol, auth, hostname, port, path)
+		= parse_url_components(url, ec);
 
 	int default_port = protocol == "https" ? 443 : 80;
 
@@ -69,9 +69,22 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 	// deletes this object
 	boost::shared_ptr<http_connection> me(shared_from_this());
 
-	if (error)
+	if (protocol != "http"
+#ifdef TORRENT_USE_OPENSSL
+		&& protocol != "https"
+#endif
+		)
 	{
-		callback(asio::error::socket_type_not_supported);
+		error_code ec(errors::unsupported_url_protocol);
+		m_resolver.get_io_service().post(boost::bind(&http_connection::callback
+			, this, ec, (char*)0, 0));
+		return;
+	}
+
+	if (ec)
+	{
+		m_resolver.get_io_service().post(boost::bind(&http_connection::callback
+			, this, ec, (char*)0, 0));
 		return;
 	}
 
@@ -79,48 +92,48 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 
 	bool ssl = false;
 	if (protocol == "https") ssl = true;
-#ifndef TORRENT_USE_OPENSSL
-	if (ssl)
-	{
-		callback(asio::error::socket_type_not_supported);
-		return;
-	}
-#endif
 	
-	std::stringstream headers;
+	char request[2048];
+	char* end = request + sizeof(request);
+	char* ptr = request;
+
+#define APPEND_FMT(fmt) ptr += snprintf(ptr, end - ptr, fmt)
+#define APPEND_FMT1(fmt, arg) ptr += snprintf(ptr, end - ptr, fmt, arg)
+#define APPEND_FMT2(fmt, arg1, arg2) ptr += snprintf(ptr, end - ptr, fmt, arg1, arg2)
+
 	if (ps && (ps->type == proxy_settings::http
 		|| ps->type == proxy_settings::http_pw)
 		&& !ssl)
 	{
 		// if we're using an http proxy and not an ssl
 		// connection, just do a regular http proxy request
-		headers << "GET " << url << " HTTP/1.0\r\n";
+		APPEND_FMT1("GET %s HTTP/1.0\r\n", url.c_str());
 		if (ps->type == proxy_settings::http_pw)
-			headers << "Proxy-Authorization: Basic " << base64encode(
-				ps->username + ":" + ps->password) << "\r\n";
+			APPEND_FMT1("Proxy-Authorization: Basic %s\r\n", base64encode(
+				ps->username + ":" + ps->password).c_str());
 		hostname = ps->hostname;
 		port = ps->port;
 	}
 	else
 	{
-		headers << "GET " << path << " HTTP/1.0\r\n"
-			"Host: " << hostname;
-		if (port != default_port) headers << ":" << to_string(port).elems;
-		headers << "\r\n";
+		APPEND_FMT2("GET %s HTTP/1.0\r\n"
+			"Host: %s", path.c_str(), hostname.c_str());
+		if (port != default_port) APPEND_FMT1(":%d\r\n", port);
+		else APPEND_FMT("\r\n");
 	}
 
 	if (!auth.empty())
-		headers << "Authorization: Basic " << base64encode(auth) << "\r\n";
+		APPEND_FMT1("Authorization: Basic %s\r\n", base64encode(auth).c_str());
 
 	if (!user_agent.empty())
-		headers << "User-Agent: " << user_agent << "\r\n";
+		APPEND_FMT1("User-Agent: %s\r\n", user_agent.c_str());
 	
-	headers <<
-		"Connection: close\r\n"
-		"Accept-Encoding: gzip\r\n"
-		"\r\n";
+	if (m_bottled)
+		APPEND_FMT("Accept-Encoding: gzip\r\n");
 
-	sendbuffer = headers.str();
+	APPEND_FMT("Connection: close\r\n\r\n");
+
+	sendbuffer.assign(request);
 	m_url = url;
 	start(hostname, to_string(port).elems, timeout, prio
 		, ps, ssl, handle_redirects, bind_addr);
@@ -152,7 +165,8 @@ void http_connection::start(std::string const& hostname, std::string const& port
 
 	if (ec)
 	{
-		callback(ec);
+		m_resolver.get_io_service().post(boost::bind(&http_connection::callback
+			, this, ec, (char*)0, 0));
 		return;
 	}
 
@@ -185,6 +199,7 @@ void http_connection::start(std::string const& hostname, std::string const& port
 		{
 			m_sock.instantiate<ssl_stream<socket_type> >(m_resolver.get_io_service());
 			ssl_stream<socket_type>* s = m_sock.get<ssl_stream<socket_type> >();
+			TORRENT_ASSERT(s);
 			bool ret = instantiate_connection(m_resolver.get_io_service()
 				, ps ? *ps : null_proxy, s->next_layer());
 			TORRENT_ASSERT(ret);
@@ -208,7 +223,8 @@ void http_connection::start(std::string const& hostname, std::string const& port
 			m_sock.bind(tcp::endpoint(m_bind_addr, 0), ec);
 			if (ec)
 			{
-				callback(ec);
+				m_resolver.get_io_service().post(boost::bind(&http_connection::callback
+					, this, ec, (char*)0, 0));
 				return;
 			}
 		}
@@ -250,7 +266,7 @@ void http_connection::on_timeout(boost::weak_ptr<http_connection> p
 
 	if (e == asio::error::operation_aborted) return;
 
-	if (c->m_last_receive + c->m_timeout < time_now())
+	if (c->m_last_receive + c->m_timeout < time_now_hires())
 	{
 		if (c->m_connection_ticket > -1 && !c->m_endpoints.empty())
 		{
@@ -302,6 +318,13 @@ void http_connection::on_resolve(error_code const& e
 	std::transform(i, tcp::resolver::iterator(), std::back_inserter(m_endpoints)
 		, boost::bind(&tcp::resolver::iterator::value_type::endpoint, _1));
 
+	if (m_filter_handler) m_filter_handler(*this, m_endpoints);
+	if (m_endpoints.empty())
+	{
+		close();
+		return;
+	}
+
 	// The following statement causes msvc to crash (ICE). Since it's not
 	// necessary in the vast majority of cases, just ignore the endpoint
 	// order for windows
@@ -309,8 +332,10 @@ void http_connection::on_resolve(error_code const& e
 	// sort the endpoints so that the ones with the same IP version as our
 	// bound listen socket are first. So that when contacting a tracker,
 	// we'll talk to it from the same IP that we're listening on
-	std::partition(m_endpoints.begin(), m_endpoints.end()
-		, boost::bind(&address::is_v4, boost::bind(&tcp::endpoint::address, _1)) == m_bind_addr.is_v4());
+	if (m_bind_addr != address_v4::any())
+		std::partition(m_endpoints.begin(), m_endpoints.end()
+			, boost::bind(&address::is_v4, boost::bind(&tcp::endpoint::address, _1))
+				== m_bind_addr.is_v4());
 #endif
 
 	queue_connect();
@@ -342,7 +367,7 @@ void http_connection::on_connect(error_code const& e)
 		m_connection_ticket = -1;
 	}
 
-	m_last_receive = time_now();
+	m_last_receive = time_now_hires();
 	if (!e)
 	{ 
 		if (m_connect_handler) m_connect_handler(*this);
@@ -366,30 +391,29 @@ void http_connection::on_connect(error_code const& e)
 
 void http_connection::callback(error_code const& e, char const* data, int size)
 {
-	if (!m_bottled || !m_called)
+	if (m_bottled && m_called) return;
+
+	std::vector<char> buf;
+	if (m_bottled && m_parser.header_finished())
 	{
-		std::vector<char> buf;
-		if (m_bottled && m_parser.header_finished())
+		std::string const& encoding = m_parser.header("content-encoding");
+		if ((encoding == "gzip" || encoding == "x-gzip") && size > 0 && data)
 		{
-			std::string const& encoding = m_parser.header("content-encoding");
-			if ((encoding == "gzip" || encoding == "x-gzip") && size > 0 && data)
+			std::string error;
+			if (inflate_gzip(data, size, buf, max_bottled_buffer, error))
 			{
-				std::string error;
-				if (inflate_gzip(data, size, buf, max_bottled_buffer, error))
-				{
-					if (m_handler) m_handler(asio::error::fault, m_parser, data, size, *this);
-					close();
-					return;
-				}
-				size = int(buf.size());
-				data = size == 0 ? 0 : &buf[0];
+				if (m_handler) m_handler(errors::http_failed_decompress, m_parser, data, size, *this);
+				close();
+				return;
 			}
+			size = int(buf.size());
+			data = size == 0 ? 0 : &buf[0];
 		}
-		m_called = true;
-		error_code ec;
-		m_timer.cancel(ec);
-		if (m_handler) m_handler(e, m_parser, data, size, *this);
 	}
+	m_called = true;
+	error_code ec;
+	m_timer.cancel(ec);
+	if (m_handler) m_handler(e, m_parser, data, size, *this);
 }
 
 void http_connection::on_write(error_code const& e)
@@ -473,7 +497,7 @@ void http_connection::on_read(error_code const& e
 		if (error)
 		{
 			// HTTP parse error
-			error_code ec = asio::error::fault;
+			error_code ec = errors::http_parse_error;
 			callback(ec, 0, 0);
 			return;
 		}
@@ -490,7 +514,7 @@ void http_connection::on_read(error_code const& e
 				if (location.empty())
 				{
 					// missing location header
-					callback(asio::error::fault);
+					callback(error_code(errors::http_missing_location));
 					close();
 					return;
 				}
@@ -498,10 +522,9 @@ void http_connection::on_read(error_code const& e
 				error_code ec;
 				m_sock.close(ec);
 				using boost::tuples::ignore;
-				char const* error;
-				boost::tie(ignore, ignore, ignore, ignore, ignore, error)
-					= parse_url_components(location);
-				if (error == 0)
+				boost::tie(ignore, ignore, ignore, ignore, ignore)
+					= parse_url_components(location, ec);
+				if (!ec)
 				{
 					get(location, m_timeout, m_priority, &m_proxy, m_redirects - 1);
 				}
@@ -533,7 +556,7 @@ void http_connection::on_read(error_code const& e
 				callback(e, &m_recvbuffer[0] + m_parser.body_start()
 					, m_read_pos - m_parser.body_start());
 			m_read_pos = 0;
-			m_last_receive = time_now();
+			m_last_receive = time_now_hires();
 		}
 		else if (m_bottled && m_parser.finished())
 		{
@@ -547,7 +570,7 @@ void http_connection::on_read(error_code const& e
 		TORRENT_ASSERT(!m_bottled);
 		callback(e, &m_recvbuffer[0], m_read_pos);
 		m_read_pos = 0;
-		m_last_receive = time_now();
+		m_last_receive = time_now_hires();
 	}
 
 	if (int(m_recvbuffer.size()) == m_read_pos)
