@@ -39,20 +39,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 #include "libtorrent/http_parser.hpp"
 #include "libtorrent/assert.hpp"
+#include "libtorrent/escape_string.hpp"
 
 using namespace libtorrent;
 
-namespace
-{
-	char to_lower(char c) { return std::tolower(c); }
-}
-
 namespace libtorrent
 {
 	http_parser::http_parser()
 		: m_recv_pos(0)
 		, m_status_code(-1)
 		, m_content_length(-1)
+		, m_range_start(-1)
+		, m_range_end(-1)
 		, m_state(read_status)
 		, m_recv_buffer(0, 0)
 		, m_body_start_pos(0)
@@ -98,18 +96,18 @@ namespace libtorrent
 			char const* line_end = newline;
 			if (pos != line_end && *(line_end - 1) == '\r') --line_end;
 
-			std::istringstream line(std::string(pos, line_end));
+			char const* line = pos;
 			++newline;
-			int incoming = (int)std::distance(pos, newline);
+			int incoming = int(newline - pos);
 			m_recv_pos += incoming;
 			boost::get<1>(ret) += newline - (m_recv_buffer.begin + start_pos);
 			pos = newline;
 
-			line >> m_protocol;
+			m_protocol = read_until(line, ' ', line_end);
 			if (m_protocol.substr(0, 5) == "HTTP/")
 			{
-				line >> m_status_code;
-				std::getline(line, m_server_message);
+				m_status_code = atoi(read_until(line, ' ', line_end).c_str());
+				m_server_message = read_until(line, '\r', line_end);
 			}
 			else
 			{
@@ -118,7 +116,8 @@ namespace libtorrent
 				// the content length is assumed to be 0 for requests
 				m_content_length = 0;
 				m_protocol.clear();
-				line >> m_path >> m_protocol;
+				m_path = read_until(line, ' ', line_end);
+				m_protocol = read_until(line, ' ', line_end);
 				m_status_code = 0;
 			}
 			m_state = read_header;
@@ -149,6 +148,9 @@ namespace libtorrent
 					// the header is finished and the body
 					// starts.
 					m_state = read_body;
+					// if this is a request (not a response)
+					// we're done once we reach the end of the headers
+					if (!m_method.empty()) m_finished = true;
 					m_body_start_pos = m_recv_pos;
 					break;
 				}
@@ -165,31 +167,37 @@ namespace libtorrent
 
 				if (name == "content-length")
 				{
-#ifdef TORRENT_WINDOWS
-					m_content_length = _atoi64(value.c_str());
-#else
-					m_content_length = atoll(value.c_str());
-#endif
+					m_content_length = strtoll(value.c_str(), 0, 10);
 				}
 				else if (name == "content-range")
 				{
-					std::stringstream range_str(value);
-					char dummy;
-					std::string bytes;
-					size_type range_start, range_end;
+					bool success = true;
+					char const* ptr = value.c_str();
+
 					// apparently some web servers do not send the "bytes"
-					// in their content-range
-					if (value.find(' ') != std::string::npos)
-						range_str >> bytes;
-					range_str >> range_start >> dummy >> range_end;
-					if (!range_str || range_end < range_start)
+					// in their content-range. Don't treat it as an error
+					// if we can't find it, just assume the byte counters
+					// start immediately
+					if (string_begins_no_case("bytes ", ptr)) ptr += 6;
+					char* end;
+					m_range_start = strtoll(ptr, &end, 10);
+					if (end == ptr) success = false;
+					else if (*end != '-') success = false;
+					else
+					{
+						ptr = end + 1;
+						m_range_end = strtoll(ptr, &end, 10);
+						if (end == ptr) success = false;
+					}
+
+					if (!success || m_range_end < m_range_start)
 					{
 						m_state = error_state;
 						error = true;
 						return ret;
 					}
 					// the http range is inclusive
-					m_content_length = range_end - range_start + 1;
+					m_content_length = m_range_end - m_range_start + 1;
 				}
 
 				TORRENT_ASSERT(m_recv_pos <= (int)recv_buffer.left());
@@ -236,6 +244,8 @@ namespace libtorrent
 		m_body_start_pos = 0;
 		m_status_code = -1;
 		m_content_length = -1;
+		m_range_start = -1;
+		m_range_end = -1;
 		m_finished = false;
 		m_state = read_status;
 		m_recv_buffer.begin = 0;
