@@ -36,6 +36,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <libtorrent/kademlia/routing_table.hpp>
 #include <libtorrent/kademlia/rpc_manager.hpp>
 #include <libtorrent/kademlia/logging.hpp>
+#include <libtorrent/kademlia/node.hpp>
 #include <libtorrent/kademlia/msg.hpp>
 
 #include <libtorrent/io.hpp>
@@ -65,7 +66,7 @@ void refresh_observer::reply(msg const& in)
 		for (msg::nodes_t::const_iterator i = in.nodes.begin()
 			, end(in.nodes.end()); i != end; ++i)
 		{
-			m_algorithm->traverse(i->id, i->addr);
+			m_algorithm->traverse(i->id, udp::endpoint(i->addr, i->port));
 		}
 	}
 	m_algorithm->finished(m_self);
@@ -101,20 +102,27 @@ void ping_observer::timeout()
 
 void refresh::invoke(node_id const& nid, udp::endpoint addr)
 {
-	TORRENT_ASSERT(m_rpc.allocation_size() >= sizeof(refresh_observer));
-	observer_ptr o(new (m_rpc.allocator().malloc()) refresh_observer(
-		this, nid, m_target));
+	TORRENT_ASSERT(m_node.m_rpc.allocation_size() >= sizeof(refresh_observer));
+	void* ptr = m_node.m_rpc.allocator().malloc();
+	if (ptr == 0)
+	{
+		done();
+		return;
+	}
+	m_node.m_rpc.allocator().set_next_size(10);
+	observer_ptr o(new (ptr) refresh_observer( this, nid));
 #ifdef TORRENT_DEBUG
 	o->m_in_constructor = false;
 #endif
 
-	m_rpc.invoke(messages::find_node, addr, o);
+	m_node.m_rpc.invoke(messages::find_node, addr, o);
 }
 
 void refresh::done()
 {
-	m_leftover_nodes_iterator = (int)m_results.size() > m_max_results ?
-		m_results.begin() + m_max_results : m_results.end();
+	int max_results = m_node.m_table.bucket_size();
+	m_leftover_nodes_iterator = (int)m_results.size() > max_results ?
+		m_results.begin() + max_results : m_results.end();
 
 	invoke_pings_or_finish();
 }
@@ -154,19 +162,25 @@ void refresh::invoke_pings_or_finish(bool prevent_request)
 				continue;
 			}
 
+#ifndef BOOST_NO_EXCEPTIONS
 			try
 			{
-				TORRENT_ASSERT(m_rpc.allocation_size() >= sizeof(ping_observer));
-				observer_ptr o(new (m_rpc.allocator().malloc()) ping_observer(
-					this, node.id));
+#endif
+				TORRENT_ASSERT(m_node.m_rpc.allocation_size() >= sizeof(ping_observer));
+				void* ptr = m_node.m_rpc.allocator().malloc();
+				if (ptr == 0) return;
+				m_node.m_rpc.allocator().set_next_size(10);
+				observer_ptr o(new (ptr) ping_observer(this, node.id));
 #ifdef TORRENT_DEBUG
 				o->m_in_constructor = false;
 #endif
-				m_rpc.invoke(messages::ping, node.addr, o);
+				m_node.m_rpc.invoke(messages::ping, node.addr, o);
 				++m_active_pings;
 				++m_leftover_nodes_iterator;
+#ifndef BOOST_NO_EXCEPTIONS
 			}
 			catch (std::exception& e) {}
+#endif
 		}
 	}
 
