@@ -39,7 +39,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/mpl/sizeof.hpp>
 #include <boost/mpl/transform_view.hpp>
 #include <boost/mpl/deref.hpp>
-#include <boost/lexical_cast.hpp>
 
 #include <libtorrent/io.hpp>
 #include <libtorrent/invariant_check.hpp>
@@ -82,7 +81,7 @@ void intrusive_ptr_release(observer const* o)
 	if (--o->m_refs == 0)
 	{
 		boost::pool<>& p = o->pool_allocator;
-		o->~observer();
+		(const_cast<observer*>(o))->~observer();
 		p.free(const_cast<observer*>(o));
 	}
 }
@@ -93,7 +92,6 @@ typedef mpl::vector<
 	closest_nodes_observer
 	, find_data_observer
 	, announce_observer
-	, get_peers_observer
 	, refresh_observer
 	, ping_observer
 	, null_observer
@@ -105,8 +103,8 @@ typedef mpl::max_element<
 
 rpc_manager::rpc_manager(fun const& f, node_id const& our_id
 	, routing_table& table, send_fun const& sf)
-	: m_pool_allocator(sizeof(mpl::deref<max_observer_type_iter::base>::type))
-	, m_next_transaction_id(rand() % max_transactions)
+	: m_pool_allocator(sizeof(mpl::deref<max_observer_type_iter::base>::type), 10)
+	, m_next_transaction_id(std::rand() % max_transactions)
 	, m_oldest_transaction_id(m_next_transaction_id)
 	, m_incoming(f)
 	, m_send(sf)
@@ -117,6 +115,16 @@ rpc_manager::rpc_manager(fun const& f, node_id const& our_id
 	, m_destructing(false)
 {
 	std::srand(time(0));
+
+#ifdef TORRENT_DHT_VERBOSE_LOGGING
+	TORRENT_LOG(rpc) << "Constructing";
+	TORRENT_LOG(rpc) << " closest_nodes_observer: " << sizeof(closest_nodes_observer);
+	TORRENT_LOG(rpc) << " find_data_observer: " << sizeof(find_data_observer);
+	TORRENT_LOG(rpc) << " announce_observer: " << sizeof(announce_observer);
+	TORRENT_LOG(rpc) << " refresh_observer: " << sizeof(refresh_observer);
+	TORRENT_LOG(rpc) << " ping_observer: " << sizeof(ping_observer);
+	TORRENT_LOG(rpc) << " null_observer: " << sizeof(null_observer);
+#endif
 }
 
 rpc_manager::~rpc_manager()
@@ -136,7 +144,7 @@ rpc_manager::~rpc_manager()
 	}
 }
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 size_t rpc_manager::allocation_size() const
 {
 	size_t s = sizeof(mpl::deref<max_observer_type_iter::base>::type);
@@ -175,7 +183,7 @@ void rpc_manager::unreachable(udp::endpoint const& ep)
 		if (tid >= max_transactions) tid = 0;
 		observer_ptr const& o = m_transactions[tid];
 		if (!o) continue;
-		if (o->target_addr != ep) continue;
+		if (o->target_ep() != ep) continue;
 		observer_ptr ptr = m_transactions[tid];
 		m_transactions[tid] = 0;
 		if (tid == m_oldest_transaction_id)
@@ -213,8 +221,9 @@ bool rpc_manager::incoming(msg const& m)
 			reply.reply = true;
 			reply.message_id = messages::error;
 			reply.error_code = 203; // Protocol error
-			reply.error_msg = "reply with invalid transaction id, size "
-				+ boost::lexical_cast<std::string>(m.transaction_id.size());
+			char msg[100];
+			snprintf(msg, sizeof(msg), "reply with invaliud transaction "
+				"id, size %d", int(m.transaction_id.size()));
 			reply.addr = m.addr;
 			reply.transaction_id = "";
 			m_send(reply);
@@ -253,7 +262,7 @@ bool rpc_manager::incoming(msg const& m)
 			return false;
 		}
 		
-		if (m.addr.address() != o->target_addr.address())
+		if (m.addr.address() != o->target_addr)
 		{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 			TORRENT_LOG(rpc) << "Reply with incorrect address and valid transaction id: " 
@@ -263,7 +272,7 @@ bool rpc_manager::incoming(msg const& m)
 		}
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-		std::ofstream reply_stats("libtorrent_logs/round_trip_ms.log", std::ios::app);
+		std::ofstream reply_stats("round_trip_ms.log", std::ios::app);
 		reply_stats << m.addr << "\t" << total_milliseconds(time_now() - o->sent)
 			<< std::endl;
 #endif
@@ -273,19 +282,6 @@ bool rpc_manager::incoming(msg const& m)
 #endif
 		o->reply(m);
 		m_transactions[tid] = 0;
-		
-		if (m.piggy_backed_ping)
-		{
-			// there is a ping request piggy
-			// backed in this reply
-			msg ph;
-			ph.message_id = messages::ping;
-			ph.transaction_id = m.ping_transaction_id;
-			ph.addr = m.addr;
-			ph.reply = true;
-			
-			reply(ph);
-		}
 		return m_table.node_seen(m.id, m.addr);
 	}
 	else
@@ -335,15 +331,19 @@ time_duration rpc_manager::tick()
 			}
 		}
 		
+#ifndef BOOST_NO_EXCEPTIONS
 		try
 		{
+#endif
 			m_transactions[m_oldest_transaction_id] = 0;
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 			TORRENT_LOG(rpc) << "Timing out transaction id: " 
-				<< m_oldest_transaction_id << " from " << o->target_addr;
+				<< m_oldest_transaction_id << " from " << o->target_ep();
 #endif
 			timeouts.push_back(o);
+#ifndef BOOST_NO_EXCEPTIONS
 		} catch (std::exception) {}
+#endif
 	}
 	
 	std::for_each(timeouts.begin(), timeouts.end(), bind(&observer::timeout, _1));
@@ -371,7 +371,7 @@ unsigned int rpc_manager::new_transaction_id(observer_ptr o)
 		m_aborted_transactions.push_back(o);
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		TORRENT_LOG(rpc) << "[new_transaction_id] Aborting message with transaction id: " 
-			<< m_next_transaction_id << " sent to " << o->target_addr
+			<< m_next_transaction_id << " sent to " << o->target_ep()
 			<< " " << total_seconds(time_now() - o->sent) << " seconds ago";
 #endif
 		m_transactions[m_next_transaction_id] = 0;
@@ -423,11 +423,13 @@ void rpc_manager::invoke(int message_id, udp::endpoint target_addr
 	m.id = m_our_id;
 	m.addr = target_addr;
 	TORRENT_ASSERT(!m_transactions[m_next_transaction_id]);
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 	int potential_new_id = m_next_transaction_id;
 #endif
+#ifndef BOOST_NO_EXCEPTIONS
 	try
 	{
+#endif
 		m.transaction_id.clear();
 		std::back_insert_iterator<std::string> out(m.transaction_id);
 		io::write_uint16(m_next_transaction_id, out);
@@ -435,14 +437,20 @@ void rpc_manager::invoke(int message_id, udp::endpoint target_addr
 		o->send(m);
 
 		o->sent = time_now();
-		o->target_addr = target_addr;
+#if TORRENT_USE_IPV6
+		o->target_addr = target_addr.address();
+#else
+		o->target_addr = target_addr.address().to_v4();
+#endif
+		o->port = target_addr.port();
 
-	#ifdef TORRENT_DHT_VERBOSE_LOGGING
+#ifdef TORRENT_DHT_VERBOSE_LOGGING
 		TORRENT_LOG(rpc) << "Invoking " << messages::ids[message_id] 
 			<< " -> " << target_addr;
-	#endif	
+#endif	
 		m_send(m);
 		new_transaction_id(o);
+#ifndef BOOST_NO_EXCEPTIONS
 	}
 	catch (std::exception& e)
 	{
@@ -450,6 +458,7 @@ void rpc_manager::invoke(int message_id, udp::endpoint target_addr
 		TORRENT_ASSERT(potential_new_id == m_next_transaction_id);
 		o->abort();
 	}
+#endif
 }
 
 void rpc_manager::reply(msg& m)
@@ -459,40 +468,10 @@ void rpc_manager::reply(msg& m)
 	if (m_destructing) return;
 
 	TORRENT_ASSERT(m.reply);
-	m.piggy_backed_ping = false;
 	m.id = m_our_id;
 	
 	m_send(m);
 }
 
-void rpc_manager::reply_with_ping(msg& m)
-{
-	INVARIANT_CHECK;
-
-	if (m_destructing) return;
-	TORRENT_ASSERT(m.reply);
-
-	m.piggy_backed_ping = true;
-	m.id = m_our_id;
-
-	m.ping_transaction_id.clear();
-	std::back_insert_iterator<std::string> out(m.ping_transaction_id);
-	io::write_uint16(m_next_transaction_id, out);
-
-	TORRENT_ASSERT(allocation_size() >= sizeof(null_observer));
-	observer_ptr o(new (allocator().malloc()) null_observer(allocator()));
-#ifndef NDEBUG
-	o->m_in_constructor = false;
-#endif
-	TORRENT_ASSERT(!m_transactions[m_next_transaction_id]);
-	o->sent = time_now();
-	o->target_addr = m.addr;
-		
-	m_send(m);
-	new_transaction_id(o);
-}
-
-
-
 } } // namespace libtorrent::dht
 
