@@ -32,13 +32,15 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/lazy_entry.hpp"
 #include "libtorrent/escape_string.hpp"
-#include <iostream>
-#include <iomanip>
 #include <cstring>
 
+#if TORRENT_USE_IOSTREAM
+#include <iostream>
+#endif
+
 namespace
 {
-	const float lazy_entry_grow_factor = 1.5f;
+	const int lazy_entry_grow_factor = 150; // percent
 	const int lazy_entry_dict_init = 5;
 	const int lazy_entry_list_init = 5;
 }
@@ -47,7 +49,7 @@ namespace libtorrent
 {
 	int fail_bdecode(lazy_entry& ret, int return_value = -1)
 	{
-		ret = lazy_entry();
+		ret.clear();
 		return return_value;
 	}
 
@@ -191,26 +193,26 @@ namespace libtorrent
 		if (m_capacity == 0)
 		{
 			int capacity = lazy_entry_dict_init;
-			m_data.dict = new (std::nothrow) std::pair<char const*, lazy_entry>[capacity];
+			m_data.dict = new (std::nothrow) lazy_dict_entry[capacity];
 			if (m_data.dict == 0) return 0;
 			m_capacity = capacity;
 		}
 		else if (m_size == m_capacity)
 		{
-			int capacity = m_capacity * lazy_entry_grow_factor;
-			std::pair<char const*, lazy_entry>* tmp = new (std::nothrow) std::pair<char const*, lazy_entry>[capacity];
+			int capacity = m_capacity * lazy_entry_grow_factor / 100;
+			lazy_dict_entry* tmp = new (std::nothrow) lazy_dict_entry[capacity];
 			if (tmp == 0) return 0;
-			std::memcpy(tmp, m_data.dict, sizeof(std::pair<char const*, lazy_entry>) * m_size);
-			for (int i = 0; i < m_size; ++i) m_data.dict[i].second.release();
+			std::memcpy(tmp, m_data.dict, sizeof(lazy_dict_entry) * m_size);
+			for (int i = 0; i < m_size; ++i) m_data.dict[i].val.release();
 			delete[] m_data.dict;
 			m_data.dict = tmp;
 			m_capacity = capacity;
 		}
 
 		TORRENT_ASSERT(m_size < m_capacity);
-		std::pair<char const*, lazy_entry>& ret = m_data.dict[m_size++];
-		ret.first = name;
-		return &ret.second;
+		lazy_dict_entry& ret = m_data.dict[m_size++];
+		ret.name = name;
+		return &ret.val;
 	}
 
 	namespace
@@ -257,6 +259,14 @@ namespace libtorrent
 		}
 	}
 
+	std::pair<std::string, lazy_entry const*> lazy_entry::dict_at(int i) const
+	{
+		TORRENT_ASSERT(m_type == dict_t);
+		TORRENT_ASSERT(i < m_size);
+		lazy_dict_entry const& e = m_data.dict[i];
+		return std::make_pair(std::string(e.name, e.val.m_begin - e.name), &e.val);
+	}
+
 	std::string lazy_entry::dict_find_string_value(char const* name) const
 	{
 		lazy_entry const* e = dict_find(name);
@@ -271,6 +281,13 @@ namespace libtorrent
 		return e;
 	}
 
+	lazy_entry const* lazy_entry::dict_find_int(char const* name) const
+	{
+		lazy_entry const* e = dict_find(name);
+		if (e == 0 || e->type() != lazy_entry::int_t) return 0;
+		return e;
+	}
+
 	size_type lazy_entry::dict_find_int_value(char const* name, size_type default_val) const
 	{
 		lazy_entry const* e = dict_find(name);
@@ -297,9 +314,9 @@ namespace libtorrent
 		TORRENT_ASSERT(m_type == dict_t);
 		for (int i = 0; i < m_size; ++i)
 		{
-			std::pair<char const*, lazy_entry> const& e = m_data.dict[i];
-			if (string_equal(name, e.first, e.second.m_begin - e.first))
-				return &m_data.dict[i].second;
+			lazy_dict_entry& e = m_data.dict[i];
+			if (string_equal(name, e.name, e.val.m_begin - e.name))
+				return &e.val;
 		}
 		return 0;
 	}
@@ -317,7 +334,7 @@ namespace libtorrent
 		}
 		else if (m_size == m_capacity)
 		{
-			int capacity = m_capacity * lazy_entry_grow_factor;
+			int capacity = m_capacity * lazy_entry_grow_factor / 100;
 			lazy_entry* tmp = new (std::nothrow) lazy_entry[capacity];
 			if (tmp == 0) return 0;
 			std::memcpy(tmp, m_data.list, sizeof(lazy_entry) * m_size);
@@ -365,12 +382,25 @@ namespace libtorrent
 		return return_t(m_begin, m_end - m_begin);
 	}
 
+#if TORRENT_USE_IOSTREAM
 	std::ostream& operator<<(std::ostream& os, lazy_entry const& e)
 	{
+		return os << print_entry(e);
+	}
+#endif // TORRENT_USE_IOSTREAM
+
+	std::string print_entry(lazy_entry const& e)
+	{
+		std::string ret;
 		switch (e.type())
 		{
-			case lazy_entry::none_t: return os << "none";
-			case lazy_entry::int_t: return os << std::dec << std::setw(0) << e.int_value();
+			case lazy_entry::none_t: return "none";
+			case lazy_entry::int_t:
+			{
+				char str[100];
+				snprintf(str, sizeof(str), "%"PRId64, e.int_value());
+				return str;
+			}
 			case lazy_entry::string_t:
 			{
 				bool printable = true;
@@ -378,20 +408,29 @@ namespace libtorrent
 				for (int i = 0; i < e.string_length(); ++i)
 				{
 					using namespace std;
-					if (isprint((unsigned char)str[i])) continue;
+					if (is_print((unsigned char)str[i])) continue;
 					printable = false;
 					break;
 				}
-				os << "'";
-				if (printable) return os << e.string_value() << "'";
+				ret += "'";
+				if (printable)
+				{
+					ret += e.string_value();
+					ret += "'";
+					return ret;
+				}
 				for (int i = 0; i < e.string_length(); ++i)
-					os << std::hex << std::setfill('0') << std::setw(2)
-					<< int((unsigned char)(str[i]));
-				return os << "'" << std::dec;
+				{
+					char tmp[5];
+					snprintf(tmp, sizeof(tmp), "%02x", (unsigned char)str[i]);
+					ret += tmp;
+				}
+				ret += "'";
+				return ret;
 			}
 			case lazy_entry::list_t:
 			{
-				os << "[";
+				ret += '[';
 				bool one_liner = (e.list_size() == 0
 					|| (e.list_at(0)->type() == lazy_entry::int_t
 						&& e.list_size() < 20)
@@ -399,19 +438,20 @@ namespace libtorrent
 						&& (e.list_at(0)->string_length() < 10
 							|| e.list_size() < 2)
 						&& e.list_size() < 5));
-				if (!one_liner) os << "\n";
+				if (!one_liner) ret += "\n";
 				for (int i = 0; i < e.list_size(); ++i)
 				{
-					if (i == 0 && one_liner) os << " ";
-					os << *e.list_at(i);
-					if (i < e.list_size() - 1) os << (one_liner?", ":",\n");
-					else os << (one_liner?" ":"\n");
+					if (i == 0 && one_liner) ret += " ";
+					ret += print_entry(*e.list_at(i));
+					if (i < e.list_size() - 1) ret += (one_liner?", ":",\n");
+					else ret += (one_liner?" ":"\n");
 				}
-				return os << "]";
+				ret += "]";
+				return ret;
 			}
 			case lazy_entry::dict_t:
 			{
-				os << "{";
+				ret += "{";
 				bool one_liner = (e.dict_size() == 0
 					|| e.dict_at(0).second->type() == lazy_entry::int_t
 					|| (e.dict_at(0).second->type() == lazy_entry::string_t
@@ -419,20 +459,23 @@ namespace libtorrent
 					|| e.dict_at(0).first.size() < 10)
 					&& e.dict_size() < 5;
 
-				if (!one_liner) os << "\n";
+				if (!one_liner) ret += "\n";
 				for (int i = 0; i < e.dict_size(); ++i)
 				{
-					if (i == 0 && one_liner) os << " ";
+					if (i == 0 && one_liner) ret += " ";
 					std::pair<std::string, lazy_entry const*> ent = e.dict_at(i);
-					os << "'" << ent.first << "': " << *ent.second;
-					if (i < e.dict_size() - 1) os << (one_liner?", ":",\n");
-					else os << (one_liner?" ":"\n");
+					ret += "'";
+					ret += ent.first;
+					ret += "': ";
+					ret += print_entry(*ent.second);
+					if (i < e.dict_size() - 1) ret += (one_liner?", ":",\n");
+					else ret += (one_liner?" ":"\n");
 				}
-				return os << "}";
+				ret += "}";
+				return ret;
 			}
 		}
-		return os;
+		return ret;
 	}
-
 };
 
