@@ -31,15 +31,17 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/pch.hpp"
-#include "libtorrent/config.hpp"
-
-#ifdef TORRENT_WINDOWS
-#include <winsock2.h> // for error codes
-#endif
 
 #include <boost/version.hpp>
 #include <boost/bind.hpp>
 
+#include "libtorrent/config.hpp"
+
+#if defined TORRENT_WINDOWS || defined TORRENT_CYGWIN
+// asio assumes that the windows error codes are defined already
+#include <winsock2.h>
+#endif
+
 #if BOOST_VERSION < 103500
 #include <asio/ip/host_name.hpp>
 #else
@@ -54,8 +56,10 @@ POSSIBILITY OF SUCH DAMAGE.
 using boost::bind;
 using namespace libtorrent;
 
-natpmp::natpmp(io_service& ios, address const& listen_interface, portmap_callback_t const& cb)
+natpmp::natpmp(io_service& ios, address const& listen_interface
+	, portmap_callback_t const& cb, log_callback_t const& lcb)
 	: m_callback(cb)
+	, m_log_callback(lcb)
 	, m_currently_mapping(-1)
 	, m_retry_count(0)
 	, m_socket(ios)
@@ -65,9 +69,6 @@ natpmp::natpmp(io_service& ios, address const& listen_interface, portmap_callbac
 	, m_disabled(false)
 	, m_abort(false)
 {
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log.open("natpmp.log", std::ios::in | std::ios::out | std::ios::trunc);
-#endif
 	rebind(listen_interface);
 }
 
@@ -79,11 +80,10 @@ void natpmp::rebind(address const& listen_interface)
 	address gateway = get_default_gateway(m_socket.get_io_service(), ec);
 	if (ec)
 	{
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << time_now_string() << " failed to find default router: "
-			<< ec.message() << std::endl;
-#endif
-		disable("failed to find default router");
+		char msg[200];
+		snprintf(msg, sizeof(msg), "failed to find default route: %s", ec.message().c_str());
+		log(msg, l);
+		disable(ec, l);
 		return;
 	}
 
@@ -93,21 +93,21 @@ void natpmp::rebind(address const& listen_interface)
 	if (nat_endpoint == m_nat_endpoint) return;
 	m_nat_endpoint = nat_endpoint;
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string() << " found router at: "
-		<< m_nat_endpoint.address() << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "found router at: %s"
+		, print_address(m_nat_endpoint.address()).c_str());
+	log(msg, l);
 
 	m_socket.open(udp::v4(), ec);
 	if (ec)
 	{
-		disable(ec.message().c_str());
+		disable(ec, l);
 		return;
 	}
 	m_socket.bind(udp::endpoint(address_v4::any(), 0), ec);
 	if (ec)
 	{
-		disable(ec.message().c_str());
+		disable(ec, l);
 		return;
 	}
 
@@ -121,11 +121,32 @@ void natpmp::rebind(address const& listen_interface)
 			|| i->action != mapping_t::action_none)
 			continue;
 		i->action = mapping_t::action_add;
-		update_mapping(i - m_mappings.begin());
+		update_mapping(i - m_mappings.begin(), l);
 	}
 }
 
-void natpmp::disable(char const* message)
+bool natpmp::get_mapping(int index, int& local_port, int& external_port, int& protocol) const
+{
+	mutex_t::scoped_lock l(m_mutex);
+
+	TORRENT_ASSERT(index < int(m_mappings.size()) && index >= 0);
+	if (index >= int(m_mappings.size()) || index < 0) return false;
+	mapping_t const& m = m_mappings[index];
+	if (m.protocol == none) return false;
+	local_port = m.local_port;
+	external_port = m.external_port;
+	protocol = m.protocol;
+	return true;
+}
+
+void natpmp::log(char const* msg, mutex_t::scoped_lock& l)
+{
+	l.unlock();
+	m_log_callback(msg);
+	l.lock();
+}
+
+void natpmp::disable(error_code const& ec, mutex_t::scoped_lock& l)
 {
 	m_disabled = true;
 
@@ -134,24 +155,32 @@ void natpmp::disable(char const* message)
 	{
 		if (i->protocol == none) continue;
 		i->protocol = none;
-		m_callback(i - m_mappings.begin(), 0, message);
+		int index = i - m_mappings.begin();
+		l.unlock();
+		m_callback(index, 0, ec);
+		l.lock();
 	}
-
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string() << " NAT-PMP disabled: " << message << std::endl;
-#endif
-	close_impl();
+	close_impl(l);
 }
+
 void natpmp::delete_mapping(int index)
 {
+	mutex_t::scoped_lock l(m_mutex);
+
 	TORRENT_ASSERT(index < int(m_mappings.size()) && index >= 0);
 	if (index >= int(m_mappings.size()) || index < 0) return;
 	mapping_t& m = m_mappings[index];
 
 	if (m.protocol == none) return;
+	if (!m.map_sent)
+	{
+		m.action = mapping_t::action_none;
+		m.protocol = none;
+		return;
+	}
 
 	m.action = mapping_t::action_delete;
-	update_mapping(index);
+	update_mapping(index, l);
 }
 
 int natpmp::add_mapping(protocol_type p, int external_port, int local_port)
@@ -174,16 +203,13 @@ int natpmp::add_mapping(protocol_type p, int external_port, int local_port)
 
 	int mapping_index = i - m_mappings.begin();
 
-	update_mapping(mapping_index);
+	update_mapping(mapping_index, l);
 	return mapping_index;
 }
 
-void natpmp::try_next_mapping(int i)
+void natpmp::try_next_mapping(int i, mutex_t::scoped_lock& l)
 {
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string() << " try_next_mapping [ " << i << " ]" << std::endl;
-#endif
-
+/*
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	ptime now = time_now();
 	for (std::vector<mapping_t>::iterator m = m_mappings.begin()
@@ -198,10 +224,10 @@ void natpmp::try_next_mapping(int i)
 			<< " ]" << std::endl;
 	}
 #endif
-
+*/
 	if (i < int(m_mappings.size()) - 1)
 	{
-		update_mapping(i + 1);
+		update_mapping(i + 1, l);
 		return;
 	}
 
@@ -217,20 +243,20 @@ void natpmp::try_next_mapping(int i)
 			m_send_timer.cancel(ec);
 			m_socket.close(ec);
 		}
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << "     done" << (m_abort?" shutting down":"") << std::endl;
-#endif
+//#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
+//		m_log << "     done" << (m_abort?" shutting down":"") << std::endl;
+//#endif
 		return;
 	}
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << "     updating " << (m - m_mappings.begin()) << std::endl;
-#endif
+//#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
+//	m_log << "     updating " << (m - m_mappings.begin()) << std::endl;
+//#endif
 
-	update_mapping(m - m_mappings.begin());
+	update_mapping(m - m_mappings.begin(), l);
 }
 
-void natpmp::update_mapping(int i)
+void natpmp::update_mapping(int i, mutex_t::scoped_lock& l)
 {
 	if (i == m_mappings.size())
 	{
@@ -240,9 +266,9 @@ void natpmp::update_mapping(int i)
 			m_send_timer.cancel(ec);
 			m_socket.close(ec);
 		}
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << "     done" << (m_abort?" shutting down":"") << std::endl;
-#endif
+//#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
+//		m_log << "     done" << (m_abort?" shutting down":"") << std::endl;
+//#endif
 		return;
 	}
 
@@ -250,7 +276,7 @@ void natpmp::update_mapping(int i)
 	if (m.action == mapping_t::action_none
 		|| m.protocol == none)
 	{
-		try_next_mapping(i);
+		try_next_mapping(i, l);
 		return;
 	}
 
@@ -259,11 +285,11 @@ void natpmp::update_mapping(int i)
 		// the socket is not currently in use
 		// send out a mapping request
 		m_retry_count = 0;
-		send_map_request(i);
+		send_map_request(i, l);
 	}
 }
 
-void natpmp::send_map_request(int i)
+void natpmp::send_map_request(int i, mutex_t::scoped_lock& l)
 {
 	using namespace libtorrent::detail;
 
@@ -282,17 +308,17 @@ void natpmp::send_map_request(int i)
 	int ttl = m.action == mapping_t::action_add ? 3600 : 0;
 	write_uint32(ttl, out); // port mapping lifetime
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string()
-		<< " ==> port map ["
-		<< " action: " << (m.action == mapping_t::action_add ? "add" : "delete") << " "
-		<< " proto: " << (m.protocol == udp ? "udp" : "tcp")
-		<< " local: " << m.local_port << " external: " << m.external_port
-		<< " ttl: " << ttl << " ]" << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "==> port map [ action: %s"
+		" proto: %s local: %u external: %u ttl: %u ]"
+		, m.action == mapping_t::action_add ? "add" : "delete"
+		, m.protocol == udp ? "udp" : "tcp"
+		, m.local_port, m.external_port, ttl);
+	log(msg, l);
 
 	error_code ec;
 	m_socket.send_to(asio::buffer(buf, 12), m_nat_endpoint, 0, ec);
+	m.map_sent = true;
 	if (m_abort)
 	{
 		// when we're shutting down, ignore the
@@ -300,7 +326,7 @@ void natpmp::send_map_request(int i)
 		// immediately
 		m_currently_mapping = -1;
 		m.action = mapping_t::action_none;
-		try_next_mapping(i);
+		try_next_mapping(i, l);
 	}
 	else
 	{
@@ -325,23 +351,23 @@ void natpmp::resend_request(int i, error_code const& e)
 		m_mappings[i].action = mapping_t::action_none;
 		// try again in two hours
 		m_mappings[i].expires = time_now() + hours(2);
-		try_next_mapping(i);
+		try_next_mapping(i, l);
 		return;
 	}
-	send_map_request(i);
+	send_map_request(i, l);
 }
 
 void natpmp::on_reply(error_code const& e
 	, std::size_t bytes_transferred)
 {
+	mutex_t::scoped_lock l(m_mutex);
+
 	using namespace libtorrent::detail;
 	if (e)
 	{
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << time_now_string()
-			<< " <== on_receive ["
-			" error: " << e.message() << " ]" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error on receiving reply: %s", e.message().c_str());
+		log(msg, l);
 		return;
 	}
 
@@ -352,22 +378,19 @@ void natpmp::on_reply(error_code const& e
 /*
 	if ((rand() % 2) == 0)
 	{
-		log(" simulating drop");
+		log(" simulating drop", l);
 		return;
 	}
 */
 	if (m_remote != m_nat_endpoint)
 	{
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << time_now_string()
-			<< " <== received packet from the wrong IP ["
-			" ip: " << m_remote << " ]" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "received packet from wrong IP: %s"
+			, print_endpoint(m_remote).c_str());
+		log(msg, l);
 		return;
 	}
 
-	mutex_t::scoped_lock l(m_mutex);
-
 	error_code ec;
 	m_send_timer.cancel(ec);
 
@@ -384,20 +407,18 @@ void natpmp::on_reply(error_code const& e
 
 	int protocol = (cmd - 128 == 1)?udp:tcp;
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string()
-		<< " <== port map ["
-		<< " protocol: " << (cmd - 128 == 1 ? "udp" : "tcp")
-		<< " local: " << private_port << " external: " << public_port
-		<< " ttl: " << lifetime << " ]" << std::endl;
-#endif
+	char msg[200];
+	int num_chars = snprintf(msg, sizeof(msg), "<== port map ["
+		" protocol: %s local: %u external: %u ttl: %u ]"
+		, (cmd - 128 == 1 ? "udp" : "tcp")
+		, private_port, public_port, lifetime);
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	if (version != 0)
 	{
-		m_log << "*** unexpected version: " << version << std::endl;
+		snprintf(msg + num_chars, sizeof(msg) - num_chars, "unexpected version: %u"
+			, version);
+		log(msg, l);
 	}
-#endif
 
 	mapping_t* m = 0;
 	int index = -1;
@@ -406,6 +427,7 @@ void natpmp::on_reply(error_code const& e
 	{
 		if (private_port != i->local_port) continue;
 		if (protocol != i->protocol) continue;
+		if (!i->map_sent) continue;
 		m = &*i;
 		index = i - m_mappings.begin();
 		break;
@@ -413,11 +435,11 @@ void natpmp::on_reply(error_code const& e
 
 	if (m == 0)
 	{
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << "*** not found in map table" << std::endl;
-#endif
+		snprintf(msg + num_chars, sizeof(msg) - num_chars, " not found in map table");
+		log(msg, l);
 		return;
 	}
+	log(msg, l);
 
 	if (public_port == 0 || lifetime == 0)
 	{
@@ -433,32 +455,36 @@ void natpmp::on_reply(error_code const& e
 
 	if (result != 0)
 	{
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-		m_log << "*** ERROR: " << result << std::endl;
-#endif
-		std::stringstream errmsg;
-		errmsg << "NAT router reports error (" << result << ") ";
-		switch (result)
+		int errors[] =
 		{
-			case 1: errmsg << "Unsupported protocol version"; break;
-			case 2: errmsg << "Not authorized to create port map (enable NAT-PMP on your router)"; break;
-			case 3: errmsg << "Network failure"; break;
-			case 4: errmsg << "Out of resources"; break;
-			case 5: errmsg << "Unsupported opcode"; break;
-		}
+			errors::unsupported_protocol_version,
+			errors::natpmp_not_authorized,
+			errors::network_failure,
+			errors::no_resources,
+			errors::unsupported_opcode,
+		};
+		int ev = errors::no_error;
+		if (result >= 1 && result <= 5) ev = errors[result - 1];
+
 		m->expires = time_now() + hours(2);
-		m_callback(index, 0, errmsg.str());
+		l.unlock();
+		m_callback(index, 0, error_code(ev, get_libtorrent_category()));
+		l.lock();
 	}
 	else if (m->action == mapping_t::action_add)
 	{
-		m_callback(index, m->external_port, "");
+		l.unlock();
+		m_callback(index, m->external_port, error_code(errors::no_error, get_libtorrent_category()));
+		l.lock();
 	}
 
+	if (m_abort) return;
+
 	m_currently_mapping = -1;
 	m->action = mapping_t::action_none;
 	m_send_timer.cancel(ec);
 	update_expiration_timer();
-	try_next_mapping(index);
+	try_next_mapping(index, l);
 }
 
 void natpmp::update_expiration_timer()
@@ -466,8 +492,9 @@ void natpmp::update_expiration_timer()
 	if (m_abort) return;
 
 	ptime now = time_now();
+/*
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string() << " update_expiration_timer " << std::endl;
+	m_log << time_now_string() << "update_expiration_timer " << std::endl;
 	for (std::vector<mapping_t>::iterator i = m_mappings.begin()
 		, end(m_mappings.end()); i != end; ++i)
 	{
@@ -480,7 +507,7 @@ void natpmp::update_expiration_timer()
 			<< " ]" << std::endl;
 	}
 #endif
-
+*/
 	ptime min_expire = now + seconds(3600);
 	int min_index = -1;
 	for (std::vector<mapping_t>::iterator i = m_mappings.begin()
@@ -500,12 +527,14 @@ void natpmp::update_expiration_timer()
 
 	if (min_index >= 0)
 	{
+/*
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 		m_log << time_now_string() << " next expiration ["
 			" i: " << min_index
 			<< " ttl: " << total_seconds(min_expire - time_now())
 			<< " ]" << std::endl;
 #endif
+*/
 		error_code ec;
 		if (m_next_refresh >= 0) m_refresh_timer.cancel(ec);
 		m_refresh_timer.expires_from_now(min_expire - now, ec);
@@ -518,32 +547,35 @@ void natpmp::mapping_expired(error_code const& e, int i)
 {
 	if (e) return;
 	mutex_t::scoped_lock l(m_mutex);
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	m_log << time_now_string() << " mapping expired [ i: " << i << " ]" << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "mapping %u expired", i);
+	log(msg, l);
 	m_mappings[i].action = mapping_t::action_add;
 	if (m_next_refresh == i) m_next_refresh = -1;
-	update_mapping(i);
+	update_mapping(i, l);
 }
 
 void natpmp::close()
 {
 	mutex_t::scoped_lock l(m_mutex);
-	close_impl();
+	close_impl(l);
 }
 
-void natpmp::close_impl()
+void natpmp::close_impl(mutex_t::scoped_lock& l)
 {
 	m_abort = true;
-	error_code ec;
+	log("closing", l);
+/*
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	m_log << time_now_string() << " close" << std::endl;
 #endif
+*/
 	if (m_disabled) return;
 	ptime now = time_now();
 	for (std::vector<mapping_t>::iterator i = m_mappings.begin()
 		, end(m_mappings.end()); i != end; ++i)
 	{
+/*
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 		m_log << "     " << (i - m_mappings.begin()) << " [ "
 			"proto: " << (i->protocol == none ? "none" : i->protocol == tcp ? "tcp" : "udp")
@@ -553,11 +585,13 @@ void natpmp::close_impl()
 			<< " ttl: " << total_seconds(i->expires - now)
 			<< " ]" << std::endl;
 #endif
+*/
 		if (i->protocol == none) continue;
 		i->action = mapping_t::action_delete;
 	}
+	error_code ec;
 	m_refresh_timer.cancel(ec);
 	m_currently_mapping = -1;
-	update_mapping(0);
+	update_mapping(0, l);
 }
 
