@@ -65,6 +65,7 @@ namespace libtorrent
 		, m_num_have(0)
 		, m_cursor(0)
 		, m_reverse_cursor(0)
+		, m_sparse_regions(1)
 		, m_dirty(false)
 	{
 #ifdef TORRENT_PICKER_LOG
@@ -104,10 +105,10 @@ namespace libtorrent
 			i->index = 0;
 		}
 
-		for (std::vector<piece_pos>::const_iterator i = m_piece_map.begin() + m_cursor
+		for (std::vector<piece_pos>::iterator i = m_piece_map.begin() + m_cursor
 			, end(m_piece_map.end()); i != end && (i->have() || i->filtered());
 			++i, ++m_cursor);
-		for (std::vector<piece_pos>::const_reverse_iterator i = m_piece_map.rend()
+		for (std::vector<piece_pos>::reverse_iterator i = m_piece_map.rend()
 			- m_reverse_cursor; m_reverse_cursor > 0 && (i->have() || i->filtered());
 			++i, --m_reverse_cursor);
 
@@ -194,6 +195,7 @@ namespace libtorrent
 			std::copy(other->info, other->info + m_blocks_per_piece, i->info);
 			other->info = i->info;
 		}
+		m_piece_map[i->index].downloading = false;
 		m_downloads.erase(i);
 	}
 
@@ -228,7 +230,7 @@ namespace libtorrent
 		}
 	}
 
-#if defined TORRENT_PICKER_LOG || defined TORRENT_DEBUG
+#if defined TORRENT_PICKER_LOG
 	void piece_picker::print_pieces() const
 	{
 		for (std::vector<int>::const_iterator i = m_priority_boundries.begin()
@@ -342,7 +344,7 @@ namespace libtorrent
 		index = num_pieces;
 		if (num_pieces > 0)
 		{
-			for (std::vector<piece_pos>::const_reverse_iterator i = m_piece_map.rend()
+			for (std::vector<piece_pos>::reverse_iterator i = m_piece_map.rend()
 				- index; index > 0 && (i->have() || i->filtered()); ++i, --index);
 			TORRENT_ASSERT(index == num_pieces
 				|| m_piece_map[index].have()
@@ -466,12 +468,12 @@ namespace libtorrent
 	}
 #endif
 
-	float piece_picker::distributed_copies() const
+	std::pair<int, int> piece_picker::distributed_copies() const
 	{
 		TORRENT_ASSERT(m_seeds >= 0);
-		const float num_pieces = static_cast<float>(m_piece_map.size());
+		const int num_pieces = m_piece_map.size();
 
-		if (num_pieces == 0) return 1.0f;
+		if (num_pieces == 0) return std::make_pair(1, 0);
 		int min_availability = piece_pos::max_peer_count;
 		// find the lowest availability count
 		// count the number of pieces that have that availability
@@ -501,7 +503,7 @@ namespace libtorrent
 			}
 		}
 		TORRENT_ASSERT(integer_part + fraction_part == num_pieces);
-		return float(min_availability + m_seeds) + (fraction_part / num_pieces);
+		return std::make_pair(min_availability + m_seeds, fraction_part * 1000 / num_pieces);
 	}
 
 	void piece_picker::priority_range(int prio, int* start, int* end)
@@ -804,18 +806,17 @@ namespace libtorrent
 
 		TORRENT_ASSERT(i != m_downloads.end());
 #ifdef TORRENT_DEBUG
-			int num_blocks = blocks_in_piece(i->index);
-			for (int k = 0; k < num_blocks; ++k)
-			{
-				TORRENT_ASSERT(i->info[k].state == block_info::state_finished);
-				TORRENT_ASSERT(i->info[k].num_peers == 0);
-			}
+		int num_blocks = blocks_in_piece(i->index);
+		for (int k = 0; k < num_blocks; ++k)
+		{
+			TORRENT_ASSERT(i->info[k].state == block_info::state_finished);
+			TORRENT_ASSERT(i->info[k].num_peers == 0);
+		}
 #endif
-		erase_download_piece(i);
 
 		piece_pos& p = m_piece_map[index];
 		int prev_priority = p.priority(this);
-		p.downloading = 0;
+		erase_download_piece(i);
 		int new_priority = p.priority(this);
 
 		if (new_priority == prev_priority) return;
@@ -906,7 +907,9 @@ namespace libtorrent
 
 	void piece_picker::inc_refcount(bitfield const& bitmask)
 	{
+#ifdef TORRENT_EXPENSIVE_INVARIANT_CHECKS
 		TORRENT_PIECE_PICKER_INVARIANT_CHECK;
+#endif
 		TORRENT_ASSERT(bitmask.size() == m_piece_map.size());
 
 		int index = 0;
@@ -926,7 +929,9 @@ namespace libtorrent
 
 	void piece_picker::dec_refcount(bitfield const& bitmask)
 	{
+#ifdef TORRENT_EXPENSIVE_INVARIANT_CHECKS
 		TORRENT_PIECE_PICKER_INVARIANT_CHECK;
+#endif
 		TORRENT_ASSERT(bitmask.size() == m_piece_map.size());
 
 		int index = 0;
@@ -1082,13 +1087,34 @@ namespace libtorrent
 				, has_index(index));
 			TORRENT_ASSERT(i != m_downloads.end());
 			erase_download_piece(i);
-			p.downloading = 0;
 		}
 
 		TORRENT_ASSERT(std::find_if(m_downloads.begin(), m_downloads.end()
 			, has_index(index)) == m_downloads.end());
 
 		if (p.have()) return;
+
+// maintain sparse_regions
+		if (index == 0)
+		{
+			if (index == m_piece_map.size() - 1
+				|| m_piece_map[index + 1].have())
+				--m_sparse_regions;
+		}
+		else if (index == int(m_piece_map.size() - 1))
+		{
+			if (index == 0
+				|| m_piece_map[index - 1].have())
+				--m_sparse_regions;
+		}
+		else
+		{
+			bool have_before = m_piece_map[index-1].have();
+			bool have_after = m_piece_map[index+1].have();
+			if (have_after && have_before) --m_sparse_regions;
+			else if (!have_after && !have_before) ++m_sparse_regions;
+		}
+
 		if (p.filtered())
 		{
 			--m_num_filtered;
@@ -1312,6 +1338,12 @@ namespace libtorrent
 	//     peer
 	// * prioritize_partials
 	//     pick blocks from downloading pieces first
+	// * speed_affinity
+	//     have an affinity to pick pieces in the same speed
+	//     category.
+	// * ignore_whole_pieces
+	//     ignores the prefer_whole_pieces parameter (as if
+	//     it was 0)
 
 	// only one of rarest_first, sequential can be set
 
@@ -1323,6 +1355,8 @@ namespace libtorrent
 		// prevent the number of partial pieces to grow indefinitely
 		if (m_downloads.size() > 20) options |= prioritize_partials;
 
+		if (options & ignore_whole_pieces) prefer_whole_pieces = 0;
+
 		// only one of rarest_first and sequential can be set.
 		TORRENT_ASSERT(bool(options & rarest_first)
 			+ bool(options & sequential) <= 1);
@@ -1808,7 +1842,7 @@ namespace libtorrent
 			// if the state of the piece is none (the
 			// piece will in that case change state).
 			if (dp.state != none && dp.state != speed
-				&& !exclusive_active)
+				&& !exclusive_active && (options & speed_affinity))
 			{
 				if (abs(dp.state - speed) == 1)
 				{
@@ -1888,16 +1922,17 @@ namespace libtorrent
 		TORRENT_ASSERT(i != m_downloads.end());
 		TORRENT_ASSERT((int)i->finished <= m_blocks_per_piece);
 		int max_blocks = blocks_in_piece(index);
-		if ((int)i->finished < max_blocks) return false;
+		if (int(i->finished) + int(i->writing) < max_blocks) return false;
+		TORRENT_ASSERT(int(i->finished) + int(i->writing) == max_blocks);
 
 #ifdef TORRENT_DEBUG
 		for (int k = 0; k < max_blocks; ++k)
 		{
-			TORRENT_ASSERT(i->info[k].state == block_info::state_finished);
+			TORRENT_ASSERT(i->info[k].state == block_info::state_finished
+				|| i->info[k].state == block_info::state_writing);
 		}
 #endif
 
-		TORRENT_ASSERT((int)i->finished == max_blocks);
 		return true;
 	}
 
@@ -2066,32 +2101,55 @@ namespace libtorrent
 		TORRENT_ASSERT(block.piece_index < (int)m_piece_map.size());
 		TORRENT_ASSERT(block.block_index < blocks_in_piece(block.piece_index));
 
-		TORRENT_ASSERT(m_piece_map[block.piece_index].downloading);
-
-		std::vector<downloading_piece>::iterator i
-			= std::find_if(m_downloads.begin(), m_downloads.end(), has_index(block.piece_index));
-		TORRENT_ASSERT(i != m_downloads.end());
-		block_info& info = i->info[block.block_index];
-
-		info.peer = peer;
-		TORRENT_ASSERT(info.state == block_info::state_requested);
-		if (info.state == block_info::state_requested) --i->requested;
-		TORRENT_ASSERT(i->requested >= 0);
-		TORRENT_ASSERT(info.state != block_info::state_writing);
-		++i->writing;
-		info.state = block_info::state_writing;
-
-		// all other requests for this block should have been
-		// cancelled now
-		info.num_peers = 0;
-
-		if (i->requested == 0)
+		piece_pos& p = m_piece_map[block.piece_index];
+		if (p.downloading == 0)
 		{
-			// there are no blocks requested in this piece.
-			// remove the fast/slow state from it
-			i->state = none;
+#ifdef TORRENT_EXPENSIVE_INVARIANT_CHECKS
+			TORRENT_PIECE_PICKER_INVARIANT_CHECK;
+#endif
+			int prio = p.priority(this);
+			TORRENT_ASSERT(prio < int(m_priority_boundries.size())
+				|| m_dirty);
+			TORRENT_ASSERT(prio >= 0);
+			p.downloading = 1;
+			if (prio >= 0 && !m_dirty) update(prio, p.index);
+
+			downloading_piece& dp = add_download_piece();
+			dp.index = block.piece_index;
+			dp.state = none;
+			block_info& info = dp.info[block.block_index];
+			info.state = block_info::state_writing;
+			info.peer = peer;
+			info.num_peers = 0;
+			dp.writing = 1;
+			sort_piece(m_downloads.end()-1);
+		}
+		else
+		{
+			std::vector<downloading_piece>::iterator i
+				= std::find_if(m_downloads.begin(), m_downloads.end(), has_index(block.piece_index));
+			TORRENT_ASSERT(i != m_downloads.end());
+			block_info& info = i->info[block.block_index];
+
+			info.peer = peer;
+			if (info.state == block_info::state_requested) --i->requested;
+			TORRENT_ASSERT(i->requested >= 0);
+			TORRENT_ASSERT(info.state != block_info::state_writing);
+			++i->writing;
+			info.state = block_info::state_writing;
+
+			// all other requests for this block should have been
+			// cancelled now
+			info.num_peers = 0;
+
+			if (i->requested == 0)
+			{
+				// there are no blocks requested in this piece.
+				// remove the fast/slow state from it
+				i->state = none;
+			}
+			sort_piece(i);
 		}
-		sort_piece(i);
 	}
 
 	void piece_picker::write_failed(piece_block block)
@@ -2101,13 +2159,38 @@ namespace libtorrent
 		std::vector<downloading_piece>::iterator i
 			= std::find_if(m_downloads.begin(), m_downloads.end(), has_index(block.piece_index));
 		TORRENT_ASSERT(i != m_downloads.end());
+		if (i == m_downloads.end()) return;
+
 		block_info& info = i->info[block.block_index];
 		TORRENT_ASSERT(info.state == block_info::state_writing);
 		TORRENT_ASSERT(info.num_peers == 0);
 
-		--i->writing;
-		info.state = block_info::state_none;
+		TORRENT_ASSERT(i->writing > 0);
+		TORRENT_ASSERT(info.state == block_info::state_writing);
+
+		if (info.state == block_info::state_finished) return;
+		if (info.state == block_info::state_writing) --i->writing;
+
 		info.peer = 0;
+
+		info.state = block_info::state_none;
+
+		if (i->finished + i->writing + i->requested == 0)
+		{
+			piece_pos& p = m_piece_map[block.piece_index];
+			int prev_priority = p.priority(this);
+			erase_download_piece(i);
+			int new_priority = p.priority(this);
+
+			if (m_dirty) return;
+			if (new_priority == prev_priority) return;
+			if (prev_priority == -1) add(p.index);
+			else update(prev_priority, p.index);
+		}
+		else
+		{
+			sort_piece(i);
+		}
 	}
 
 	void piece_picker::mark_as_finished(piece_block block, void* peer)
@@ -2156,6 +2239,9 @@ namespace libtorrent
 				= std::find_if(m_downloads.begin(), m_downloads.end(), has_index(block.piece_index));
 			TORRENT_ASSERT(i != m_downloads.end());
 			block_info& info = i->info[block.block_index];
+
+			if (info.state == block_info::state_finished) return;
+
 			TORRENT_ASSERT(info.num_peers == 0);
 			info.peer = peer;
 			TORRENT_ASSERT(info.state == block_info::state_writing
@@ -2262,12 +2348,11 @@ namespace libtorrent
 		// that's being downloaded, remove it from the list
 		if (i->requested + i->finished + i->writing == 0)
 		{
-			erase_download_piece(i);
 			piece_pos& p = m_piece_map[block.piece_index];
 			int prev_prio = p.priority(this);
 			TORRENT_ASSERT(prev_prio < int(m_priority_boundries.size())
 				|| m_dirty);
-			p.downloading = 0;
+			erase_download_piece(i);
 			if (!m_dirty)
 			{
 				int prio = p.priority(this);
