@@ -142,7 +142,7 @@ namespace libtorrent { namespace
 				if (*i != 0)
 				{
 					m_torrent.filesystem().async_read(r, bind(&smart_ban_plugin::on_read_failed_block
-						, shared_from_this(), pb, ((policy::peer*)*i)->addr, _1, _2));
+						, shared_from_this(), pb, ((policy::peer*)*i)->address(), _1, _2));
 				}
 
 				r.start += 16*1024;
@@ -182,7 +182,7 @@ namespace libtorrent { namespace
 			// there is no peer with this address anymore
 			if (range.first == range.second) return;
 
-			policy::peer* p = &range.first->second;
+			policy::peer* p = (*range.first);
 			block_entry e = {p, crc.final()};
 
 			std::map<piece_block, block_entry>::iterator i = m_block_crc.lower_bound(b);
@@ -196,6 +196,10 @@ namespace libtorrent { namespace
 					// from the first time it sent it
 					// at least one of them must be bad
 
+					// verify that this is not a dangling pointer
+					// if the pointer is in the policy's list, it
+					// still live, if it's not, it has been removed
+					// and we can't use this pointer
 					if (!m_torrent.get_policy().has_peer(p)) return;
 
 #ifdef TORRENT_LOGGING
@@ -213,15 +217,16 @@ namespace libtorrent { namespace
 						<< " | crc2: " << e.crc
 						<< " | ip: " << p->ip() << " ]\n";
 #endif
-					p->banned = true;
-					if (p->connection) p->connection->disconnect("banning peer for sending bad data");
+					m_torrent.get_policy().ban_peer(p);
+					if (p->connection) p->connection->disconnect(
+						errors::peer_banned);
 				}
 				// we already have this exact entry in the map
 				// we don't have to insert it
 				return;
 			}
 			
-			m_block_crc.insert(i, std::make_pair(b, e));
+			m_block_crc.insert(i, std::pair<const piece_block, block_entry>(b, e));
 
 #ifdef TORRENT_LOGGING
 			char const* client = "-";
@@ -275,8 +280,9 @@ namespace libtorrent { namespace
 				<< " | bad_crc: " << b.second.crc
 				<< " | ip: " << p->ip() << " ]\n";
 #endif
-			p->banned = true;
-			if (p->connection) p->connection->disconnect("banning peer for sending bad data");
+			m_torrent.get_policy().ban_peer(p);
+			if (p->connection) p->connection->disconnect(
+				errors::peer_banned);
 		}
 		
 		torrent& m_torrent;
