@@ -33,9 +33,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/pch.hpp"
 
 #include <ctime>
-#include <iostream>
-#include <fstream>
-#include <iomanip>
 #include <iterator>
 #include <algorithm>
 #include <set>
@@ -64,6 +61,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/session.hpp"
 #include "libtorrent/aux_/session_impl.hpp"
 #include "libtorrent/invariant_check.hpp"
+#include "libtorrent/utf8.hpp"
 
 #if defined(_MSC_VER) && _MSC_VER < 1300
 namespace std
@@ -122,15 +120,12 @@ namespace libtorrent
 {
 	namespace fs = boost::filesystem;
 
-	namespace
-	{
 #ifndef BOOST_NO_EXCEPTIONS
-		void throw_invalid_handle()
-		{
-			throw invalid_handle();
-		}
-#endif
+	void throw_invalid_handle()
+	{
+		throw libtorrent_exception(errors::invalid_torrent_handle);
 	}
+#endif
 
 #ifdef TORRENT_DEBUG
 
@@ -146,6 +141,12 @@ namespace libtorrent
 		TORRENT_FORWARD_RETURN(torrent_file().info_hash(), empty);
 	}
 
+	int torrent_handle::max_uploads() const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD_RETURN(max_uploads(), 0);
+	}
+
 	void torrent_handle::set_max_uploads(int max_uploads) const
 	{
 		INVARIANT_CHECK;
@@ -159,6 +160,12 @@ namespace libtorrent
 		TORRENT_FORWARD(use_interface(net_interface));
 	}
 
+	int torrent_handle::max_connections() const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD_RETURN(max_connections(), 0);
+	}
+
 	void torrent_handle::set_max_connections(int max_connections) const
 	{
 		INVARIANT_CHECK;
@@ -213,6 +220,25 @@ namespace libtorrent
 		TORRENT_FORWARD(move_storage(save_path));
 	}
 
+#ifndef BOOST_FILESYSTEM_NARROW_ONLY
+	void torrent_handle::move_storage(
+		fs::wpath const& save_path) const
+	{
+		INVARIANT_CHECK;
+		std::string utf8;
+		wchar_utf8(save_path.string(), utf8);
+		TORRENT_FORWARD(move_storage(utf8));
+	}
+
+	void torrent_handle::rename_file(int index, fs::wpath const& new_name) const
+	{
+		INVARIANT_CHECK;
+		std::string utf8;
+		wchar_utf8(new_name.string(), utf8);
+		TORRENT_FORWARD(rename_file(index, utf8));
+	}
+#endif
+
 	void torrent_handle::rename_file(int index, fs::path const& new_name) const
 	{
 		INVARIANT_CHECK;
@@ -233,6 +259,12 @@ namespace libtorrent
 		TORRENT_FORWARD_RETURN(valid_metadata(), false);
 	}
 
+	bool torrent_handle::set_metadata(char const* metadata, int size) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD_RETURN(set_metadata(metadata, size), false);
+	}
+
 	bool torrent_handle::is_seed() const
 	{
 		INVARIANT_CHECK;
@@ -257,6 +289,18 @@ namespace libtorrent
 		TORRENT_FORWARD(pause());
 	}
 
+	void torrent_handle::set_upload_mode(bool b) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(set_upload_mode(b));
+	}
+
+	void torrent_handle::flush_cache() const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(flush_cache());
+	}
+
 	void torrent_handle::save_resume_data() const
 	{
 		INVARIANT_CHECK;
@@ -287,6 +331,12 @@ namespace libtorrent
 		TORRENT_FORWARD(auto_managed(m));
 	}
 
+	void torrent_handle::set_priority(int p) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(set_priority(p));
+	}
+
 	int torrent_handle::queue_position() const
 	{
 		INVARIANT_CHECK;
@@ -332,17 +382,19 @@ namespace libtorrent
 	}
 
 #ifndef TORRENT_NO_DEPRECATE
+#if !TORRENT_NO_FPU
 	void torrent_handle::file_progress(std::vector<float>& progress) const
 	{
 		INVARIANT_CHECK;
 		TORRENT_FORWARD(file_progress(progress));
 	}
+#endif
 #endif
 
-	void torrent_handle::file_progress(std::vector<size_type>& progress) const
+	void torrent_handle::file_progress(std::vector<size_type>& progress, int flags) const
 	{
 		INVARIANT_CHECK;
-		TORRENT_FORWARD(file_progress(progress));
+		TORRENT_FORWARD(file_progress(progress, flags));
 	}
 
 	torrent_status torrent_handle::status() const
@@ -465,7 +517,7 @@ namespace libtorrent
 // ============ end deprecation ===============
 #endif
 
-	std::vector<announce_entry> const& torrent_handle::trackers() const
+	std::vector<announce_entry> torrent_handle::trackers() const
 	{
 		INVARIANT_CHECK;
 		const static std::vector<announce_entry> empty;
@@ -475,20 +527,39 @@ namespace libtorrent
 	void torrent_handle::add_url_seed(std::string const& url) const
 	{
 		INVARIANT_CHECK;
-		TORRENT_FORWARD(add_url_seed(url));
+		TORRENT_FORWARD(add_web_seed(url, web_seed_entry::url_seed));
 	}
 
 	void torrent_handle::remove_url_seed(std::string const& url) const
 	{
 		INVARIANT_CHECK;
-		TORRENT_FORWARD(remove_url_seed(url));
+		TORRENT_FORWARD(remove_web_seed(url, web_seed_entry::url_seed));
 	}
 
 	std::set<std::string> torrent_handle::url_seeds() const
 	{
 		INVARIANT_CHECK;
 		const static std::set<std::string> empty;
-		TORRENT_FORWARD_RETURN(url_seeds(), empty);
+		TORRENT_FORWARD_RETURN(web_seeds(web_seed_entry::url_seed), empty);
+	}
+
+	void torrent_handle::add_http_seed(std::string const& url) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(add_web_seed(url, web_seed_entry::http_seed));
+	}
+
+	void torrent_handle::remove_http_seed(std::string const& url) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(remove_web_seed(url, web_seed_entry::http_seed));
+	}
+
+	std::set<std::string> torrent_handle::http_seeds() const
+	{
+		INVARIANT_CHECK;
+		const static std::set<std::string> empty;
+		TORRENT_FORWARD_RETURN(web_seeds(web_seed_entry::http_seed), empty);
 	}
 
 	void torrent_handle::replace_trackers(
@@ -498,6 +569,24 @@ namespace libtorrent
 		TORRENT_FORWARD(replace_trackers(urls));
 	}
 
+	void torrent_handle::add_tracker(announce_entry const& url) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(add_tracker(url));
+	}
+
+	void torrent_handle::add_piece(int piece, char const* data, int flags) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(add_piece(piece, data, flags));
+	}
+
+	void torrent_handle::read_piece(int piece) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(read_piece(piece));
+	}
+
 	storage_interface* torrent_handle::get_storage_impl() const
 	{
 		INVARIANT_CHECK;
@@ -567,7 +656,7 @@ namespace libtorrent
 		
 		peer_id id;
 		std::fill(id.begin(), id.end(), 0);
-		t->get_policy().peer_from_tracker(adr, id, source, 0);
+		t->get_policy().add_peer(adr, id, source, 0);
 	}
 
 	void torrent_handle::force_reannounce(
@@ -577,6 +666,14 @@ namespace libtorrent
 		TORRENT_FORWARD(force_tracker_request(time_now() + seconds(duration.total_seconds())));
 	}
 
+#ifndef TORRENT_DISABLE_DHT
+	void torrent_handle::force_dht_announce() const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(force_dht_announce());
+	}
+#endif
+
 	void torrent_handle::force_reannounce() const
 	{
 		INVARIANT_CHECK;
@@ -589,6 +686,18 @@ namespace libtorrent
 		TORRENT_FORWARD(scrape_tracker());
 	}
 
+	bool torrent_handle::super_seeding() const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD_RETURN(super_seeding(), false);
+	}
+
+	void torrent_handle::super_seeding(bool on) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(super_seeding(on));
+	}
+
 	void torrent_handle::set_ratio(float ratio) const
 	{
 		INVARIANT_CHECK;
@@ -631,5 +740,11 @@ namespace libtorrent
 		TORRENT_FORWARD(get_download_queue(queue));
 	}
 
+	void torrent_handle::set_piece_deadline(int index, int deadline, int flags) const
+	{
+		INVARIANT_CHECK;
+		TORRENT_FORWARD(set_piece_deadline(index, deadline, flags));
+	}
+
 }
 
