@@ -48,7 +48,9 @@ udp_socket::udp_socket(asio::io_service& ios, udp_socket::callback_t const& c
 	, connection_queue& cc)
 	: m_callback(c)
 	, m_ipv4_sock(ios)
+#if TORRENT_USE_IPV6
 	, m_ipv6_sock(ios)
+#endif
 	, m_bind_port(0)
 	, m_outstanding(0)
 	, m_socks5_sock(ios)
@@ -62,6 +64,7 @@ udp_socket::udp_socket(asio::io_service& ios, udp_socket::callback_t const& c
 #ifdef TORRENT_DEBUG
 	m_magic = 0x1337;
 	m_started = false;
+	m_outstanding_when_aborted = -1;
 #endif
 }
 
@@ -90,8 +93,11 @@ udp_socket::~udp_socket()
 void udp_socket::send(udp::endpoint const& ep, char const* p, int len, error_code& ec)
 {
 	CHECK_MAGIC;
+
+	TORRENT_ASSERT(is_open());
+
 	// if the sockets are closed, the udp_socket is closing too
-	if (!m_ipv4_sock.is_open() && !m_ipv6_sock.is_open()) return;
+	if (!is_open()) return;
 
 	if (m_tunnel_packets)
 	{
@@ -109,10 +115,14 @@ void udp_socket::send(udp::endpoint const& ep, char const* p, int len, error_cod
 		return;
 	}
 
+#if TORRENT_USE_IPV6
 	if (ep.address().is_v4() && m_ipv4_sock.is_open())
+#endif
 		m_ipv4_sock.send_to(asio::buffer(p, len), ep, 0, ec);
+#if TORRENT_USE_IPV6
 	else
 		m_ipv6_sock.send_to(asio::buffer(p, len), ep, 0, ec);
+#endif
 }
 
 void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_transferred)
@@ -145,10 +155,16 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 #ifndef BOOST_NO_EXCEPTIONS
 		try {
 #endif
+
+#if TORRENT_USE_IPV6
 		if (s == &m_ipv4_sock)
+#endif
 			m_callback(e, m_v4_ep, 0, 0);
+#if TORRENT_USE_IPV6
 		else
 			m_callback(e, m_v6_ep, 0, 0);
+#endif
+
 #ifndef BOOST_NO_EXCEPTIONS
 		} catch(std::exception&) {}
 #endif
@@ -175,22 +191,29 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 
 		if (m_abort) return;
 
+#if TORRENT_USE_IPV6
 		if (s == &m_ipv4_sock)
+#endif
 			s->async_receive_from(asio::buffer(m_v4_buf, sizeof(m_v4_buf))
 				, m_v4_ep, boost::bind(&udp_socket::on_read, this, s, _1, _2));
+#if TORRENT_USE_IPV6
 		else
 			s->async_receive_from(asio::buffer(m_v6_buf, sizeof(m_v6_buf))
 				, m_v6_ep, boost::bind(&udp_socket::on_read, this, s, _1, _2));
+#endif
 
 		++m_outstanding;
 #ifdef TORRENT_DEBUG
-	m_started = true;
+		m_started = true;
 #endif
 		return;
 	}
 
+#if TORRENT_USE_IPV6
 	if (s == &m_ipv4_sock)
+#endif
 	{
+
 #ifndef BOOST_NO_EXCEPTIONS
 		try {
 #endif
@@ -216,6 +239,7 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 		s->async_receive_from(asio::buffer(m_v4_buf, sizeof(m_v4_buf))
 			, m_v4_ep, boost::bind(&udp_socket::on_read, this, s, _1, _2));
 	}
+#if TORRENT_USE_IPV6
 	else
 	{
 #ifndef BOOST_NO_EXCEPTIONS
@@ -243,6 +267,7 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 		s->async_receive_from(asio::buffer(m_v6_buf, sizeof(m_v6_buf))
 			, m_v6_ep, boost::bind(&udp_socket::on_read, this, s, _1, _2));
 	}
+#endif
 	++m_outstanding;
 #ifdef TORRENT_DEBUG
 	m_started = true;
@@ -267,10 +292,14 @@ void udp_socket::wrap(udp::endpoint const& ep, char const* p, int len, error_cod
 	iovec[0] = asio::const_buffer(header, h - header);
 	iovec[1] = asio::const_buffer(p, len);
 
+#if TORRENT_USE_IPV6
 	if (m_proxy_addr.address().is_v4() && m_ipv4_sock.is_open())
+#endif
 		m_ipv4_sock.send_to(iovec, m_proxy_addr, 0, ec);
+#if TORRENT_USE_IPV6
 	else
 		m_ipv6_sock.send_to(iovec, m_proxy_addr, 0, ec);
+#endif
 }
 
 // unwrap the UDP packet from the SOCKS5 header
@@ -296,11 +325,13 @@ void udp_socket::unwrap(error_code const& e, char const* buf, int size)
 		// IPv4
 		sender = read_v4_endpoint<udp::endpoint>(p);
 	}
+#if TORRENT_USE_IPV6
 	else if (atyp == 4)
 	{
 		// IPv6
 		sender = read_v6_endpoint<udp::endpoint>(p);
 	}
+#endif
 	else
 	{
 		// domain name not supported
@@ -317,10 +348,15 @@ void udp_socket::close()
 
 	error_code ec;
 	m_ipv4_sock.close(ec);
+#if TORRENT_USE_IPV6
 	m_ipv6_sock.close(ec);
+#endif
 	m_socks5_sock.close(ec);
 	m_resolver.cancel();
 	m_abort = true;
+#ifdef TORRENT_DEBUG
+	m_outstanding_when_aborted = m_outstanding;
+#endif
 	if (m_connection_ticket >= 0)
 	{
 		m_cc.done(m_connection_ticket);
@@ -341,8 +377,13 @@ void udp_socket::bind(udp::endpoint const& ep, error_code& ec)
 	CHECK_MAGIC;
 	mutex_t::scoped_lock l(m_mutex);	
 
+	TORRENT_ASSERT(m_abort == false);
+	if (m_abort) return;
+
 	if (m_ipv4_sock.is_open()) m_ipv4_sock.close(ec);
+#if TORRENT_USE_IPV6
 	if (m_ipv6_sock.is_open()) m_ipv6_sock.close(ec);
+#endif
 
 	if (ep.address().is_v4())
 	{
@@ -352,7 +393,9 @@ void udp_socket::bind(udp::endpoint const& ep, error_code& ec)
 		if (ec) return;
 		m_ipv4_sock.async_receive_from(asio::buffer(m_v4_buf, sizeof(m_v4_buf))
 			, m_v4_ep, boost::bind(&udp_socket::on_read, this, &m_ipv4_sock, _1, _2));
+		++m_outstanding;
 	}
+#if TORRENT_USE_IPV6
 	else
 	{
 		m_ipv6_sock.set_option(v6only(true), ec);
@@ -361,8 +404,9 @@ void udp_socket::bind(udp::endpoint const& ep, error_code& ec)
 		if (ec) return;
 		m_ipv6_sock.async_receive_from(asio::buffer(m_v6_buf, sizeof(m_v6_buf))
 			, m_v6_ep, boost::bind(&udp_socket::on_read, this, &m_ipv6_sock, _1, _2));
+		++m_outstanding;
 	}
-	++m_outstanding;
+#endif
 #ifdef TORRENT_DEBUG
 	m_started = true;
 #endif
@@ -374,10 +418,15 @@ void udp_socket::bind(int port)
 	CHECK_MAGIC;
 	mutex_t::scoped_lock l(m_mutex);	
 
+	TORRENT_ASSERT(m_abort == false);
+	if (m_abort) return;
+
 	error_code ec;
 
 	if (m_ipv4_sock.is_open()) m_ipv4_sock.close(ec);
+#if TORRENT_USE_IPV6
 	if (m_ipv6_sock.is_open()) m_ipv6_sock.close(ec);
+#endif
 
 	m_ipv4_sock.open(udp::v4(), ec);
 	if (!ec)
@@ -390,6 +439,7 @@ void udp_socket::bind(int port)
 		m_started = true;
 #endif
 	}
+#if TORRENT_USE_IPV6
 	m_ipv6_sock.open(udp::v6(), ec);
 	if (!ec)
 	{
@@ -402,6 +452,7 @@ void udp_socket::bind(int port)
 		m_started = true;
 #endif
 	}
+#endif
 	m_bind_port = port;
 }
 
@@ -651,3 +702,69 @@ void udp_socket::connect2(error_code const& e)
 	}
 }
 
+rate_limited_udp_socket::rate_limited_udp_socket(io_service& ios
+	, callback_t const& c, connection_queue& cc)
+	: udp_socket(ios, c, cc)
+	, m_timer(ios)
+	, m_queue_size_limit(200)
+	, m_rate_limit(4000)
+	, m_quota(4000)
+	, m_last_tick(time_now())
+{
+	error_code ec;
+	m_timer.expires_from_now(seconds(1), ec);
+	m_timer.async_wait(boost::bind(&rate_limited_udp_socket::on_tick, this, _1));
+	TORRENT_ASSERT(!ec);
+}
+
+bool rate_limited_udp_socket::send(udp::endpoint const& ep, char const* p, int len, error_code& ec, int flags)
+{
+	if (m_quota < len)
+	{
+		// bit 1 of flags means "don't drop"
+		if (int(m_queue.size()) >= m_queue_size_limit && (flags & 1) == 0)
+			return false;
+		m_queue.push_back(queued_packet());
+		queued_packet& qp = m_queue.back();
+		qp.ep = ep;
+		qp.buf.insert(qp.buf.begin(), p, p + len);
+		return true;
+	}
+
+	m_quota -= len;
+	udp_socket::send(ep, p, len, ec);
+	return true;
+}
+
+void rate_limited_udp_socket::on_tick(error_code const& e)
+{
+	if (e) return;
+	if (is_closed()) return;
+	error_code ec;
+	ptime now = time_now();
+	m_timer.expires_at(now + seconds(1), ec);
+	m_timer.async_wait(boost::bind(&rate_limited_udp_socket::on_tick, this, _1));
+
+	time_duration delta = now - m_last_tick;
+	m_last_tick = now;
+	if (m_quota < m_rate_limit) m_quota += m_rate_limit * total_milliseconds(delta) / 1000;
+
+	if (m_queue.empty()) return;
+
+	while (!m_queue.empty() && int(m_queue.front().buf.size()) >= m_quota)
+	{
+		queued_packet const& p = m_queue.front();
+		m_quota -= p.buf.size();
+		error_code ec;
+		udp_socket::send(p.ep, &p.buf[0], p.buf.size(), ec);
+		m_queue.pop_front();
+	}
+}
+
+void rate_limited_udp_socket::close()
+{
+	error_code ec;
+	m_timer.cancel(ec);
+	udp_socket::close();
+}
+
