@@ -53,10 +53,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/thread/mutex.hpp>
 #include <cstdlib>
 
-#if (defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)) && !defined(TORRENT_UPNP_LOGGING)
-#define TORRENT_UPNP_LOGGING
-#endif
-
 using boost::bind;
 using namespace libtorrent;
 
@@ -64,13 +60,15 @@ static error_code ec;
 
 upnp::upnp(io_service& ios, connection_queue& cc
 	, address const& listen_interface, std::string const& user_agent
-	, portmap_callback_t const& cb, bool ignore_nonrouters, void* state)
-	:  m_user_agent(user_agent)
+	, portmap_callback_t const& cb, log_callback_t const& lcb
+	, bool ignore_nonrouters, void* state)
+	: m_user_agent(user_agent)
 	, m_callback(cb)
+	, m_log_callback(lcb)
 	, m_retry_count(0)
 	, m_io_service(ios)
 	, m_socket(ios, udp::endpoint(address_v4::from_string("239.255.255.250", ec), 1900)
-		, bind(&upnp::on_reply, self(), _1, _2, _3), false)
+		, bind(&upnp::on_reply, self(), _1, _2, _3))
 	, m_broadcast_timer(ios)
 	, m_refresh_timer(ios)
 	, m_disabled(false)
@@ -78,10 +76,7 @@ upnp::upnp(io_service& ios, connection_queue& cc
 	, m_ignore_non_routers(ignore_nonrouters)
 	, m_cc(cc)
 {
-#ifdef TORRENT_UPNP_LOGGING
-	m_log.open("upnp.log", std::ios::in | std::ios::out | std::ios::trunc);
-#endif
-	m_retry_count = 0;
+	TORRENT_ASSERT(cb);
 
 	if (state)
 	{
@@ -111,11 +106,20 @@ upnp::~upnp()
 void upnp::discover_device()
 {
 	mutex_t::scoped_lock l(m_mutex);
+	if (m_socket.num_send_sockets() == 0)
+		log("No network interfaces to broadcast to", l);
 
-	discover_device_impl();
+	discover_device_impl(l);
 }
 
-void upnp::discover_device_impl()
+void upnp::log(char const* msg, mutex_t::scoped_lock& l)
+{
+	l.unlock();
+	m_log_callback(msg);
+	l.lock();
+}
+
+void upnp::discover_device_impl(mutex_t::scoped_lock& l)
 {
 	const char msearch[] = 
 		"M-SEARCH * HTTP/1.1\r\n"
@@ -134,12 +138,10 @@ void upnp::discover_device_impl()
 
 	if (ec)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " ==> Broadcast FAILED: " << ec.message() << std::endl
-			<< "aborting" << std::endl;
-#endif
-		disable(ec.message().c_str());
+		char msg[200];
+		snprintf(msg, sizeof(msg), "broadcast failed: %s. Aborting.", ec.message().c_str());
+		log(msg, l);
+		disable(ec, l);
 		return;
 	}
 
@@ -148,10 +150,7 @@ void upnp::discover_device_impl()
 	m_broadcast_timer.async_wait(bind(&upnp::resend_request
 		, self(), _1));
 
-#ifdef TORRENT_UPNP_LOGGING
-	m_log << time_now_string()
-		<< " ==> Broadcasting search for rootdevice" << std::endl;
-#endif
+	log("broadcasting search for rootdevice", l);
 }
 
 // returns a reference to a mapping or -1 on failure
@@ -159,14 +158,11 @@ int upnp::add_mapping(upnp::protocol_type p, int external_port, int local_port)
 {
 	mutex_t::scoped_lock l(m_mutex);
 
-#ifdef TORRENT_UPNP_LOGGING
-	m_log << time_now_string()
-		<< " *** add mapping [ proto: " << (p == tcp?"tcp":"udp")
-		<< " ext_port: " << external_port
-		<< " local_port :" << local_port << " ]";
-	if (m_disabled) m_log << " DISABLED";
-	m_log << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "adding port map: [ protocol: %s ext_port: %u "
+		"local_port: %u ] %s", (p == tcp?"tcp":"udp"), external_port
+		, local_port, m_disabled ? "DISABLED": "");
+	log(msg, l);
 	if (m_disabled) return -1;
 
 	std::vector<global_mapping_t>::iterator i = std::find_if(
@@ -200,7 +196,7 @@ int upnp::add_mapping(upnp::protocol_type p, int external_port, int local_port)
 		m.external_port = external_port;
 		m.local_port = local_port;
 
-		if (d.service_namespace) update_map(d, mapping_index);
+		if (d.service_namespace) update_map(d, mapping_index, l);
 	}
 
 	return mapping_index;
@@ -214,13 +210,11 @@ void upnp::delete_mapping(int mapping)
 
 	global_mapping_t& m = m_mappings[mapping];
 
-#ifdef TORRENT_UPNP_LOGGING
-	m_log << time_now_string()
-		<< " *** delete mapping [ proto: " << (m.protocol == tcp?"tcp":"udp")
-		<< " ext_port:" << m.external_port
-		<< " local_port:" << m.local_port << " ]";
-	m_log << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "deleting port map: [ protocol: %s ext_port: %u "
+		"local_port: %u ]", (m.protocol == tcp?"tcp":"udp"), m.external_port
+		, m.local_port);
+	log(msg, l);
 
 	if (m.protocol == none) return;
 	
@@ -233,10 +227,22 @@ void upnp::delete_mapping(int mapping)
 		TORRENT_ASSERT(mapping < int(d.mapping.size()));
 		d.mapping[mapping].action = mapping_t::action_delete;
 
-		if (d.service_namespace) update_map(d, mapping);
+		if (d.service_namespace) update_map(d, mapping, l);
 	}
 }
 
+bool upnp::get_mapping(int index, int& local_port, int& external_port, int& protocol) const
+{
+	TORRENT_ASSERT(index < int(m_mappings.size()) && index >= 0);
+	if (index >= int(m_mappings.size()) || index < 0) return false;
+	global_mapping_t const& m = m_mappings[index];
+	if (m.protocol == none) return false;
+	local_port = m.local_port;
+	external_port = m.external_port;
+	protocol = m.protocol;
+	return true;
+}
+
 void upnp::resend_request(error_code const& e)
 {
 	if (e) return;
@@ -250,18 +256,13 @@ void upnp::resend_request(error_code const& e)
 	if (m_retry_count < 12
 		&& (m_devices.empty() || m_retry_count < 4))
 	{
-		discover_device_impl();
+		discover_device_impl(l);
 		return;
 	}
 
 	if (m_devices.empty())
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " *** Got no response in 12 retries. Giving up, "
-			"disabling UPnP." << std::endl;
-#endif
-		disable("no UPnP router found");
+		disable(errors::no_router, l);
 		return;
 	}
 	
@@ -274,28 +275,29 @@ void upnp::resend_request(error_code const& e)
 			// ask for it
 			rootdevice& d = const_cast<rootdevice&>(*i);
 			TORRENT_ASSERT(d.magic == 1337);
+#ifndef BOOST_NO_EXCEPTIONS
 			try
 			{
-#ifdef TORRENT_UPNP_LOGGING
-				m_log << time_now_string()
-					<< " ==> connecting to " << d.url << std::endl;
 #endif
+				char msg[200];
+				snprintf(msg, sizeof(msg), "connecting to: %s", d.url.c_str());
+				log(msg, l);
 				if (d.upnp_connection) d.upnp_connection->close();
 				d.upnp_connection.reset(new http_connection(m_io_service
 					, m_cc, bind(&upnp::on_upnp_xml, self(), _1, _2
 					, boost::ref(d), _5)));
 				d.upnp_connection->get(d.url, seconds(30), 1);
+#ifndef BOOST_NO_EXCEPTIONS
 			}
 			catch (std::exception& e)
 			{
 				(void)e;
-#ifdef TORRENT_UPNP_LOGGING
-				m_log << time_now_string()
-					<< " *** Connection failed to: " << d.url
-					<< " " << e.what() << std::endl;
-#endif
+				char msg[200];
+				snprintf(msg, sizeof(msg), "connection failed to: %s %s", d.url.c_str(), e.what());
+				log(msg, l);
 				d.disabled = true;
 			}
+#endif
 		}
 	}
 }
@@ -338,55 +340,61 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 	error_code ec;
 	if (!in_local_network(m_io_service, from.address(), ec))
 	{
-#ifdef TORRENT_UPNP_LOGGING
 		if (ec)
 		{
-			m_log << time_now_string() << " <== (" << from << ") error: "
-				<< ec.message() << std::endl;
+			char msg[200];
+			snprintf(msg, sizeof(msg), "when receiving response from: %s: %s"
+				, print_endpoint(from).c_str(), ec.message().c_str());
+			log(msg, l);
 		}
 		else
 		{
-			m_log << time_now_string() << " <== (" << from << ") UPnP device "
-				"ignored because it's not on our local network ";
+			char msg[200];
+			int num_chars = snprintf(msg, sizeof(msg)
+				, "ignoring response from: %s. IP is not on local network. "
+				, print_endpoint(from).c_str());
+			log(msg, l);
+
 			std::vector<ip_interface> net = enum_net_interfaces(m_io_service, ec);
 			for (std::vector<ip_interface>::const_iterator i = net.begin()
 				, end(net.end()); i != end; ++i)
 			{
-				m_log << "(" << i->interface_address << ", " << i->netmask << ") ";
+				num_chars += snprintf(msg + num_chars, sizeof(msg) - num_chars, "(%s,%s) "
+					, print_address(i->interface_address).c_str(), print_address(i->netmask).c_str());
 			}
-			m_log << std::endl;
+			log(msg, l);
 		}
-#endif
 		return;
 	} 
 
 	if (m_ignore_non_routers)
 	{
 		std::vector<ip_route> routes = enum_routes(m_io_service, ec);
-	
 		if (std::find_if(routes.begin(), routes.end()
 			, bind(&ip_route::gateway, _1) == from.address()) == routes.end())
 		{
 			// this upnp device is filtered because it's not in the
 			// list of configured routers
-#ifdef TORRENT_UPNP_LOGGING
 			if (ec)
 			{
-				m_log << time_now_string() << " <== (" << from << ") error: "
-					<< ec.message() << std::endl;
+				char msg[200];
+				snprintf(msg, sizeof(msg), "when receiving response from: %s: %s"
+					, print_endpoint(from).c_str(), ec.message().c_str());
+				log(msg, l);
 			}
 			else
 			{
-				m_log << time_now_string() << " <== (" << from << ") UPnP device "
-					"ignored because it's not a router on our network ";
+				char msg[200];
+				int num_chars = snprintf(msg, sizeof(msg), "ignoring response from: %s: IP is not a router. "
+					, print_endpoint(from).c_str());
 				for (std::vector<ip_route>::const_iterator i = routes.begin()
 					, end(routes.end()); i != end; ++i)
 				{
-					m_log << "(" << i->gateway << ", " << i->netmask << ") ";
+					num_chars += snprintf(msg + num_chars, sizeof(msg) - num_chars, "(%s,%s) "
+						, print_address(i->gateway).c_str(), print_address(i->netmask).c_str());
 				}
-				m_log << std::endl;
+				log(msg, l);
 			}
-#endif
 			return;
 		}
 	}
@@ -397,46 +405,48 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 		, buffer + bytes_transferred), error);
 	if (error)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string() << " <== (" << from << ") Rootdevice "
-			"responded with incorrect HTTP packet. Ignoring device" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "received malformed HTTP from: %s"
+			, print_endpoint(from).c_str());
+		log(msg, l);
 		return;
 	}
 
 	if (p.status_code() != 200 && p.method() != "notify")
 	{
-#ifdef TORRENT_UPNP_LOGGING
 		if (p.method().empty())
-			m_log << time_now_string()
-				<< " <== (" << from << ") Device responded with HTTP status: " << p.status_code()
-				<< ". Ignoring device" << std::endl;
+		{
+			char msg[200];
+			snprintf(msg, sizeof(msg), "HTTP status %u from %s"
+				, p.status_code(), print_endpoint(from).c_str());
+			log(msg, l);
+		}
 		else
-			m_log << time_now_string()
-				<< " <== (" << from << ") Device with HTTP method: " << p.method()
-				<< ". Ignoring device" << std::endl;
-#endif
+		{
+			char msg[200];
+			snprintf(msg, sizeof(msg), "HTTP method %s from %s"
+				, p.method().c_str(), print_endpoint(from).c_str());
+			log(msg, l);
+		}
 		return;
 	}
 
 	if (!p.header_finished())
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== (" << from << ") Rootdevice responded with incomplete HTTP "
-			"packet. Ignoring device" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "incomplete HTTP packet from %s"
+			, print_endpoint(from).c_str());
+		log(msg, l);
 		return;
 	}
 
 	std::string url = p.header("location");
 	if (url.empty())
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== (" << from << ") Rootdevice response is missing a location header. "
-			"Ignoring device" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "missing location header from %s"
+			, print_endpoint(from).c_str());
+		log(msg, l);
 		return;
 	}
 
@@ -450,18 +460,17 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 
 		std::string protocol;
 		std::string auth;
-		char const* error;
+		error_code ec;
 		// we don't have this device in our list. Add it
-		boost::tie(protocol, auth, d.hostname, d.port, d.path, error)
-			= parse_url_components(d.url);
+		boost::tie(protocol, auth, d.hostname, d.port, d.path)
+			= parse_url_components(d.url, ec);
 
-		if (error)
+		if (ec)
 		{
-#ifdef TORRENT_UPNP_LOGGING
-			m_log << time_now_string()
-				<< " <== (" << from << ") Rootdevice advertized an invalid url: '" << d.url
-				<< "'. " << error << ". Ignoring device" << std::endl;
-#endif
+			char msg[200];
+			snprintf(msg, sizeof(msg), "invalid URL %s from %s: %s"
+				, d.url.c_str(), print_endpoint(from).c_str(), ec.message().c_str());
+			log(msg, l);
 			return;
 		}
 
@@ -470,36 +479,33 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 
 		if (protocol != "http")
 		{
-#ifdef TORRENT_UPNP_LOGGING
-			m_log << time_now_string()
-				<< " <== (" << from << ") Rootdevice uses unsupported protocol: '" << protocol
-				<< "'. Ignoring device" << std::endl;
-#endif
+			char msg[200];
+			snprintf(msg, sizeof(msg), "unsupported protocol %s from %s"
+				, protocol.c_str(), print_endpoint(from).c_str());
+			log(msg, l);
 			return;
 		}
 
 		if (d.port == 0)
 		{
-#ifdef TORRENT_UPNP_LOGGING
-			m_log << time_now_string()
-				<< " <== (" << from << ") Rootdevice responded with a url with port 0. "
-				"Ignoring device" << std::endl;
-#endif
+			char msg[200];
+			snprintf(msg, sizeof(msg), "URL with port 0 from %s"
+				, print_endpoint(from).c_str());
+			log(msg, l);
 			return;
 		}
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== (" << from << ") Found rootdevice: " << d.url
-			<< " total: " << m_devices.size() << std::endl;
-#endif
+
+		char msg[200];
+		snprintf(msg, sizeof(msg), "found rootdevice: %s (%d)"
+			, d.url.c_str(), int(m_devices.size()));
+		log(msg, l);
 
 		if (m_devices.size() >= 50)
 		{
-#ifdef TORRENT_UPNP_LOGGING
-			m_log << time_now_string()
-				<< " <== (" << from << ") Too many devices (" << m_devices.size() << "), "
-				"ignoring: " << d.url << std::endl;
-#endif
+			char msg[200];
+			snprintf(msg, sizeof(msg), "too many rootdevices: (%d). Ignoring %s"
+				, int(m_devices.size()), d.url.c_str());
+			log(msg, l);
 			return;
 		}
 
@@ -517,6 +523,7 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 		boost::tie(i, boost::tuples::ignore) = m_devices.insert(d);
 	}
 
+
 	if (!m_devices.empty())
 	{
 		for (std::set<rootdevice>::iterator i = m_devices.begin()
@@ -532,10 +539,11 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 				try
 				{
 #endif
-#ifdef TORRENT_UPNP_LOGGING
-					m_log << time_now_string()
-						<< " ==> connecting to " << d.url << std::endl;
-#endif
+					char msg[200];
+					snprintf(msg, sizeof(msg), "connecting to: %s"
+						, d.url.c_str());
+					log(msg, l);
+
 					if (d.upnp_connection) d.upnp_connection->close();
 					d.upnp_connection.reset(new http_connection(m_io_service
 						, m_cc, bind(&upnp::on_upnp_xml, self(), _1, _2
@@ -546,11 +554,11 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 				catch (std::exception& e)
 				{
 					(void)e;
-#ifdef TORRENT_UPNP_LOGGING
-					m_log << time_now_string()
-						<< " *** Connection failed to: " << d.url
-						<< " " << e.what() << std::endl;
-#endif
+
+					char msg[200];
+					snprintf(msg, sizeof(msg), "connection failed to: %s %s"
+						, d.url.c_str(), e.what());
+					log(msg, l);
 					d.disabled = true;
 				}
 #endif
@@ -559,27 +567,28 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 	}
 }
 
-void upnp::post(upnp::rootdevice const& d, std::string const& soap
-	, std::string const& soap_action)
+void upnp::post(upnp::rootdevice const& d, char const* soap
+	, char const* soap_action, mutex_t::scoped_lock& l)
 {
 	TORRENT_ASSERT(d.magic == 1337);
 	TORRENT_ASSERT(d.upnp_connection);
 
-	std::stringstream header;
-	
-	header << "POST " << d.path << " HTTP/1.0\r\n"
-		"Host: " << d.hostname << ":" << d.port << "\r\n"
+	char header[2048];
+	snprintf(header, sizeof(header), "POST %s HTTP/1.0\r\n"
+		"Host: %s:%u\r\n"
 		"Content-Type: text/xml; charset=\"utf-8\"\r\n"
-		"Content-Length: " << soap.size() << "\r\n"
-		"Soapaction: \"" << d.service_namespace << "#" << soap_action << "\"\r\n\r\n" << soap;
+		"Content-Length: %d\r\n"
+		"Soapaction: \"%s#%s\"\r\n\r\n"
+		"%s"
+		, d.path.c_str(), d.hostname.c_str(), d.port
+		, int(strlen(soap)), d.service_namespace, soap_action
+		, soap);
 
-	d.upnp_connection->sendbuffer = header.str();
+	d.upnp_connection->sendbuffer = header;
 
-#ifdef TORRENT_UPNP_LOGGING
-	m_log << time_now_string()
-		<< " ==> sending: " << header.str() << std::endl;
-#endif
-	
+	char msg[1024];
+	snprintf(msg, sizeof(msg), "sending: %s", header);
+	log(msg, l);
 }
 
 void upnp::create_port_mapping(http_connection& c, rootdevice& d, int i)
@@ -591,43 +600,46 @@ void upnp::create_port_mapping(http_connection& c, rootdevice& d, int i)
 	if (!d.upnp_connection)
 	{
 		TORRENT_ASSERT(d.disabled);
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string() << " *** mapping (" << i
-			<< ") aborted" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "mapping %u aborted", i);
+		log(msg, l);
 		return;
 	}
 	
-	std::string soap_action = "AddPortMapping";
+	char const* soap_action = "AddPortMapping";
 
-	std::stringstream soap;
-	
-	soap << "<?xml version=\"1.0\"?>\n"
+	std::string local_endpoint = print_address(c.socket().local_endpoint(ec).address());
+
+	char soap[2048];
+	error_code ec;
+	snprintf(soap, sizeof(soap), "<?xml version=\"1.0\"?>\n"
 		"<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
 		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
-		"<s:Body><u:" << soap_action << " xmlns:u=\"" << d.service_namespace << "\">";
-
-	error_code ec;
-	soap << "<NewRemoteHost></NewRemoteHost>"
-		"<NewExternalPort>" << d.mapping[i].external_port << "</NewExternalPort>"
-		"<NewProtocol>" << (d.mapping[i].protocol == udp ? "UDP" : "TCP") << "</NewProtocol>"
-		"<NewInternalPort>" << d.mapping[i].local_port << "</NewInternalPort>"
-		"<NewInternalClient>" << c.socket().local_endpoint(ec).address() << "</NewInternalClient>"
+		"<s:Body><u:%s xmlns:u=\"%s\">"
+		"<NewRemoteHost></NewRemoteHost>"
+		"<NewExternalPort>%u</NewExternalPort>"
+		"<NewProtocol>%s</NewProtocol>"
+		"<NewInternalPort>%u</NewInternalPort>"
+		"<NewInternalClient>%s</NewInternalClient>"
 		"<NewEnabled>1</NewEnabled>"
-		"<NewPortMappingDescription>" << m_user_agent << " at " <<
-			c.socket().local_endpoint(ec).address() << ":" << to_string(d.mapping[i].local_port).elems
-			<< "</NewPortMappingDescription>"
-		"<NewLeaseDuration>" << d.lease_duration << "</NewLeaseDuration>";
-	soap << "</u:" << soap_action << "></s:Body></s:Envelope>";
+		"<NewPortMappingDescription>%s at %s:%d</NewPortMappingDescription>"
+		"<NewLeaseDuration>%u</NewLeaseDuration>"
+		"</u:%s></s:Body></s:Envelope>"
+		, soap_action, d.service_namespace, d.mapping[i].external_port
+		, (d.mapping[i].protocol == udp ? "UDP" : "TCP")
+		, d.mapping[i].local_port
+		, local_endpoint.c_str()
+		, m_user_agent.c_str(), local_endpoint.c_str(), d.mapping[i].local_port
+		, d.lease_duration, soap_action);
 
-	post(d, soap.str(), soap_action);
+	post(d, soap, soap_action, l);
 }
 
-void upnp::next(rootdevice& d, int i)
+void upnp::next(rootdevice& d, int i, mutex_t::scoped_lock& l)
 {
 	if (i < num_mappings() - 1)
 	{
-		update_map(d, i + 1);
+		update_map(d, i + 1, l);
 	}
 	else
 	{
@@ -636,11 +648,11 @@ void upnp::next(rootdevice& d, int i)
 			, boost::bind(&mapping_t::action, _1) != int(mapping_t::action_none));
 		if (i == d.mapping.end()) return;
 
-		update_map(d, i - d.mapping.begin());
+		update_map(d, i - d.mapping.begin(), l);
 	}
 }
 
-void upnp::update_map(rootdevice& d, int i)
+void upnp::update_map(rootdevice& d, int i, mutex_t::scoped_lock& l)
 {
 	TORRENT_ASSERT(d.magic == 1337);
 	TORRENT_ASSERT(i < int(d.mapping.size()));
@@ -655,30 +667,27 @@ void upnp::update_map(rootdevice& d, int i)
 	if (m.action == mapping_t::action_none
 		|| m.protocol == none)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		if (m.protocol != none)
-			m_log << time_now_string() << " *** mapping (" << i
-				<< ") does not need update, skipping" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "mapping %u does not need updating, skipping", i);
+		log(msg, l);
 		m.action = mapping_t::action_none;
-		next(d, i);
+		next(d, i, l);
 		return;
 	}
 
 	TORRENT_ASSERT(!d.upnp_connection);
 	TORRENT_ASSERT(d.service_namespace);
 
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " ==> connecting to " << d.hostname << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "connecting to %s", d.hostname.c_str());
+	log(msg, l);
 	if (m.action == mapping_t::action_add)
 	{
 		if (m.failcount > 5)
 		{
 			m.action = mapping_t::action_none;
 			// giving up
-			next(d, i);
+			next(d, i, l);
 			return;
 		}
 
@@ -714,53 +723,37 @@ void upnp::delete_port_mapping(rootdevice& d, int i)
 	if (!d.upnp_connection)
 	{
 		TORRENT_ASSERT(d.disabled);
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string() << " *** unmapping (" << i
-			<< ") aborted" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "unmapping %u aborted", i);
+		log(msg, l);
 		return;
 	}
 
-	std::stringstream soap;
-	
-	std::string soap_action = "DeletePortMapping";
+	char const* soap_action = "DeletePortMapping";
 
-	soap << "<?xml version=\"1.0\"?>\n"
+	char soap[2048];
+	error_code ec;
+	snprintf(soap, sizeof(soap), "<?xml version=\"1.0\"?>\n"
 		"<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
 		"s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
-		"<s:Body><u:" << soap_action << " xmlns:u=\"" << d.service_namespace << "\">";
-
-	soap << "<NewRemoteHost></NewRemoteHost>"
-		"<NewExternalPort>" << d.mapping[i].external_port << "</NewExternalPort>"
-		"<NewProtocol>" << (d.mapping[i].protocol == udp ? "UDP" : "TCP") << "</NewProtocol>";
-	soap << "</u:" << soap_action << "></s:Body></s:Envelope>";
+		"<s:Body><u:%s xmlns:u=\"%s\">"
+		"<NewExternalPort>%u</NewExternalPort>"
+		"<NewProtocol>%s</NewProtocol>"
+		"</u:%s></s:Body></s:Envelope>"
+		, soap_action, d.service_namespace
+		, d.mapping[i].external_port
+		, (d.mapping[i].protocol == udp ? "UDP" : "TCP")
+		, soap_action);
 	
-	post(d, soap.str(), soap_action);
+	post(d, soap, soap_action, l);
 }
 
 namespace
 {
-	char tolower(char c)
-	{
-		if (c >= 'A' && c <= 'Z') return c + ('a' - 'A');
-		return c;
-	}
-
 	void copy_tolower(std::string& dst, char const* src)
 	{
 		dst.clear();
-		while (*src) dst.push_back(tolower(*src++));
-	}
-	
-	bool string_equal_nocase(char const* lhs, char const* rhs)
-	{
-		while (tolower(*lhs) == tolower(*rhs))
-		{
-			if (*lhs == 0) return true;
-			++lhs;
-			++rhs;
-		}
-		return false;
+		while (*src) dst.push_back(to_lower(*src++));
 	}
 }
 
@@ -786,15 +779,15 @@ struct parse_state
 	{
 		std::list<std::string>::reverse_iterator i = tag_stack.rbegin();
 		if (i == tag_stack.rend()) return false;
-		if (!string_equal_nocase(i->c_str(), str2)) return false;
+		if (!string_equal_no_case(i->c_str(), str2)) return false;
 		++i;
 		if (i == tag_stack.rend()) return false;
-		if (!string_equal_nocase(i->c_str(), str1)) return false;
+		if (!string_equal_no_case(i->c_str(), str1)) return false;
 		return true;
 	}
 };
 
-void find_control_url(int type, char const* string, parse_state& state)
+TORRENT_EXPORT void find_control_url(int type, char const* string, parse_state& state)
 {
 	if (type == xml_start_tag)
 	{
@@ -819,7 +812,7 @@ void find_control_url(int type, char const* string, parse_state& state)
 //		std::cout << " " << string << std::endl;
 		if (!state.in_service && state.top_tags("service", "servicetype"))
 		{
-			if (string_equal_nocase(string, state.service_type))
+			if (string_equal_no_case(string, state.service_type))
 				state.in_service = true;
 		}
 		else if (state.in_service && state.top_tags("service", "controlurl"))
@@ -854,31 +847,30 @@ void upnp::on_upnp_xml(error_code const& e
 
 	if (e && e != asio::error::eof)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== (" << d.url << ") error while fetching control url: "
-			<< e.message() << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while fetching control url from: %s: %s"
+			, d.url.c_str(), e.message().c_str());
+		log(msg, l);
 		d.disabled = true;
 		return;
 	}
 
 	if (!p.header_finished())
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== (" << d.url << ") error while fetching control url: incomplete http message" << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while fetching control url from: %s: incomplete HTTP message"
+			, d.url.c_str());
+		log(msg, l);
 		d.disabled = true;
 		return;
 	}
 
 	if (p.status_code() != 200)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== (" << d.url << ") error while fetching control url: " << p.message() << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while fetching control url from: %s: %s"
+			, d.url.c_str(), p.message().c_str());
+		log(msg, l);
 		d.disabled = true;
 		return;
 	}
@@ -906,11 +898,10 @@ void upnp::on_upnp_xml(error_code const& e
 		}
 		else
 		{
-#ifdef TORRENT_UPNP_LOGGING
-			m_log << time_now_string()
-				<< " <== (" << d.url << ") Rootdevice response, did not find "
-				"a port mapping interface" << std::endl;
-#endif
+			char msg[200];
+			snprintf(msg, sizeof(msg), "could not find a port mapping interface in response from: %s"
+				, d.url.c_str());
+			log(msg, l);
 			d.disabled = true;
 			return;
 		}
@@ -929,38 +920,39 @@ void upnp::on_upnp_xml(error_code const& e
 
 	std::string protocol;
 	std::string auth;
-	char const* error;
+	error_code ec;
 	if (!d.control_url.empty() && d.control_url[0] == '/')
 	{
-		boost::tie(protocol, auth, d.hostname, d.port, d.path, error)
-			= parse_url_components(d.url);
+		boost::tie(protocol, auth, d.hostname, d.port, d.path)
+			= parse_url_components(d.url, ec);
 		d.control_url = protocol + "://" + d.hostname + ":"
 			+ to_string(d.port).elems + s.control_url;
 	}
 
-#ifdef TORRENT_UPNP_LOGGING
-	m_log << time_now_string()
-		<< " <== (" << d.url << ") Rootdevice response, found control URL: " << d.control_url
-		<< " urlbase: " << s.url_base << " namespace: " << d.service_namespace << std::endl;
-#endif
+	char msg[200];
+	snprintf(msg, sizeof(msg), "found control URL: %s namespace %s "
+		"urlbase: %s in response from %s"
+		, d.control_url.c_str(), d.service_namespace
+		, s.url_base.c_str(), d.url.c_str());
+	log(msg, l);
 
-	boost::tie(protocol, auth, d.hostname, d.port, d.path, error)
-		= parse_url_components(d.control_url);
+	boost::tie(protocol, auth, d.hostname, d.port, d.path)
+		= parse_url_components(d.control_url, ec);
 
-	if (error)
+	if (ec)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " *** Failed to parse URL '" << d.control_url << "': " << error << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "failed to parse URL '%s': %s"
+			, d.control_url.c_str(), ec.message().c_str());
+		log(msg, l);
 		d.disabled = true;
 		return;
 	}
 
-	if (num_mappings() > 0) update_map(d, 0);
+	if (num_mappings() > 0) update_map(d, 0, l);
 }
 
-void upnp::disable(char const* msg)
+void upnp::disable(error_code const& ec, mutex_t::scoped_lock& l)
 {
 	m_disabled = true;
 
@@ -970,13 +962,18 @@ void upnp::disable(char const* msg)
 	{
 		if (i->protocol == none) continue;
 		i->protocol = none;
-		m_callback(i - m_mappings.begin(), 0, msg);
+		l.unlock();
+		m_callback(i - m_mappings.begin(), 0, ec);
+		l.lock();
 	}
 	
-//	m_devices.clear();
-	error_code ec;
-	m_broadcast_timer.cancel(ec);
-	m_refresh_timer.cancel(ec);
+	// we cannot clear the devices since there
+	// might be outstanding requests relying on
+	// the device entry being present when they
+	// complete
+	error_code e;
+	m_broadcast_timer.cancel(e);
+	m_refresh_timer.cancel(e);
 	m_socket.close();
 }
 
@@ -1003,10 +1000,7 @@ namespace
 			state.exit = true;
 		}
 	}
-}
 
-namespace
-{
 	struct error_code_t
 	{
 		int code;
@@ -1015,7 +1009,8 @@ namespace
 	
 	error_code_t error_codes[] =
 	{
-		{402, "Invalid Arguments"}
+		{0, "no error"}
+		, {402, "Invalid Arguments"}
 		, {501, "Action Failed"}
 		, {714, "The specified value does not exist in the array"}
 		, {715, "The source IP address cannot be wild-carded"}
@@ -1032,6 +1027,42 @@ namespace
 
 }
 
+#if BOOST_VERSION >= 103500
+
+
+const char* upnp_error_category::name() const
+{
+	return "UPnP error";
+}
+
+std::string upnp_error_category::message(int ev) const
+{
+	int num_errors = sizeof(error_codes) / sizeof(error_codes[0]);
+	error_code_t* end = error_codes + num_errors;
+	error_code_t tmp = {ev, 0};
+	error_code_t* e = std::lower_bound(error_codes, end, tmp
+		, bind(&error_code_t::code, _1) < bind(&error_code_t::code, _2));
+	if (e != end && e->code == ev)
+	{
+		return e->msg;
+	}
+	return "unknown UPnP error";
+}
+
+namespace libtorrent
+{
+	TORRENT_EXPORT upnp_error_category upnp_category;
+}
+
+#else
+
+namespace libtorrent
+{
+	TORRENT_EXPORT ::asio::error::error_category upnp_category(21);
+}
+
+#endif
+
 void upnp::on_upnp_map_response(error_code const& e
 	, libtorrent::http_parser const& p, rootdevice& d, int mapping
 	, http_connection& c)
@@ -1049,10 +1080,10 @@ void upnp::on_upnp_map_response(error_code const& e
 
 	if (e && e != asio::error::eof)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== error while adding portmap: " << e.message() << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while adding port map: %s"
+			, e.message().c_str());
+		log(msg, l);
 		d.disabled = true;
 		return;
 	}
@@ -1078,11 +1109,8 @@ void upnp::on_upnp_map_response(error_code const& e
 
 	if (!p.header_finished())
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== error while adding portmap: incomplete http message" << std::endl;
-#endif
-		next(d, mapping);
+		log("error while adding port map: incomplete http message", l);
+		next(d, mapping, l);
 		return;
 	}
 
@@ -1093,13 +1121,13 @@ void upnp::on_upnp_map_response(error_code const& e
 	xml_parse((char*)p.get_body().begin, (char*)p.get_body().end
 		, bind(&find_error_code, _1, _2, boost::ref(s)));
 
-#ifdef TORRENT_UPNP_LOGGING
 	if (s.error_code != -1)
 	{
-		m_log << time_now_string()
-			<< " <== got error message: " << s.error_code << std::endl;
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while adding port map, code: %u"
+			, s.error_code);
+		log(msg, l);
 	}
-#endif
 	
 	mapping_t& m = d.mapping[mapping];
 
@@ -1109,7 +1137,7 @@ void upnp::on_upnp_map_response(error_code const& e
 		d.lease_duration = 0;
 		m.action = mapping_t::action_add;
 		++m.failcount;
-		update_map(d, mapping);
+		update_map(d, mapping, l);
 		return;
 	}
 	else if (s.error_code == 718 || s.error_code == 727)
@@ -1121,10 +1149,10 @@ void upnp::on_upnp_map_response(error_code const& e
 			m.external_port = 0;
 			m.action = mapping_t::action_add;
 			++m.failcount;
-			update_map(d, mapping);
+			update_map(d, mapping, l);
 			return;
 		}
-		return_error(mapping, s.error_code);
+		return_error(mapping, s.error_code, l);
 	}
 	else if (s.error_code == 716)
 	{
@@ -1133,23 +1161,24 @@ void upnp::on_upnp_map_response(error_code const& e
 		m.external_port = 40000 + (std::rand() % 10000);
 		m.action = mapping_t::action_add;
 		++m.failcount;
-		update_map(d, mapping);
+		update_map(d, mapping, l);
 		return;
 	}
 	else if (s.error_code != -1)
 	{
-		return_error(mapping, s.error_code);
+		return_error(mapping, s.error_code, l);
 	}
 
-#ifdef TORRENT_UPNP_LOGGING
-	m_log << time_now_string()
-		<< " <== map response: " << std::string(p.get_body().begin, p.get_body().end)
-		<< std::endl;
-#endif
+	char msg[500];
+	snprintf(msg, sizeof(msg), "map response: %s"
+		, std::string(p.get_body().begin, p.get_body().end).c_str());
+	log(msg, l);
 
 	if (s.error_code == -1)
 	{
-		m_callback(mapping, m.external_port, "");
+		l.unlock();
+		m_callback(mapping, m.external_port, error_code());
+		l.lock();
 		if (d.lease_duration > 0)
 		{
 			m.expires = time_now()
@@ -1170,10 +1199,10 @@ void upnp::on_upnp_map_response(error_code const& e
 		m.failcount = 0;
 	}
 
-	next(d, mapping);
+	next(d, mapping, l);
 }
 
-void upnp::return_error(int mapping, int code)
+void upnp::return_error(int mapping, int code, mutex_t::scoped_lock& l)
 {
 	int num_errors = sizeof(error_codes) / sizeof(error_codes[0]);
 	error_code_t* end = error_codes + num_errors;
@@ -1187,7 +1216,9 @@ void upnp::return_error(int mapping, int code)
 		error_string += ": ";
 		error_string += e->msg;
 	}
-	m_callback(mapping, 0, error_string);
+	l.unlock();
+	m_callback(mapping, 0, error_code(code, upnp_category));
+	l.lock();
 }
 
 void upnp::on_upnp_unmap_response(error_code const& e
@@ -1207,37 +1238,31 @@ void upnp::on_upnp_unmap_response(error_code const& e
 
 	if (e && e != asio::error::eof)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== error while deleting portmap: " << e.message() << std::endl;
-#endif
-	} else if (!p.header_finished())
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while deleting portmap: %s", e.message().c_str());
+		log(msg, l);
+	}
+	else if (!p.header_finished())
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== error while deleting portmap: incomplete http message" << std::endl;
-#endif
+		log("error while deleting portmap: incomplete http message", l);
 	}
 	else if (p.status_code() != 200)
 	{
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== error while deleting portmap: " << p.message() << std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "error while deleting portmap: %s", p.message().c_str());
+		log(msg, l);
 	}
 	else
 	{
-
-#ifdef TORRENT_UPNP_LOGGING
-		m_log << time_now_string()
-			<< " <== unmap response: " << std::string(p.get_body().begin, p.get_body().end)
-			<< std::endl;
-#endif
+		char msg[200];
+		snprintf(msg, sizeof(msg), "unmap response: %s"
+			, std::string(p.get_body().begin, p.get_body().end).c_str());
+		log(msg, l);
 	}
 
 	d.mapping[mapping].protocol = none;
 
-	next(d, mapping);
+	next(d, mapping, l);
 }
 
 void upnp::on_expire(error_code const& e)
@@ -1262,7 +1287,7 @@ void upnp::on_expire(error_code const& e)
 			if (d.mapping[m].expires < now)
 			{
 				d.mapping[m].expires = max_time();
-				update_map(d, m);
+				update_map(d, m, l);
 			}
 			else if (d.mapping[m].expires < next_expire)
 			{
@@ -1306,7 +1331,7 @@ void upnp::close()
 			j->action = mapping_t::action_delete;
 			m_mappings[j - d.mapping.begin()].protocol = none;
 		}
-		if (num_mappings() > 0) update_map(d, 0);
+		if (num_mappings() > 0) update_map(d, 0, l);
 	}
 }
 
