@@ -33,11 +33,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/pch.hpp"
 
 #include <vector>
-#include <iostream>
-#include <iomanip>
 #include <limits>
 #include <boost/bind.hpp>
-#include <sstream>
 #include <stdlib.h>
 
 #include "libtorrent/web_peer_connection.hpp"
@@ -70,6 +67,7 @@ namespace libtorrent
 		, m_original_url(url)
 		, m_first_request(true)
 		, m_range_pos(0)
+		, m_block_pos(0)
 	{
 		INVARIANT_CHECK;
 
@@ -78,7 +76,7 @@ namespace libtorrent
 		request_large_blocks(true);
 
 		// we only want left-over bandwidth
-		set_priority(0);
+		set_priority(1);
 		shared_ptr<torrent> tor = t.lock();
 		TORRENT_ASSERT(tor);
 		int blocks_per_piece = tor->torrent_file().piece_length() / tor->block_size();
@@ -100,10 +98,10 @@ namespace libtorrent
 #endif
 
 		std::string protocol;
-		char const* error;
-		boost::tie(protocol, m_auth, m_host, m_port, m_path, error)
-			= parse_url_components(url);
-		TORRENT_ASSERT(error == 0);
+		error_code ec;
+		boost::tie(protocol, m_auth, m_host, m_port, m_path)
+			= parse_url_components(url, ec);
+		TORRENT_ASSERT(!ec);
 		
 		if (!m_auth.empty())
 			m_auth = base64encode(m_auth);
@@ -134,23 +132,12 @@ namespace libtorrent
 		piece_block_progress ret;
 
 		ret.piece_index = m_requests.front().piece;
-		if (!m_piece.empty())
-		{
-			ret.bytes_downloaded = int(m_piece.size());
-		}
+		ret.bytes_downloaded = m_block_pos;
+		if (m_block_pos)
+			ret.block_index = (m_requests.front().start + m_block_pos - 1) / t->block_size();
 		else
-		{
-			if (!m_parser.header_finished())
-			{
-				ret.bytes_downloaded = 0;
-			}
-			else
-			{
-				int receive_buffer_size = receive_buffer().left() - m_parser.body_start();
-				ret.bytes_downloaded = receive_buffer_size % t->block_size();
-			}
-		}
-		ret.block_index = (m_requests.front().start + ret.bytes_downloaded) / t->block_size();
+			ret.block_index = (m_requests.front().start + m_block_pos) / t->block_size();
+
 		ret.full_block_bytes = t->block_size();
 		const int last_piece = t->torrent_file().num_pieces() - 1;
 		if (ret.piece_index == last_piece && ret.block_index
@@ -347,6 +334,7 @@ namespace libtorrent
 
 		if (error)
 		{
+			m_statistics.received_bytes(0, bytes_transferred);
 #ifdef TORRENT_VERBOSE_LOGGING
 			(*m_logger) << "*** web_peer_connection error: "
 				<< error.message() << "\n";
@@ -357,8 +345,6 @@ namespace libtorrent
 		boost::shared_ptr<torrent> t = associated_torrent().lock();
 		TORRENT_ASSERT(t);
 
-		incoming_piece_fragment();
-
 		for (;;)
 		{
 			buffer::const_interval recv_buffer = receive_buffer();
@@ -375,10 +361,11 @@ namespace libtorrent
 
 				if (error)
 				{
+					m_statistics.received_bytes(0, bytes_transferred);
 #ifdef TORRENT_VERBOSE_LOGGING
 					(*m_logger) << "*** " << std::string(recv_buffer.begin, recv_buffer.end) << "\n";
 #endif
-					disconnect("failed to parse HTTP response", 2);
+					disconnect(errors::http_parse_error, 2);
 					return;
 				}
 
@@ -403,18 +390,18 @@ namespace libtorrent
 					if (m_parser.status_code() == 503)
 					{
 						// temporarily unavailable, retry later
-						t->retry_url_seed(m_original_url);
+						t->retry_web_seed(m_original_url, web_seed_entry::url_seed);
 					}
-					t->remove_url_seed(m_original_url);
+					t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
 					std::string error_msg = to_string(m_parser.status_code()).elems
 						+ (" " + m_parser.message());
 					if (m_ses.m_alerts.should_post<url_seed_alert>())
 					{
-						session_impl::mutex_t::scoped_lock l(m_ses.m_mutex);
 						m_ses.m_alerts.post_alert(url_seed_alert(t->get_handle(), url()
 							, error_msg));
 					}
-					disconnect(error_msg.c_str(), 1);
+					m_statistics.received_bytes(0, bytes_transferred);
+					disconnect(errors::http_error, 1);
 					return;
 				}
 				if (!m_parser.header_finished())
@@ -444,12 +431,13 @@ namespace libtorrent
 					// this means we got a redirection request
 					// look for the location header
 					std::string location = m_parser.header("location");
+					m_statistics.received_bytes(0, bytes_transferred);
 
 					if (location.empty())
 					{
 						// we should not try this server again.
-						t->remove_url_seed(m_original_url);
-						disconnect("got HTTP redirection status without location header", 2);
+						t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
+						disconnect(errors::missing_location, 2);
 						return;
 					}
 					
@@ -460,6 +448,7 @@ namespace libtorrent
 					// add the redirected url and remove the current one
 					if (!single_file_request)
 					{
+						m_statistics.received_bytes(0, bytes_transferred);
 						TORRENT_ASSERT(!m_file_requests.empty());
 						int file_index = m_file_requests.front();
 
@@ -469,20 +458,15 @@ namespace libtorrent
 						size_t i = location.rfind(path);
 						if (i == std::string::npos)
 						{
-							t->remove_url_seed(m_original_url);
-							std::stringstream msg;
-							msg << "got invalid HTTP redirection location (\"" << location << "\") "
-								"expected it to end with: " << path;
-							disconnect(msg.str().c_str(), 2);
+							t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
+							disconnect(errors::invalid_redirection, 2);
 							return;
 						}
 						location.resize(i);
 					}
-					t->add_url_seed(location);
-					t->remove_url_seed(m_original_url);
-					std::stringstream msg;
-					msg << "redirecting to \"" << location << "\"";
-					disconnect(msg.str().c_str());
+					t->add_web_seed(location, web_seed_entry::url_seed);
+					t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
+					disconnect(errors::redirecting, 2);
 					return;
 				}
 
@@ -510,17 +494,13 @@ namespace libtorrent
 			size_type range_end;
 			if (m_parser.status_code() == 206)
 			{
-				std::stringstream range_str(m_parser.header("content-range"));
-				char dummy;
-				std::string bytes;
-				range_str >> bytes >> range_start >> dummy >> range_end;
-				if (!range_str)
+				boost::tie(range_start, range_end) = m_parser.content_range();
+				if (range_start < 0 || range_end < range_start)
 				{
+					m_statistics.received_bytes(0, bytes_transferred);
 					// we should not try this server again.
-					t->remove_url_seed(m_original_url);
-					std::stringstream msg;
-					msg << "invalid range in HTTP response: " << range_str.str();
-					disconnect(msg.str().c_str(), 2);
+					t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
+					disconnect(errors::invalid_range);
 					return;
 				}
 				// the http range is inclusive
@@ -532,37 +512,46 @@ namespace libtorrent
 				range_end = m_parser.content_length();
 				if (range_end == -1)
 				{
+					m_statistics.received_bytes(0, bytes_transferred);
 					// we should not try this server again.
-					t->remove_url_seed(m_original_url);
-					disconnect("no content-length in HTTP response", 2);
+					t->remove_web_seed(m_original_url, web_seed_entry::url_seed);
+					disconnect(errors::no_content_length, 2);
 					return;
 				}
 			}
 
+			if (m_requests.empty() || m_file_requests.empty())
+			{
+				m_statistics.received_bytes(0, bytes_transferred);
+				disconnect(errors::http_error, 2);
+				return;
+			}
+
 			int left_in_response = range_end - range_start - m_range_pos;
 			int payload_transferred = (std::min)(left_in_response, int(bytes_transferred));
+
+			torrent_info const& info = t->torrent_file();
+
+			peer_request front_request = m_requests.front();
+
+			TORRENT_ASSERT(m_block_pos >= 0);
+			if (m_block_pos + payload_transferred > front_request.length)
+				payload_transferred = front_request.length - m_block_pos;
+
 			m_statistics.received_bytes(payload_transferred, 0);
+			incoming_piece_fragment(payload_transferred);
 			bytes_transferred -= payload_transferred;
-			m_range_pos += payload_transferred;;
+			m_range_pos += payload_transferred;
+			m_block_pos += payload_transferred;
 			if (m_range_pos > range_end - range_start) m_range_pos = range_end - range_start;
 
 //			std::cerr << "REQUESTS: m_requests: " << m_requests.size()
 //				<< " file_requests: " << m_file_requests.size() << std::endl;
 
-			torrent_info const& info = t->torrent_file();
-
-			if (m_requests.empty() || m_file_requests.empty())
-			{
-				disconnect("unexpected HTTP response", 2);
-				return;
-			}
-
 			int file_index = m_file_requests.front();
 			peer_request in_range = info.orig_files().map_file(file_index, range_start
 				, int(range_end - range_start));
 
-			peer_request front_request = m_requests.front();
-
 			size_type rs = size_type(in_range.piece) * info.piece_length() + in_range.start;
 			size_type re = rs + in_range.length;
 			size_type fs = size_type(front_request.piece) * info.piece_length() + front_request.start;
@@ -584,9 +573,10 @@ namespace libtorrent
 
 			if (!range_overlaps_request)
 			{
+				m_statistics.received_bytes(0, bytes_transferred);
 				// this means the end of the incoming request ends _before_ the
 				// first expected byte (fs + m_piece.size())
-				disconnect("invalid range in HTTP response", 2);
+				disconnect(errors::invalid_range, 2);
 				return;
 			}
 
@@ -621,9 +611,11 @@ namespace libtorrent
 					// will be disconnected, including this web seed. We need to
 					// check for the disconnect condition after the call.
 
-					m_requests.pop_front();
 					incoming_piece(front_request, &m_piece[0]);
+					m_requests.pop_front();
 					if (associated_torrent().expired()) return;
+					TORRENT_ASSERT(m_block_pos == front_request.length);
+					m_block_pos = 0;
 					cut_receive_buffer(m_body_start, t->block_size() + 1024);
 					m_body_start = 0;
 					recv_buffer = receive_buffer();
@@ -639,11 +631,13 @@ namespace libtorrent
 				&& recv_buffer.left() >= m_requests.front().length)
 			{
 				peer_request r = m_requests.front();
-				m_requests.pop_front();
 				TORRENT_ASSERT(recv_buffer.left() >= r.length);
 
 				incoming_piece(r, recv_buffer.begin);
+				m_requests.pop_front();
 				if (associated_torrent().expired()) return;
+				TORRENT_ASSERT(m_block_pos == front_request.length);
+				m_block_pos = 0;
 				m_received_body += r.length;
 				TORRENT_ASSERT(receive_buffer().begin + m_body_start == recv_buffer.begin);
 				TORRENT_ASSERT(m_received_body <= range_end - range_start);
@@ -714,7 +708,6 @@ namespace libtorrent
 		return m_server_string.empty();
 	}
 
-	// throws exception when the client should be disconnected
 	void web_peer_connection::on_sent(error_code const& error
 		, std::size_t bytes_transferred)
 	{
